@@ -8,8 +8,10 @@
 {-# LANGUAGE RankNTypes            #-}
 {-# LANGUAGE GADTs                 #-}
 {-# LANGUAGE StandaloneDeriving    #-}
+{-# LANGUAGE TemplateHaskell       #-}
 module Network.HTTP.Download.Verified
   ( verifiedDownload
+  , recoveringHttp
   , DownloadRequest(..)
   , drRetryPolicyDefault
   , HashCheck(..)
@@ -18,38 +20,41 @@ module Network.HTTP.Download.Verified
   , VerifiedDownloadException(..)
   ) where
 
-import qualified Data.List as List
-import qualified Data.ByteString as ByteString
-import qualified Data.ByteString.Base64 as B64
-import qualified Data.Conduit.Binary as CB
-import qualified Data.Conduit.List as CL
-import qualified Data.Text as Text
-import qualified Data.Text.Encoding as Text
+import qualified    Data.List as List
+import qualified    Data.ByteString as ByteString
+import qualified    Data.ByteString.Base64 as B64
+import qualified    Data.Conduit.Binary as CB
+import qualified    Data.Conduit.List as CL
+import qualified    Data.Text as Text
+import qualified    Data.Text.Encoding as Text
 
-import Control.Monad
-import Control.Monad.Catch
-import Control.Monad.IO.Class
-import Control.Monad.Reader
-import Control.Retry (recovering,limitRetries,RetryPolicy,constantDelay)
-import Control.Applicative
+import              Control.Applicative
+import              Control.Monad
+import              Control.Monad.Catch
+import              Control.Monad.IO.Class
+import              Control.Monad.Logger (logDebug, MonadLogger)
+import              Control.Monad.Reader
+import              Control.Retry (recovering,limitRetries,RetryPolicy,constantDelay)
 import "cryptohash" Crypto.Hash
-import Crypto.Hash.Conduit (sinkHash)
-import Data.ByteString (ByteString)
-import Data.ByteString.Char8 (readInteger)
-import Data.Conduit
-import Data.Conduit.Binary (sourceHandle, sinkHandle)
-import Data.Foldable (traverse_,for_)
-import Data.Monoid
-import Data.String
-import Data.Typeable (Typeable)
-import GHC.IO.Exception (IOException(..),IOErrorType(..))
-import Network.HTTP.Client.Conduit
-import Network.HTTP.Types.Header (hContentLength, hContentMD5)
-import Path
-import Prelude -- Fix AMP warning
-import System.FilePath((<.>))
-import System.Directory
-import System.IO
+import              Crypto.Hash.Conduit (sinkHash)
+import              Data.ByteString (ByteString)
+import              Data.ByteString.Char8 (readInteger)
+import              Data.Conduit
+import              Data.Conduit.Binary (sourceHandle, sinkHandle)
+import              Data.Foldable (traverse_,for_)
+import              Data.Monoid
+import              Data.String
+import              Data.Text.Encoding (decodeUtf8With)
+import              Data.Text.Encoding.Error (lenientDecode)
+import              Data.Typeable (Typeable)
+import              GHC.IO.Exception (IOException(..),IOErrorType(..))
+import              Network.HTTP.Client.Conduit
+import              Network.HTTP.Types.Header (hContentLength, hContentMD5)
+import              Path
+import              Prelude -- Fix AMP warning
+import              System.Directory
+import              System.FilePath ((<.>))
+import              System.IO
 
 -- | A request together with some checks to perform.
 data DownloadRequest = DownloadRequest
@@ -179,38 +184,15 @@ sinkHashUsing _ = sinkHash
 hashChecksToZipSink :: MonadThrow m => Request -> [HashCheck] -> ZipSink ByteString m ()
 hashChecksToZipSink req = traverse_ (ZipSink . sinkCheckHash req)
 
--- | Copied and extended version of Network.HTTP.Download.download.
---
--- Has the following additional features:
--- * Verifies that response content-length header (if present)
---     matches expected length
--- * Limits the download to (close to) the expected # of bytes
--- * Verifies that the expected # bytes were downloaded (not too few)
--- * Verifies md5 if response includes content-md5 header
--- * Verifies the expected hashes
---
--- Throws VerifiedDownloadException.
--- Throws IOExceptions related to file system operations.
--- Throws HttpException.
-verifiedDownload :: (MonadReader env m, HasHttpManager env, MonadIO m)
-         => DownloadRequest
-         -> Path Abs File -- ^ destination
-         -> (Maybe Integer -> Sink ByteString (ReaderT env IO) ()) -- ^ custom hook to observe progress
-         -> m Bool -- ^ Whether a download was performed
-verifiedDownload DownloadRequest{..} destpath progressSink = do
-    let req = drRequest
-    env <- ask
-    liftIO $ whenM' getShouldDownload $ do
-        createDirectoryIfMissing True dir
-        withBinaryFile fptmp WriteMode $ \h ->
+-- 'Control.Retry.recovering' customized for HTTP failures
+recoveringHttp :: (MonadMask m, MonadIO m)
+               => RetryPolicy -> m a -> m a
+recoveringHttp retryPolicy =
 #if MIN_VERSION_retry(0,7,0)
-            recovering drRetryPolicy handlers $ const $
+    recovering retryPolicy handlers . const
 #else
-            recovering drRetryPolicy handlers $
+    recovering retryPolicy handlers
 #endif
-                flip runReaderT env $
-                    withResponse req (go h)
-        renameFile fptmp fp
   where
     handlers = [const $ Handler alwaysRetryHttp,const $ Handler retrySomeIO]
 
@@ -224,6 +206,37 @@ verifiedDownload DownloadRequest{..} destpath progressSink = do
                                -- conservatively exclude all others
                                _ -> False
 
+-- | Copied and extended version of Network.HTTP.Download.download.
+--
+-- Has the following additional features:
+-- * Verifies that response content-length header (if present)
+--     matches expected length
+-- * Limits the download to (close to) the expected # of bytes
+-- * Verifies that the expected # bytes were downloaded (not too few)
+-- * Verifies md5 if response includes content-md5 header
+-- * Verifies the expected hashes
+--
+-- Throws VerifiedDownloadException.
+-- Throws IOExceptions related to file system operations.
+-- Throws HttpException.
+verifiedDownload :: (MonadReader env m, HasHttpManager env, MonadIO m, MonadLogger m)
+         => DownloadRequest
+         -> Path Abs File -- ^ destination
+         -> (Maybe Integer -> Sink ByteString (ReaderT env IO) ()) -- ^ custom hook to observe progress
+         -> m Bool -- ^ Whether a download was performed
+verifiedDownload DownloadRequest{..} destpath progressSink = do
+    let req = drRequest
+    env <- ask
+    whenM' (liftIO getShouldDownload) $ do
+        $logDebug $ "Downloading " <> decodeUtf8With lenientDecode (path req)
+        liftIO $ do
+            createDirectoryIfMissing True dir
+            withBinaryFile fptmp WriteMode $ \h ->
+                recoveringHttp drRetryPolicy $
+                    flip runReaderT env $
+                        withResponse req (go h)
+            renameFile fptmp fp
+  where
     whenM' mp m = do
         p <- mp
         if p then m >> return True else return False
@@ -243,8 +256,8 @@ verifiedDownload DownloadRequest{..} destpath progressSink = do
     -- precondition: file exists
     -- TODO: add logging
     fileMatchesExpectations =
-        (checkExpectations >> return True)
-          `catch` \(_ :: VerifyFileException) -> return False
+        ((checkExpectations >> return True)
+          `catch` \(_ :: VerifyFileException) -> return False)
           `catch` \(_ :: VerifiedDownloadException) -> return False
 
     checkExpectations = bracket (openFile fp ReadMode) hClose $ \h -> do
