@@ -5,16 +5,20 @@
 module Path.Find
   (findFileUp
   ,findDirUp
-  ,findFiles)
+  ,findFiles
+  ,findInParents)
   where
 
+import Control.Exception (evaluate)
+import Control.DeepSeq (force)
 import Control.Monad
 import Control.Monad.Catch
 import Control.Monad.IO.Class
 import System.IO.Error (isPermissionError)
 import Data.List
 import Path
-import Path.IO
+import Path.IO hiding (findFiles)
+import System.PosixCompat.Files (getSymbolicLinkStatus, isSymbolicLink)
 
 -- | Find the location of a file matching the given predicate.
 findFileUp :: (MonadIO m,MonadThrow m)
@@ -41,7 +45,7 @@ findPathUp :: (MonadIO m,MonadThrow m)
            -> Maybe (Path Abs Dir)             -- ^ Do not ascend above this directory.
            -> m (Maybe (Path Abs t))           -- ^ Absolute path.
 findPathUp pathType dir p upperBound =
-  do entries <- listDirectory dir
+  do entries <- listDir dir
      case find p (pathType entries) of
        Just path -> return (Just path)
        Nothing | Just dir == upperBound -> return Nothing
@@ -49,6 +53,12 @@ findPathUp pathType dir p upperBound =
                | otherwise -> findPathUp pathType (parent dir) p upperBound
 
 -- | Find files matching predicate below a root directory.
+--
+-- NOTE: this skips symbolic directory links, to avoid loops. This may
+-- not make sense for all uses of file finding.
+--
+-- TODO: write one of these that traverses symbolic links but
+-- efficiently ignores loops.
 findFiles :: Path Abs Dir            -- ^ Root directory to begin with.
           -> (Path Abs File -> Bool) -- ^ Predicate to match files.
           -> (Path Abs Dir -> Bool)  -- ^ Predicate for which directories to traverse.
@@ -57,12 +67,30 @@ findFiles dir p traversep =
   do (dirs,files) <- catchJust (\ e -> if isPermissionError e
                                          then Just ()
                                          else Nothing)
-                               (listDirectory dir)
+                               (listDir dir)
                                (\ _ -> return ([], []))
+     filteredFiles <- evaluate $ force (filter p files)
+     filteredDirs <- filterM (fmap not . isSymLink) dirs
      subResults <-
-       forM dirs
+       forM filteredDirs
             (\entry ->
                if traversep entry
                   then findFiles entry p traversep
                   else return [])
-     return (concat (filter p files : subResults))
+     return (concat (filteredFiles : subResults))
+
+isSymLink :: Path Abs t -> IO Bool
+isSymLink = fmap isSymbolicLink . getSymbolicLinkStatus . toFilePath
+
+-- | @findInParents f path@ applies @f@ to @path@ and its 'parent's until
+-- it finds a 'Just' or reaches the root directory.
+findInParents :: MonadIO m => (Path Abs Dir -> m (Maybe a)) -> Path Abs Dir -> m (Maybe a)
+findInParents f path = do
+    mres <- f path
+    case mres of
+        Just res -> return (Just res)
+        Nothing -> do
+            let next = parent path
+            if next == path
+                then return Nothing
+                else findInParents f next
