@@ -15,19 +15,22 @@ module Stack.Build
   (build
   ,withLoadPackage
   ,mkBaseConfigOpts
-  ,queryBuildInfo)
+  ,queryBuildInfo
+  ,splitObjsWarning
+  ,CabalVersionException(..))
   where
 
+import           Control.Exception (Exception)
 import           Control.Monad
-import           Control.Monad.Catch (MonadCatch, MonadMask)
+import           Control.Monad.Catch (MonadMask, MonadMask)
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
 import           Control.Monad.Reader (MonadReader, asks)
 import           Control.Monad.Trans.Resource
+import           Control.Monad.Trans.Unlift (MonadBaseUnlift)
 import           Data.Aeson (Value (Object, Array), (.=), object)
 import           Data.Function
 import qualified Data.HashMap.Strict as HM
-import           Data.IORef.RunOnce (runOnce)
 import           Data.List ((\\))
 import           Data.List.Extra (groupSort)
 import           Data.List.NonEmpty (NonEmpty(..))
@@ -42,6 +45,7 @@ import qualified Data.Text as T
 import           Data.Text.Encoding (decodeUtf8)
 import qualified Data.Text.IO as TIO
 import           Data.Text.Read (decimal)
+import           Data.Typeable (Typeable)
 import qualified Data.Vector as V
 import qualified Data.Yaml as Yaml
 import           Network.HTTP.Client.Conduit (HasHttpManager)
@@ -61,11 +65,11 @@ import           Stack.Types.Internal
 import           System.FileLock (FileLock, unlockFile)
 
 #ifdef WINDOWS
-import System.Win32.Console (setConsoleCP, setConsoleOutputCP, getConsoleCP, getConsoleOutputCP)
+import           System.Win32.Console (setConsoleCP, setConsoleOutputCP, getConsoleCP, getConsoleOutputCP)
 import qualified Control.Monad.Catch as Catch
 #endif
 
-type M env m = (MonadIO m,MonadReader env m,HasHttpManager env,HasBuildConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,MonadMask m,HasLogLevel env,HasEnvConfig env,HasTerminal env)
+type M env m = (MonadIO m,MonadReader env m,HasHttpManager env,HasBuildConfig env,MonadLogger m,MonadBaseUnlift IO m,MonadMask m,HasLogLevel env,HasEnvConfig env,HasTerminal env)
 
 -- | Build.
 --
@@ -75,12 +79,14 @@ type M env m = (MonadIO m,MonadReader env m,HasHttpManager env,HasBuildConfig en
 build :: M env m
       => (Set (Path Abs File) -> IO ()) -- ^ callback after discovering all local files
       -> Maybe FileLock
-      -> BuildOpts
+      -> BuildOptsCLI
       -> m ()
-build setLocalFiles mbuildLk bopts = fixCodePage' $ do
+build setLocalFiles mbuildLk boptsCli = fixCodePage $ do
+    bopts <- asks (configBuild . getConfig)
+    let profiling = boptsLibProfile bopts || boptsExeProfile bopts
     menv <- getMinimalEnvOverride
 
-    (_, mbp, locals, extraToBuild, sourceMap) <- loadSourceMap NeedTargets bopts
+    (targets, mbp, locals, extraToBuild, sourceMap) <- loadSourceMap NeedTargets boptsCli
 
     -- Set local files, necessary for file watching
     stackYaml <- asks $ bcStackYaml . getBuildConfig
@@ -96,7 +102,7 @@ build setLocalFiles mbuildLk bopts = fixCodePage' $ do
                          , getInstalledHaddock   = shouldHaddockDeps bopts }
                      sourceMap
 
-    baseConfigOpts <- mkBaseConfigOpts bopts
+    baseConfigOpts <- mkBaseConfigOpts boptsCli
     plan <- withLoadPackage menv $ \loadPackage ->
         constructPlan mbp baseConfigOpts locals extraToBuild localDumpPkgs loadPackage sourceMap installedMap
 
@@ -110,21 +116,22 @@ build setLocalFiles mbuildLk bopts = fixCodePage' $ do
                            liftIO $ unlockFile lk
       _ -> return ()
 
+    checkCabalVersion
+    warnAboutSplitObjs bopts
     warnIfExecutablesWithSameNameCouldBeOverwritten locals plan
 
     when (boptsPreFetch bopts) $
         preFetch plan
 
-    if boptsDryrun bopts
+    if boptsCLIDryrun boptsCli
         then printPlan plan
-        else executePlan menv bopts baseConfigOpts locals
+        else executePlan menv boptsCli baseConfigOpts locals
                          globalDumpPkgs
                          snapshotDumpPkgs
                          localDumpPkgs
                          installedMap
+                         targets
                          plan
-  where
-    profiling = boptsLibProfile bopts || boptsExeProfile bopts
 
 -- | If all the tasks are local, they don't mutate anything outside of our local directory.
 allLocal :: Plan -> Bool
@@ -134,6 +141,23 @@ allLocal =
     Map.elems .
     planTasks
 
+checkCabalVersion :: M env m => m ()
+checkCabalVersion = do
+    allowNewer <- asks (configAllowNewer . getConfig)
+    cabalVer <- asks (envConfigCabalVersion . getEnvConfig)
+    -- https://github.com/haskell/cabal/issues/2023
+    when (allowNewer && cabalVer < $(mkVersion "1.22")) $ throwM $
+        CabalVersionException $
+            "Error: --allow-newer requires at least Cabal version 1.22, but version " ++
+            versionString cabalVer ++
+            " was found."
+
+data CabalVersionException = CabalVersionException { unCabalVersionException :: String }
+    deriving (Typeable)
+
+instance Show CabalVersionException where show = unCabalVersionException
+instance Exception CabalVersionException
+
 -- | See https://github.com/commercialhaskell/stack/issues/1198.
 warnIfExecutablesWithSameNameCouldBeOverwritten
     :: MonadLogger m => [LocalPackage] -> Plan -> m ()
@@ -207,10 +231,24 @@ warnIfExecutablesWithSameNameCouldBeOverwritten locals plan =
     collect :: Ord k => [(k,v)] -> Map k (NonEmpty v)
     collect = Map.map NE.fromList . Map.fromDistinctAscList . groupSort
 
+warnAboutSplitObjs :: MonadLogger m => BuildOpts -> m ()
+warnAboutSplitObjs bopts | boptsSplitObjs bopts = do
+    $logWarn $ "Building with --split-objs is enabled. " <> T.pack splitObjsWarning
+warnAboutSplitObjs _ = return ()
+
+splitObjsWarning :: String
+splitObjsWarning = unwords
+     [ "Note that this feature is EXPERIMENTAL, and its behavior may be changed and improved."
+     , "You will need to clean your workdirs before use. If you want to compile all dependencies"
+     , "with split-objs, you will need to delete the snapshot (and all snapshots that could"
+     , "reference that snapshot)."
+     ]
+
 -- | Get the @BaseConfigOpts@ necessary for constructing configure options
 mkBaseConfigOpts :: (MonadIO m, MonadReader env m, HasEnvConfig env, MonadThrow m)
-                 => BuildOpts -> m BaseConfigOpts
-mkBaseConfigOpts bopts = do
+                 => BuildOptsCLI -> m BaseConfigOpts
+mkBaseConfigOpts boptsCli = do
+    bopts <- asks (configBuild . getConfig)
     snapDBPath <- packageDatabaseDeps
     localDBPath <- packageDatabaseLocal
     snapInstallRoot <- installationRootDeps
@@ -222,6 +260,7 @@ mkBaseConfigOpts bopts = do
         , bcoSnapInstallRoot = snapInstallRoot
         , bcoLocalInstallRoot = localInstallRoot
         , bcoBuildOpts = bopts
+        , bcoBuildOptsCLI = boptsCli
         , bcoExtraDBs = packageExtraDBs
         }
 
@@ -229,67 +268,74 @@ mkBaseConfigOpts bopts = do
 withLoadPackage :: ( MonadIO m
                    , HasHttpManager env
                    , MonadReader env m
-                   , MonadBaseControl IO m
-                   , MonadCatch m
+                   , MonadBaseUnlift IO m
+                   , MonadMask m
                    , MonadLogger m
                    , HasEnvConfig env)
                 => EnvOverride
-                -> ((PackageName -> Version -> Map FlagName Bool -> IO Package) -> m a)
+                -> ((PackageName -> Version -> Map FlagName Bool -> [Text] -> IO Package) -> m a)
                 -> m a
 withLoadPackage menv inner = do
     econfig <- asks getEnvConfig
-    withCabalLoader' <- runOnce $ withCabalLoader menv $ \cabalLoader ->
-        inner $ \name version flags -> do
-            bs <- cabalLoader $ PackageIdentifier name version -- TODO automatically update index the first time this fails
+    withCabalLoader menv $ \cabalLoader ->
+        inner $ \name version flags ghcOptions -> do
+            bs <- cabalLoader $ PackageIdentifier name version
 
             -- Intentionally ignore warnings, as it's not really
             -- appropriate to print a bunch of warnings out while
             -- resolving the package index.
-            (_warnings,pkg) <- readPackageBS (depPackageConfig econfig flags) bs
+            (_warnings,pkg) <- readPackageBS (depPackageConfig econfig flags ghcOptions) bs
             return pkg
-    withCabalLoader'
   where
     -- | Package config to be used for dependencies
-    depPackageConfig :: EnvConfig -> Map FlagName Bool -> PackageConfig
-    depPackageConfig econfig flags = PackageConfig
+    depPackageConfig :: EnvConfig -> Map FlagName Bool -> [Text] -> PackageConfig
+    depPackageConfig econfig flags ghcOptions = PackageConfig
         { packageConfigEnableTests = False
         , packageConfigEnableBenchmarks = False
         , packageConfigFlags = flags
+        , packageConfigGhcOptions = ghcOptions
         , packageConfigCompilerVersion = envConfigCompilerVersion econfig
         , packageConfigPlatform = configPlatform (getConfig econfig)
         }
 
 -- | Set the code page for this process as necessary. Only applies to Windows.
 -- See: https://github.com/commercialhaskell/stack/issues/738
-fixCodePage :: (MonadIO m, MonadMask m, MonadLogger m) => m a -> m a
 #ifdef WINDOWS
+fixCodePage :: M env m => m a -> m a
 fixCodePage inner = do
-    origCPI <- liftIO getConsoleCP
-    origCPO <- liftIO getConsoleOutputCP
-
-    let setInput = origCPI /= expected
-        setOutput = origCPO /= expected
-        fixInput
-            | setInput = Catch.bracket_
-                (liftIO $ do
-                    setConsoleCP expected)
-                (liftIO $ setConsoleCP origCPI)
-            | otherwise = id
-        fixOutput
-            | setInput = Catch.bracket_
-                (liftIO $ do
-                    setConsoleOutputCP expected)
-                (liftIO $ setConsoleOutputCP origCPO)
-            | otherwise = id
-
-    case (setInput, setOutput) of
-        (False, False) -> return ()
-        (True, True) -> warn ""
-        (True, False) -> warn " input"
-        (False, True) -> warn " output"
-
-    fixInput $ fixOutput inner
+    mcp <- asks $ configModifyCodePage . getConfig
+    ec <- asks getEnvConfig
+    if mcp && getGhcVersion (envConfigCompilerVersion ec) < $(mkVersion "7.10.3")
+        then fixCodePage'
+        -- GHC >=7.10.3 doesn't need this code page hack.
+        else inner
   where
+    fixCodePage' = do
+        origCPI <- liftIO getConsoleCP
+        origCPO <- liftIO getConsoleOutputCP
+
+        let setInput = origCPI /= expected
+            setOutput = origCPO /= expected
+            fixInput
+                | setInput = Catch.bracket_
+                    (liftIO $ do
+                        setConsoleCP expected)
+                    (liftIO $ setConsoleCP origCPI)
+                | otherwise = id
+            fixOutput
+                | setInput = Catch.bracket_
+                    (liftIO $ do
+                        setConsoleOutputCP expected)
+                    (liftIO $ setConsoleOutputCP origCPO)
+                | otherwise = id
+
+        case (setInput, setOutput) of
+            (False, False) -> return ()
+            (True, True) -> warn ""
+            (True, False) -> warn " input"
+            (False, True) -> warn " output"
+
+        fixInput $ fixOutput inner
     expected = 65001 -- UTF-8
     warn typ = $logInfo $ T.concat
         [ "Setting"
@@ -297,18 +343,10 @@ fixCodePage inner = do
         , " codepage to UTF-8 (65001) to ensure correct output from GHC"
         ]
 #else
+fixCodePage :: a -> a
 fixCodePage = id
 #endif
 
-fixCodePage' :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env m, HasConfig env)
-             => m a
-             -> m a
-fixCodePage' inner = do
-    mcp <- asks $ configModifyCodePage . getConfig
-    if mcp
-        then fixCodePage inner
-        else inner
-
 -- | Query information about the build and print the result to stdout in YAML format.
 queryBuildInfo :: M env m
                => [Text] -- ^ selectors
@@ -339,7 +377,7 @@ queryBuildInfo selectors0 =
 -- | Get the raw build information object
 rawBuildInfo :: M env m => m Value
 rawBuildInfo = do
-    (_, _mbp, locals, _extraToBuild, _sourceMap) <- loadSourceMap NeedTargets defaultBuildOpts
+    (_, _mbp, locals, _extraToBuild, _sourceMap) <- loadSourceMap NeedTargets defaultBuildOptsCLI
     return $ object
         [ "locals" .= Object (HM.fromList $ map localToPair locals)
         ]
