@@ -3,6 +3,7 @@
 {-# LANGUAGE TemplateHaskell       #-}
 {-# LANGUAGE TupleSections         #-}
 {-# LANGUAGE ConstraintKinds       #-}
+{-# LANGUAGE OverloadedStrings     #-}
 -- | Cache information about previous builds
 module Stack.Build.Cache
     ( tryGetBuildCache
@@ -26,9 +27,9 @@ module Stack.Build.Cache
     ) where
 
 import           Control.Exception.Enclosed (handleIO)
-import           Control.Monad.Catch (MonadThrow)
+import           Control.Monad.Catch (MonadThrow, MonadCatch)
 import           Control.Monad.IO.Class
-import           Control.Monad.Logger (MonadLogger)
+import           Control.Monad.Logger (MonadLogger, logDebug)
 import           Control.Monad.Reader
 import qualified Crypto.Hash.SHA256 as SHA256
 import qualified Data.Binary as Binary (encode)
@@ -37,6 +38,7 @@ import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Base16 as B16
 import           Data.Map (Map)
 import           Data.Maybe (fromMaybe, mapMaybe)
+import           Data.Monoid ((<>))
 import           Data.Set (Set)
 import qualified Data.Set as Set
 import           Data.Text (Text)
@@ -59,7 +61,7 @@ getInstalledExes :: (MonadReader env m, HasEnvConfig env, MonadIO m, MonadThrow
                  => InstallLocation -> m [PackageIdentifier]
 getInstalledExes loc = do
     dir <- exeInstalledDir loc
-    (_, files) <- liftIO $ handleIO (const $ return ([], [])) $ listDirectory dir
+    (_, files) <- liftIO $ handleIO (const $ return ([], [])) $ listDir dir
     return $ mapMaybe (parsePackageIdentifierFromString . toFilePath . filename) files
 
 -- | Mark the given executable as installed
@@ -67,7 +69,7 @@ markExeInstalled :: (MonadReader env m, HasEnvConfig env, MonadIO m, MonadThrow
                  => InstallLocation -> PackageIdentifier -> m ()
 markExeInstalled loc ident = do
     dir <- exeInstalledDir loc
-    createTree dir
+    ensureDir dir
     ident' <- parseRelFile $ packageIdentifierString ident
     let fp = toFilePath $ dir </> ident'
     -- TODO consideration for the future: list all of the executables
@@ -76,12 +78,12 @@ markExeInstalled loc ident = do
     liftIO $ writeFile fp "Installed"
 
 -- | Mark the given executable as not installed
-markExeNotInstalled :: (MonadReader env m, HasEnvConfig env, MonadIO m, MonadThrow m)
+markExeNotInstalled :: (MonadReader env m, HasEnvConfig env, MonadIO m, MonadCatch m)
                     => InstallLocation -> PackageIdentifier -> m ()
 markExeNotInstalled loc ident = do
     dir <- exeInstalledDir loc
     ident' <- parseRelFile $ packageIdentifierString ident
-    removeFileIfExists (dir </> ident')
+    ignoringAbsence (removeFile $ dir </> ident')
 
 -- | Stored on disk to know whether the flags have changed or any
 -- files have changed.
@@ -96,17 +98,17 @@ instance HasSemanticVersion BuildCache
 instance NFData BuildCache
 
 -- | Try to read the dirtiness cache for the given package directory.
-tryGetBuildCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
+tryGetBuildCache :: (MonadIO m, MonadReader env m, MonadThrow m, HasEnvConfig env)
                  => Path Abs Dir -> m (Maybe (Map FilePath FileCacheInfo))
 tryGetBuildCache = liftM (fmap buildCacheTimes) . tryGetCache buildCacheFile
 
 -- | Try to read the dirtiness cache for the given package directory.
-tryGetConfigCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
+tryGetConfigCache :: (MonadIO m, MonadReader env m, MonadThrow m, HasEnvConfig env)
                   => Path Abs Dir -> m (Maybe ConfigCache)
 tryGetConfigCache = tryGetCache configCacheFile
 
 -- | Try to read the mod time of the cabal file from the last build
-tryGetCabalMod :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
+tryGetCabalMod :: (MonadIO m, MonadReader env m, MonadThrow m, HasEnvConfig env)
                => Path Abs Dir -> m (Maybe ModTime)
 tryGetCabalMod = tryGetCache configCabalMod
 
@@ -120,7 +122,7 @@ tryGetCache get' dir = do
     decodeFileOrFailDeep fp
 
 -- | Write the dirtiness cache for this package's files.
-writeBuildCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
+writeBuildCache :: (MonadIO m, MonadReader env m, MonadThrow m, HasEnvConfig env)
                 => Path Abs Dir -> Map FilePath FileCacheInfo -> m ()
 writeBuildCache dir times =
     writeCache
@@ -131,21 +133,21 @@ writeBuildCache dir times =
          }
 
 -- | Write the dirtiness cache for this package's configuration.
-writeConfigCache :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
+writeConfigCache :: (MonadIO m, MonadReader env m, MonadThrow m, HasEnvConfig env)
                 => Path Abs Dir
                 -> ConfigCache
                 -> m ()
 writeConfigCache dir = writeCache dir configCacheFile
 
 -- | See 'tryGetCabalMod'
-writeCabalMod :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m, HasEnvConfig env)
+writeCabalMod :: (MonadIO m, MonadReader env m, MonadThrow m, HasEnvConfig env)
               => Path Abs Dir
               -> ModTime
               -> m ()
 writeCabalMod dir = writeCache dir configCabalMod
 
 -- | Delete the caches for the project.
-deleteCaches :: (MonadIO m, MonadReader env m, HasConfig env, MonadLogger m, MonadThrow m, HasEnvConfig env)
+deleteCaches :: (MonadIO m, MonadReader env m, MonadCatch m, HasEnvConfig env)
              => Path Abs Dir -> m ()
 deleteCaches dir = do
     {- FIXME confirm that this is acceptable to remove
@@ -153,7 +155,7 @@ deleteCaches dir = do
     removeFileIfExists bfp
     -}
     cfp <- configCacheFile dir
-    removeFileIfExists cfp
+    ignoringAbsence (removeFile cfp)
 
 -- | Write to a cache.
 writeCache :: (BinarySchema a, MonadIO m)
@@ -191,11 +193,11 @@ writeFlagCache :: (MonadIO m, MonadReader env m, HasEnvConfig env, MonadThrow m)
 writeFlagCache gid cache = do
     file <- flagCacheFile gid
     liftIO $ do
-        createTree (parent file)
+        ensureDir (parent file)
         taggedEncodeFile file cache
 
 -- | Mark a test suite as having succeeded
-setTestSuccess :: (MonadIO m, MonadLogger m, MonadThrow m, MonadReader env m, HasConfig env, HasEnvConfig env)
+setTestSuccess :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
                => Path Abs Dir
                -> m ()
 setTestSuccess dir =
@@ -205,7 +207,7 @@ setTestSuccess dir =
         True
 
 -- | Mark a test suite as not having succeeded
-unsetTestSuccess :: (MonadIO m, MonadLogger m, MonadThrow m, MonadReader env m, HasConfig env, HasEnvConfig env)
+unsetTestSuccess :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
                  => Path Abs Dir
                  -> m ()
 unsetTestSuccess dir =
@@ -215,7 +217,7 @@ unsetTestSuccess dir =
         False
 
 -- | Check if the test suite already passed
-checkTestSuccess :: (MonadIO m, MonadLogger m, MonadThrow m, MonadReader env m, HasConfig env, HasEnvConfig env)
+checkTestSuccess :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
                  => Path Abs Dir
                  -> m Bool
 checkTestSuccess dir =
@@ -234,7 +236,7 @@ checkTestSuccess dir =
 -- | The file containing information on the given package/configuration
 -- combination. The filename contains a hash of the non-directory configure
 -- options for quick lookup if there's a match.
-precompiledCacheFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
+precompiledCacheFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env, MonadLogger m)
                      => PackageIdentifier
                      -> ConfigureOpts
                      -> Set GhcPkgId -- ^ dependencies
@@ -255,28 +257,30 @@ precompiledCacheFile pkgident copts installedPackageIDs = do
     -- unnecessarily.
     --
     -- See issue: https://github.com/commercialhaskell/stack/issues/1103
-    let cacheInput
-            | envConfigCabalVersion ec >= $(mkVersion "1.22") =
-                Binary.encode $ coNoDirs copts
-            | otherwise =
-                Binary.encode
-                    ( coNoDirs copts
-                    , installedPackageIDs
-                    )
+    let computeCacheSource input = do
+            $logDebug $ "Precompiled cache input = " <> T.pack (show input)
+            return $ Binary.encode input
+    cacheInput <-
+        if envConfigCabalVersion ec >= $(mkVersion "1.22")
+            then computeCacheSource (coNoDirs copts)
+            else computeCacheSource (coNoDirs copts, installedPackageIDs)
 
     -- We only pay attention to non-directory options. We don't want to avoid a
     -- cache hit just because it was installed in a different directory.
     copts' <- parseRelFile $ S8.unpack $ B16.encode $ SHA256.hashlazy cacheInput
 
+    platformRelDir <- platformGhcRelDir
+
     return $ getStackRoot ec
          </> $(mkRelDir "precompiled")
+         </> platformRelDir
          </> compiler
          </> cabal
          </> pkg
          </> copts'
 
 -- | Write out information about a newly built package
-writePrecompiledCache :: (MonadThrow m, MonadReader env m, HasEnvConfig env, MonadIO m)
+writePrecompiledCache :: (MonadThrow m, MonadReader env m, HasEnvConfig env, MonadIO m, MonadLogger m)
                       => BaseConfigOpts
                       -> PackageIdentifier
                       -> ConfigureOpts
@@ -286,7 +290,7 @@ writePrecompiledCache :: (MonadThrow m, MonadReader env m, HasEnvConfig env, Mon
                       -> m ()
 writePrecompiledCache baseConfigOpts pkgident copts depIDs mghcPkgId exes = do
     file <- precompiledCacheFile pkgident copts depIDs
-    createTree $ parent file
+    ensureDir (parent file)
     mlibpath <-
         case mghcPkgId of
             Executable _ -> return Nothing
@@ -303,7 +307,7 @@ writePrecompiledCache baseConfigOpts pkgident copts depIDs mghcPkgId exes = do
 
 -- | Check the cache for a precompiled package matching the given
 -- configuration.
-readPrecompiledCache :: (MonadThrow m, MonadReader env m, HasEnvConfig env, MonadIO m)
+readPrecompiledCache :: (MonadThrow m, MonadReader env m, HasEnvConfig env, MonadIO m, MonadLogger m)
                      => PackageIdentifier -- ^ target package
                      -> ConfigureOpts
                      -> Set GhcPkgId -- ^ dependencies
