@@ -1,3 +1,4 @@
+{-# LANGUAGE DeriveGeneric         #-}
 {-# LANGUAGE FlexibleContexts      #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE OverloadedStrings     #-}
@@ -11,7 +12,7 @@ module Stack.Build.ConstructPlan
     ( constructPlan
     ) where
 
-import           Control.Arrow ((&&&), second)
+import           Control.Arrow ((&&&))
 import           Control.Exception.Lifted
 import           Control.Monad
 import           Control.Monad.Catch (MonadCatch)
@@ -32,18 +33,20 @@ import           Data.Text (Text)
 import qualified Data.Text as T
 import           Data.Text.Encoding (decodeUtf8With)
 import           Data.Text.Encoding.Error (lenientDecode)
-import           Distribution.Package (Dependency (..))
-import           Distribution.Version (anyVersion)
+import qualified Distribution.Package as Cabal
+import qualified Distribution.Version as Cabal
+import           GHC.Generics (Generic)
+import           Generics.Deriving.Monoid (memptydefault, mappenddefault)
 import           Network.HTTP.Client.Conduit (HasHttpManager)
 import           Prelude hiding (pi, writeFile)
 import           Stack.Build.Cache
 import           Stack.Build.Haddock
 import           Stack.Build.Installed
 import           Stack.Build.Source
-import           Stack.Types.Build
 import           Stack.BuildPlan
 import           Stack.Package
 import           Stack.PackageDump
+import           Stack.PackageIndex
 import           Stack.Types
 
 data PackageInfo
@@ -85,10 +88,10 @@ data W = W
     -- ^ Packages which count as dependencies
     , wWarnings :: !([Text] -> [Text])
     -- ^ Warnings
-    }
+    } deriving Generic
 instance Monoid W where
-    mempty = W mempty mempty mempty mempty mempty
-    mappend (W a b c d e) (W w x y z z') = W (mappend a w) (mappend b x) (mappend c y) (mappend d z) (mappend e z')
+    mempty = memptydefault
+    mappend = mappenddefault
 
 type M = RWST
     Ctx
@@ -99,13 +102,13 @@ type M = RWST
 data Ctx = Ctx
     { mbp            :: !MiniBuildPlan
     , baseConfigOpts :: !BaseConfigOpts
-    , loadPackage    :: !(PackageName -> Version -> Map FlagName Bool -> IO Package)
+    , loadPackage    :: !(PackageName -> Version -> Map FlagName Bool -> [Text] -> IO Package)
     , combinedMap    :: !CombinedMap
-    , toolToPackages :: !(Dependency -> Map PackageName VersionRange)
+    , toolToPackages :: !(Cabal.Dependency -> Map PackageName VersionRange)
     , ctxEnvConfig   :: !EnvConfig
     , callStack      :: ![PackageName]
     , extraToBuild   :: !(Set PackageName)
-    , ctxVersions    :: !(Map PackageName (Set Version))
+    , getVersions    :: !(PackageName -> IO (Set Version))
     , wanted         :: !(Set PackageName)
     , localNames     :: !(Set PackageName)
     }
@@ -126,17 +129,13 @@ constructPlan :: forall env m.
               -> [LocalPackage]
               -> Set PackageName -- ^ additional packages that must be built
               -> [DumpPackage () ()] -- ^ locally registered
-              -> (PackageName -> Version -> Map FlagName Bool -> IO Package) -- ^ load upstream package
+              -> (PackageName -> Version -> Map FlagName Bool -> [Text] -> IO Package) -- ^ load upstream package
               -> SourceMap
               -> InstalledMap
               -> m Plan
 constructPlan mbp0 baseConfigOpts0 locals extraToBuild0 localDumpPkgs loadPackage0 sourceMap installedMap = do
     let locallyRegistered = Map.fromList $ map (dpGhcPkgId &&& dpPackageIdent) localDumpPkgs
-    bconfig <- asks getBuildConfig
-    let versions =
-            Map.fromListWith Set.union $
-            map (second Set.singleton . toTuple) $
-            Map.keys (bcPackageCaches bconfig)
+    getVersions0 <- getPackageVersionsIO
 
     econfig <- asks getEnvConfig
     let onWanted = void . addDep False . packageName . lpPackage
@@ -144,7 +143,7 @@ constructPlan mbp0 baseConfigOpts0 locals extraToBuild0 localDumpPkgs loadPackag
             mapM_ onWanted $ filter lpWanted locals
             mapM_ (addDep False) $ Set.toList extraToBuild0
     ((), m, W efinals installExes dirtyReason deps warnings) <-
-        liftIO $ runRWST inner (ctx econfig versions) M.empty
+        liftIO $ runRWST inner (ctx econfig getVersions0) M.empty
     mapM_ $logWarn (warnings [])
     let toEither (_, Left e)  = Left e
         toEither (k, Right v) = Right (k, v)
@@ -157,7 +156,7 @@ constructPlan mbp0 baseConfigOpts0 locals extraToBuild0 localDumpPkgs loadPackag
                 toTask (name, ADRToInstall task) = Just (name, task)
                 tasks = M.fromList $ mapMaybe toTask adrs
                 takeSubset =
-                    case boptsBuildSubset $ bcoBuildOpts baseConfigOpts0 of
+                    case boptsCLIBuildSubset $ bcoBuildOptsCLI baseConfigOpts0 of
                         BSAll -> id
                         BSOnlySnapshot -> stripLocals
                         BSOnlyDependencies -> stripNonDeps deps
@@ -172,18 +171,18 @@ constructPlan mbp0 baseConfigOpts0 locals extraToBuild0 localDumpPkgs loadPackag
                 }
         else throwM $ ConstructPlanExceptions errs (bcStackYaml $ getBuildConfig econfig)
   where
-    ctx econfig versions = Ctx
+    ctx econfig getVersions0 = Ctx
         { mbp = mbp0
         , baseConfigOpts = baseConfigOpts0
         , loadPackage = loadPackage0
         , combinedMap = combineMap sourceMap installedMap
-        , toolToPackages = \ (Dependency name _) ->
-          maybe Map.empty (Map.fromSet (const anyVersion)) $
+        , toolToPackages = \(Cabal.Dependency name _) ->
+          maybe Map.empty (Map.fromSet (const Cabal.anyVersion)) $
           Map.lookup (T.pack . packageNameString . fromCabalPackageName $ name) toolMap
         , ctxEnvConfig = econfig
         , callStack = []
         , extraToBuild = extraToBuild0
-        , ctxVersions = versions
+        , getVersions = getVersions0
         , wanted = wantedLocalPackages locals
         , localNames = Set.fromList $ map (packageName . lpPackage) locals
         }
@@ -206,7 +205,7 @@ mkUnregisterLocal tasks dirtyReason locallyRegistered sourceMap =
         case M.lookup name tasks of
             Nothing ->
                 case M.lookup name sourceMap of
-                    Just (PSUpstream _ Snap _) -> Map.singleton gid
+                    Just (PSUpstream _ Snap _ _ _) -> Map.singleton gid
                         ( ident
                         , Just "Switching to snapshot installed package"
                         )
@@ -235,7 +234,6 @@ addFinal lp package isAllInOne = do
                             (getEnvConfig ctx)
                             (baseConfigOpts ctx)
                             allDeps
-                            True -- wanted
                             True -- local
                             Local
                             package
@@ -280,14 +278,16 @@ tellExecutables :: PackageName -> PackageSource -> M ()
 tellExecutables _ (PSLocal lp)
     | lpWanted lp = tellExecutablesPackage Local $ lpPackage lp
     | otherwise = return ()
-tellExecutables name (PSUpstream version loc flags) =
+-- Ignores ghcOptions because they don't matter for enumerating
+-- executables.
+tellExecutables name (PSUpstream version loc flags _ghcOptions _gitSha) =
     tellExecutablesUpstream name version loc flags
 
 tellExecutablesUpstream :: PackageName -> Version -> InstallLocation -> Map FlagName Bool -> M ()
 tellExecutablesUpstream name version loc flags = do
     ctx <- ask
     when (name `Set.member` extraToBuild ctx) $ do
-        p <- liftIO $ loadPackage ctx name version flags
+        p <- liftIO $ loadPackage ctx name version flags []
         tellExecutablesPackage loc p
 
 tellExecutablesPackage :: InstallLocation -> Package -> M ()
@@ -320,8 +320,8 @@ installPackage :: Bool -- ^ is this being used by a dependency?
 installPackage treatAsDep name ps minstalled = do
     ctx <- ask
     case ps of
-        PSUpstream version _ flags -> do
-            package <- liftIO $ loadPackage ctx name version flags
+        PSUpstream version _ flags ghcOptions _ -> do
+            package <- liftIO $ loadPackage ctx name version flags ghcOptions
             resolveDepsAndInstall False treatAsDep ps package minstalled
         PSLocal lp ->
             case lpTestBench lp of
@@ -404,7 +404,6 @@ installPackageGivenDeps isAllInOne ps package minstalled (missing, present, minL
                         (getEnvConfig ctx)
                         (baseConfigOpts ctx)
                         allDeps
-                        (psWanted ps)
                         (psLocal ps)
                         -- An assertion to check for a recurrence of
                         -- https://github.com/commercialhaskell/stack/issues/345
@@ -414,7 +413,7 @@ installPackageGivenDeps isAllInOne ps package minstalled (missing, present, minL
             , taskType =
                 case ps of
                     PSLocal lp -> TTLocal lp
-                    PSUpstream _ loc _ -> TTUpstream package $ loc <> minLoc
+                    PSUpstream _ loc _ _ sha -> TTUpstream package (loc <> minLoc) sha
             , taskAllInOne = isAllInOne
             }
 
@@ -430,15 +429,17 @@ addPackageDeps treatAsDep package = do
     deps' <- packageDepsWithTools package
     deps <- forM (Map.toList deps') $ \(depname, range) -> do
         eres <- addDep treatAsDep depname
-        let mlatestApplicable =
-                (latestApplicableVersion range <=< Map.lookup depname) (ctxVersions ctx)
+        let getLatestApplicable = do
+                vs <- liftIO $ getVersions ctx depname
+                return (latestApplicableVersion range vs)
         case eres of
-            Left e ->
+            Left e -> do
                 let bd =
                         case e of
                             UnknownPackage name -> assert (name == depname) NotInBuildPlan
                             _ -> Couldn'tResolveItsDependencies
-                 in return $ Left (depname, (range, mlatestApplicable, bd))
+                mlatestApplicable <- getLatestApplicable
+                return $ Left (depname, (range, mlatestApplicable, bd))
             Right adr -> do
                 inRange <- if adrVersion adr `withinRange` range
                     then return True
@@ -477,13 +478,13 @@ addPackageDeps treatAsDep package = do
                             (Set.empty, Map.empty, loc)
                         ADRFound loc (Library ident gid) -> return $ Right
                             (Set.empty, Map.singleton ident gid, loc)
-                    else return $ Left (depname, (range, mlatestApplicable, DependencyMismatch $ adrVersion adr))
+                    else do
+                        mlatestApplicable <- getLatestApplicable
+                        return $ Left (depname, (range, mlatestApplicable, DependencyMismatch $ adrVersion adr))
     case partitionEithers deps of
         ([], pairs) -> return $ Right $ mconcat pairs
         (errs, _) -> return $ Left $ DependencyPlanFailures
-            (PackageIdentifier
-                (packageName package)
-                (packageVersion package))
+            package
             (Map.fromList errs)
   where
     adrVersion (ADRToInstall task) = packageIdentifierVersion $ taskProvides task
@@ -502,7 +503,6 @@ checkDirtiness ps installed package present wanted = do
             (getEnvConfig ctx)
             (baseConfigOpts ctx)
             present
-            (psWanted ps)
             (psLocal ps)
             (piiLocation ps) -- should be Local always
             package
@@ -524,6 +524,7 @@ checkDirtiness ps installed package present wanted = do
                 Nothing -> Just "old configure information not found"
                 Just oldOpts
                     | Just reason <- describeConfigDiff config oldOpts wantConfigCache -> Just reason
+                    | True <- psForceDirty ps -> Just "--force-dirty specified"
                     | Just files <- psDirty ps -> Just $ "local file changes: " <>
                                                          addEllipsis (T.pack $ unwords $ Set.toList files)
                     | otherwise -> Nothing
@@ -549,25 +550,6 @@ describeConfigDiff config old new
         ]
     | otherwise = Nothing
   where
-    -- options set by stack
-    isStackOpt t = any (`T.isPrefixOf` t)
-        [ "--dependency="
-        , "--constraint="
-        , "--package-db="
-        , "--libdir="
-        , "--bindir="
-        , "--datadir="
-        , "--libexecdir="
-        , "--sysconfdir"
-        , "--docdir="
-        , "--htmldir="
-        , "--haddockdir="
-        , "--enable-tests"
-        , "--enable-benchmarks"
-        ] || elem t
-        [ "--user"
-        ]
-
     stripGhcOptions =
         go
       where
@@ -608,14 +590,14 @@ describeConfigDiff config old new
 
     newComponents = configCacheComponents new `Set.difference` configCacheComponents old
 
+psForceDirty :: PackageSource -> Bool
+psForceDirty (PSLocal lp) = lpForceDirty lp
+psForceDirty (PSUpstream {}) = False
+
 psDirty :: PackageSource -> Maybe (Set FilePath)
 psDirty (PSLocal lp) = lpDirtyFiles lp
 psDirty (PSUpstream {}) = Nothing -- files never change in an upstream package
 
-psWanted :: PackageSource -> Bool
-psWanted (PSLocal lp) = lpWanted lp
-psWanted (PSUpstream {}) = False
-
 psLocal :: PackageSource -> Bool
 psLocal (PSLocal _) = True
 psLocal (PSUpstream {}) = False
@@ -625,9 +607,51 @@ psLocal (PSUpstream {}) = False
 packageDepsWithTools :: Package -> M (Map PackageName VersionRange)
 packageDepsWithTools p = do
     ctx <- ask
+    -- TODO: it would be cool to defer these warnings until there's an
+    -- actual issue building the package.
+    -- TODO: check if the tool is on the path before warning?
+    let toEither (Cabal.Dependency (Cabal.PackageName name) _) mp =
+            case Map.toList mp of
+                [] -> Left (NoToolFound name (packageName p))
+                [_] -> Right mp
+                xs -> Left (AmbiguousToolsFound name (packageName p) (map fst xs))
+        (warnings, toolDeps) =
+             partitionEithers $
+             map (\dep -> toEither dep (toolToPackages ctx dep)) (packageTools p)
+    tell mempty { wWarnings = (map toolWarningText warnings ++) }
+    when (any isNoToolFound warnings) $ do
+        let msg = T.unlines
+                [ "Missing build-tools may be caused by dependencies of the build-tool being overridden by extra-deps."
+                , "This should be fixed soon - see this issue https://github.com/commercialhaskell/stack/issues/595"
+                ]
+        tell mempty { wWarnings = (msg:) }
     return $ Map.unionsWith intersectVersionRanges
            $ packageDeps p
-           : map (toolToPackages ctx) (packageTools p)
+           : toolDeps
+
+data ToolWarning
+    = NoToolFound String PackageName
+    | AmbiguousToolsFound String PackageName [PackageName]
+
+isNoToolFound :: ToolWarning -> Bool
+isNoToolFound NoToolFound{} = True
+isNoToolFound _ = False
+
+toolWarningText :: ToolWarning -> Text
+toolWarningText (NoToolFound toolName pkgName) =
+    "No packages found in snapshot which provide a " <>
+    T.pack (show toolName) <>
+    " executable, which is a build-tool dependency of " <>
+    T.pack (show (packageNameString pkgName))
+toolWarningText (AmbiguousToolsFound toolName pkgName options) =
+    "Multiple packages found in snapshot which provide a " <>
+    T.pack (show toolName) <>
+    " exeuctable, which is a build-tool dependency of " <>
+    T.pack (show (packageNameString pkgName)) <>
+    ", so none will be installed.\n" <>
+    "Here's the list of packages which provide it: " <>
+    T.intercalate ", " (map packageNameText options) <>
+    "\nSince there's no good way to choose, you may need to install it manually."
 
 -- | Strip out anything from the @Plan@ intended for the local database
 stripLocals :: Plan -> Plan
@@ -641,8 +665,8 @@ stripLocals plan = plan
     checkTask task =
         case taskType task of
             TTLocal _ -> False
-            TTUpstream _ Local -> False
-            TTUpstream _ Snap -> True
+            TTUpstream _ Local _ -> False
+            TTUpstream _ Snap _ -> True
 
 stripNonDeps :: Set PackageName -> Plan -> Plan
 stripNonDeps deps plan = plan
