@@ -11,90 +11,86 @@ module Stack.Build.Source
     ( loadSourceMap
     , SourceMap
     , PackageSource (..)
-    , localFlags
+    , getLocalFlags
+    , getGhcOptions
     , getLocalPackageViews
-    , loadLocalPackage
     , parseTargetsFromBuildOpts
     , addUnlistedToBuildCache
+    , getDefaultPackageConfig
     , getPackageConfig
     ) where
 
-import           Control.Applicative
-import           Control.Arrow ((&&&))
-import           Control.Exception (assert, catch)
-import           Control.Monad
-import           Control.Monad.Catch (MonadCatch)
-import           Control.Monad.IO.Class
-import           Control.Monad.Logger
-import           Control.Monad.Reader (MonadReader, asks)
-import           Control.Monad.Trans.Resource
+import              Control.Applicative
+import              Control.Arrow ((&&&))
+import              Control.Exception (assert, catch)
+import              Control.Monad hiding (sequence)
+import              Control.Monad.Catch (MonadMask)
+import              Control.Monad.IO.Class
+import              Control.Monad.Logger
+import              Control.Monad.Reader (MonadReader, asks)
+import              Control.Monad.Trans.Resource
 import "cryptohash" Crypto.Hash (Digest, SHA256)
-import           Crypto.Hash.Conduit (sinkHash)
-import qualified Data.ByteString as S
-import           Data.Byteable (toBytes)
-import           Data.Conduit (($$), ZipSink (..))
-import qualified Data.Conduit.Binary as CB
-import qualified Data.Conduit.List as CL
-import           Data.Either
-import           Data.Function
-import qualified Data.HashSet as HashSet
-import           Data.List
-import qualified Data.Map as Map
-import           Data.Map.Strict (Map)
-import qualified Data.Map.Strict as M
-import           Data.Maybe
-import           Data.Monoid
-import           Data.Set (Set)
-import qualified Data.Set as Set
-import           Data.Text (Text)
-import qualified Data.Text as T
-import           Distribution.Package (pkgName, pkgVersion)
-import           Distribution.PackageDescription (GenericPackageDescription, package, packageDescription)
-import qualified Distribution.PackageDescription as C
-import           Network.HTTP.Client.Conduit (HasHttpManager)
-import           Path
-import           Path.IO
-import           Prelude
-import           Stack.Build.Cache
-import           Stack.Build.Target
-import           Stack.BuildPlan (loadMiniBuildPlan, shadowMiniBuildPlan,
-                                  parseCustomMiniBuildPlan)
-import           Stack.Constants (wiredInPackages)
-import           Stack.Package
-import           Stack.Types
+import              Crypto.Hash.Conduit (sinkHash)
+import qualified    Data.ByteString as S
+import              Data.Byteable (toBytes)
+import              Data.Conduit (($$), ZipSink (..))
+import qualified    Data.Conduit.Binary as CB
+import qualified    Data.Conduit.List as CL
+import              Data.Either
+import              Data.Function
+import qualified    Data.HashSet as HashSet
+import              Data.List
+import qualified    Data.Map as Map
+import              Data.Map.Strict (Map)
+import qualified    Data.Map.Strict as M
+import              Data.Maybe
+import              Data.Monoid
+import              Data.Set (Set)
+import qualified    Data.Set as Set
+import              Data.Text (Text)
+import qualified    Data.Text as T
+import              Data.Traversable (sequence)
+import              Distribution.Package (pkgName, pkgVersion)
+import              Distribution.PackageDescription (GenericPackageDescription, package, packageDescription)
+import qualified    Distribution.PackageDescription as C
+import              Network.HTTP.Client.Conduit (HasHttpManager)
+import              Path
+import              Path.IO
+import              Prelude hiding (sequence)
+import              Stack.Build.Cache
+import              Stack.Build.Target
+import              Stack.BuildPlan (shadowMiniBuildPlan)
+import              Stack.Constants (wiredInPackages)
+import              Stack.Package
+import              Stack.PackageIndex (getPackageVersions)
+import              Stack.Types
+import qualified    System.Directory as D
+import              System.FilePath (takeFileName)
+import              System.IO (withBinaryFile, IOMode (ReadMode))
+import              System.IO.Error (isDoesNotExistError)
 
-import           System.Directory
-import           System.IO (withBinaryFile, IOMode (ReadMode))
-import           System.IO.Error (isDoesNotExistError)
-
-loadSourceMap :: (MonadIO m, MonadCatch m, MonadReader env m, HasBuildConfig env, MonadBaseControl IO m, HasHttpManager env, MonadLogger m, HasEnvConfig env)
+loadSourceMap :: (MonadIO m, MonadMask m, MonadReader env m, MonadBaseControl IO m, HasHttpManager env, MonadLogger m, HasEnvConfig env)
               => NeedTargets
-              -> BuildOpts
+              -> BuildOptsCLI
               -> m ( Map PackageName SimpleTarget
                    , MiniBuildPlan
                    , [LocalPackage]
                    , Set PackageName -- non-local targets
                    , SourceMap
                    )
-loadSourceMap needTargets bopts = do
+loadSourceMap needTargets boptsCli = do
     bconfig <- asks getBuildConfig
     rawLocals <- getLocalPackageViews
-    (mbp0, cliExtraDeps, targets) <- parseTargetsFromBuildOpts needTargets bopts
-    let latestVersion =
-            Map.fromListWith max $
-            map toTuple $
-            Map.keys (bcPackageCaches bconfig)
-
+    (mbp0, cliExtraDeps, targets) <- parseTargetsFromBuildOpts needTargets boptsCli
     -- Extend extra-deps to encompass targets requested on the command line
     -- that are not in the snapshot.
     extraDeps0 <- extendExtraDeps
         (bcExtraDeps bconfig)
         cliExtraDeps
         (Map.keysSet $ Map.filter (== STUnknown) targets)
-        latestVersion
 
-    locals <- mapM (loadLocalPackage bopts targets) $ Map.toList rawLocals
-    checkFlagsUsed bopts locals extraDeps0 (mbpPackages mbp0)
+    locals <- mapM (loadLocalPackage boptsCli targets) $ Map.toList rawLocals
+    checkFlagsUsed boptsCli locals extraDeps0 (mbpPackages mbp0)
     checkComponentsBuildable locals
 
     let
@@ -116,27 +112,34 @@ loadSourceMap needTargets bopts = do
         -- Add the extra deps from the stack.yaml file to the deps grabbed from
         -- the snapshot
         extraDeps2 = Map.union
-            (Map.map (\v -> (v, Map.empty)) extraDeps0)
-            (Map.map (mpiVersion &&& mpiFlags) extraDeps1)
+            (Map.map (\v -> (v, Map.empty, [])) extraDeps0)
+            (Map.map (\mpi -> (mpiVersion mpi, mpiFlags mpi, mpiGhcOptions mpi)) extraDeps1)
 
-        -- Overwrite any flag settings with those from the config file
+        -- Add flag and ghc-option settings from the config file / cli
         extraDeps3 = Map.mapWithKey
-            (\n (v, f) -> PSUpstream v Local $
-                case ( Map.lookup (Just n) $ boptsFlags bopts
-                     , Map.lookup Nothing $ boptsFlags bopts
-                     , Map.lookup n $ bcFlags bconfig
-                     ) of
-                    -- Didn't have any flag overrides, fall back to the flags
-                    -- defined in the snapshot.
-                    (Nothing, Nothing, Nothing) -> f
-                    -- Either command line flag for this package, general
-                    -- command line flag, or flag in stack.yaml is defined.
-                    -- Take all of those and ignore the snapshot flags.
-                    (x, y, z) -> Map.unions
-                        [ fromMaybe Map.empty x
-                        , fromMaybe Map.empty y
-                        , fromMaybe Map.empty z
-                        ])
+            (\n (v, flags0, ghcOptions0) ->
+                let flags =
+                        case ( Map.lookup (Just n) $ boptsCLIFlags boptsCli
+                             , Map.lookup Nothing $ boptsCLIFlags boptsCli
+                             , Map.lookup n $ unPackageFlags $ bcFlags bconfig
+                             ) of
+                            -- Didn't have any flag overrides, fall back to the flags
+                            -- defined in the snapshot.
+                            (Nothing, Nothing, Nothing) -> flags0
+                            -- Either command line flag for this package, general
+                            -- command line flag, or flag in stack.yaml is defined.
+                            -- Take all of those and ignore the snapshot flags.
+                            (x, y, z) -> Map.unions
+                                [ fromMaybe Map.empty x
+                                , fromMaybe Map.empty y
+                                , fromMaybe Map.empty z
+                                ]
+                    ghcOptions =
+                        ghcOptions0 ++
+                        getGhcOptions bconfig boptsCli n False False
+                 -- currently have no ability for extra-deps to specify their
+                 -- cabal file hashes
+                in PSUpstream v Local flags ghcOptions Nothing)
             extraDeps2
 
     let sourceMap = Map.unions
@@ -145,24 +148,55 @@ loadSourceMap needTargets bopts = do
                  in (packageName p, PSLocal lp)
             , extraDeps3
             , flip fmap (mbpPackages mbp) $ \mpi ->
-                PSUpstream (mpiVersion mpi) Snap (mpiFlags mpi)
+                PSUpstream (mpiVersion mpi) Snap (mpiFlags mpi) (mpiGhcOptions mpi) (mpiGitSHA1 mpi)
             ] `Map.difference` Map.fromList (map (, ()) (HashSet.toList wiredInPackages))
 
     return (targets, mbp, locals, nonLocalTargets, sourceMap)
 
+-- | All flags for a local package
+getLocalFlags
+    :: BuildConfig
+    -> BuildOptsCLI
+    -> PackageName
+    -> Map FlagName Bool
+getLocalFlags bconfig boptsCli name = Map.unions
+    [ Map.findWithDefault Map.empty (Just name) cliFlags
+    , Map.findWithDefault Map.empty Nothing cliFlags
+    , Map.findWithDefault Map.empty name (unPackageFlags (bcFlags bconfig))
+    ]
+  where
+    cliFlags = boptsCLIFlags boptsCli
+
+getGhcOptions :: BuildConfig -> BuildOptsCLI -> PackageName -> Bool -> Bool -> [Text]
+getGhcOptions bconfig boptsCli name isTarget isLocal = concat
+    [ ghcOptionsFor name (configGhcOptions config)
+    , concat [["-fhpc"] | isLocal && toCoverage (boptsTestOpts bopts)]
+    , if (boptsLibProfile bopts || boptsExeProfile bopts)
+         then ["-auto-all","-caf-all"]
+         else []
+    , if includeExtraOptions
+         then boptsCLIGhcOptions boptsCli
+         else []
+    ]
+  where
+    bopts = configBuild config
+    config = bcConfig bconfig
+    includeExtraOptions =
+        case configApplyGhcOptions config of
+            AGOTargets -> isTarget
+            AGOLocals -> isLocal
+            AGOEverything -> True
+
 -- | Use the build options and environment to parse targets.
 parseTargetsFromBuildOpts
-    :: (MonadIO m, MonadCatch m, MonadReader env m, HasBuildConfig env, MonadBaseControl IO m, HasHttpManager env, MonadLogger m, HasEnvConfig env)
+    :: (MonadIO m, MonadMask m, MonadReader env m, MonadLogger m, HasEnvConfig env)
     => NeedTargets
-    -> BuildOpts
+    -> BuildOptsCLI
     -> m (MiniBuildPlan, M.Map PackageName Version, M.Map PackageName SimpleTarget)
-parseTargetsFromBuildOpts needTargets bopts = do
+parseTargetsFromBuildOpts needTargets boptscli = do
     bconfig <- asks getBuildConfig
     mbp0 <-
         case bcResolver bconfig of
-            ResolverSnapshot snapName -> do
-                $logDebug $ "Checking resolver: " <> renderSnapName snapName
-                loadMiniBuildPlan snapName
             ResolverCompiler _ -> do
                 -- We ignore the resolver version, as it might be
                 -- GhcMajorVersion, and we want the exact version
@@ -172,18 +206,16 @@ parseTargetsFromBuildOpts needTargets bopts = do
                     { mbpCompilerVersion = version
                     , mbpPackages = Map.empty
                     }
-            ResolverCustom _ url -> do
-                stackYamlFP <- asks $ bcStackYaml . getBuildConfig
-                parseCustomMiniBuildPlan stackYamlFP url
+            _ -> return (bcWantedMiniBuildPlan bconfig)
     rawLocals <- getLocalPackageViews
-    workingDir <- getWorkingDir
+    workingDir <- getCurrentDir
 
     let snapshot = mpiVersion <$> mbpPackages mbp0
     flagExtraDeps <- convertSnapshotToExtra
         snapshot
         (bcExtraDeps bconfig)
         rawLocals
-        (catMaybes $ Map.keys $ boptsFlags bopts)
+        (catMaybes $ Map.keys $ boptsCLIFlags boptscli)
 
     (cliExtraDeps, targets) <-
         parseTargets
@@ -193,7 +225,7 @@ parseTargetsFromBuildOpts needTargets bopts = do
             (flagExtraDeps <> bcExtraDeps bconfig)
             (fst <$> rawLocals)
             workingDir
-            (boptsTargets bopts)
+            (boptsCLITargets boptscli)
     return (mbp0, cliExtraDeps <> flagExtraDeps, targets)
 
 -- | For every package in the snapshot which is referenced by a flag, give the
@@ -226,8 +258,8 @@ getLocalPackageViews :: (MonadThrow m, MonadIO m, MonadReader env m, HasEnvConfi
                      => m (Map PackageName (LocalPackageView, GenericPackageDescription))
 getLocalPackageViews = do
     econfig <- asks getEnvConfig
-    locals <- forM (Map.toList $ envConfigPackages econfig) $ \(dir, validWanted) -> do
-        cabalfp <- getCabalFileName dir
+    locals <- forM (Map.toList $ envConfigPackages econfig) $ \(dir, treatLikeExtraDep) -> do
+        cabalfp <- findOrGenerateCabalFile dir
         (warnings,gpkg) <- readPackageUnresolved cabalfp
         mapM_ (printCabalFileWarning cabalfp) warnings
         let cabalID = package $ packageDescription gpkg
@@ -237,7 +269,7 @@ getLocalPackageViews = do
                 { lpvVersion = fromCabalVersion $ pkgVersion cabalID
                 , lpvRoot = dir
                 , lpvCabalFP = cabalfp
-                , lpvExtraDep = not validWanted
+                , lpvExtraDep = treatLikeExtraDep
                 , lpvComponents = getNamedComponents gpkg
                 }
         return (name, (lpv, gpkg))
@@ -280,17 +312,17 @@ splitComponents =
 -- based on the selected components
 loadLocalPackage
     :: forall m env.
-       (MonadReader env m, HasEnvConfig env, MonadCatch m, MonadLogger m, MonadIO m)
-    => BuildOpts
+       (MonadReader env m, HasEnvConfig env, MonadMask m, MonadLogger m, MonadIO m)
+    => BuildOptsCLI
     -> Map PackageName SimpleTarget
     -> (PackageName, (LocalPackageView, GenericPackageDescription))
     -> m LocalPackage
-loadLocalPackage bopts targets (name, (lpv, gpkg)) = do
-    config  <- getPackageConfig bopts name
-
+loadLocalPackage boptsCli targets (name, (lpv, gpkg)) = do
+    let mtarget = Map.lookup name targets
+    config  <- getPackageConfig boptsCli name (isJust mtarget) True
+    bopts <- asks (configBuild . getConfig)
     let pkg = resolvePackage config gpkg
 
-        mtarget = Map.lookup name targets
         (exes, tests, benches) =
             case mtarget of
                 Just (STLocalComps comps) -> splitComponents $ Set.toList comps
@@ -334,12 +366,9 @@ loadLocalPackage bopts targets (name, (lpv, gpkg)) = do
     mbuildCache <- tryGetBuildCache $ lpvRoot lpv
     (files,_) <- getPackageFilesSimple pkg (lpvCabalFP lpv)
 
-    -- Filter out the cabal_macros file to avoid spurious recompilations
-    let filteredFiles = Set.filter ((/= $(mkRelFile "cabal_macros.h")) . filename) files
-
     (dirtyFiles, newBuildCache) <- checkBuildCache
         (fromMaybe Map.empty mbuildCache)
-        (map toFilePath $ Set.toList filteredFiles)
+        (Set.toList files)
 
     return LocalPackage
         { lpPackage = pkg
@@ -347,8 +376,9 @@ loadLocalPackage bopts targets (name, (lpv, gpkg)) = do
         , lpBenchDeps = packageDeps benchpkg
         , lpTestBench = btpkg
         , lpFiles = files
+        , lpForceDirty = boptsForceDirty bopts
         , lpDirtyFiles =
-            if not (Set.null dirtyFiles) || boptsForceDirty bopts
+            if not (Set.null dirtyFiles)
                 then let tryStripPrefix y =
                           fromMaybe y (stripPrefix (toFilePath $ lpvRoot lpv) y)
                       in Just $ Set.map tryStripPrefix dirtyFiles
@@ -373,18 +403,18 @@ loadLocalPackage bopts targets (name, (lpv, gpkg)) = do
 -- | Ensure that the flags specified in the stack.yaml file and on the command
 -- line are used.
 checkFlagsUsed :: (MonadThrow m, MonadReader env m, HasBuildConfig env)
-               => BuildOpts
+               => BuildOptsCLI
                -> [LocalPackage]
                -> Map PackageName extraDeps -- ^ extra deps
                -> Map PackageName snapshot -- ^ snapshot, for error messages
                -> m ()
-checkFlagsUsed bopts lps extraDeps snapshot = do
+checkFlagsUsed boptsCli lps extraDeps snapshot = do
     bconfig <- asks getBuildConfig
 
         -- Check if flags specified in stack.yaml and the command line are
         -- used, see https://github.com/commercialhaskell/stack/issues/617
-    let flags = map (, FSCommandLine) [(k, v) | (Just k, v) <- Map.toList $ boptsFlags bopts]
-             ++ map (, FSStackYaml) (Map.toList $ bcFlags bconfig)
+    let flags = map (, FSCommandLine) [(k, v) | (Just k, v) <- Map.toList $ boptsCLIFlags boptsCli]
+             ++ map (, FSStackYaml) (Map.toList $ unPackageFlags $ bcFlags bconfig)
 
         localNameMap = Map.fromList $ map (packageName . lpPackage &&& lpPackage) lps
         checkFlagUsed ((name, userFlags), source) =
@@ -415,81 +445,82 @@ checkFlagsUsed bopts lps extraDeps snapshot = do
         $ InvalidFlagSpecification
         $ Set.fromList unusedFlags
 
--- | All flags for a local package
-localFlags :: Map (Maybe PackageName) (Map FlagName Bool)
-           -> BuildConfig
-           -> PackageName
-           -> Map FlagName Bool
-localFlags boptsflags bconfig name = Map.unions
-    [ Map.findWithDefault Map.empty (Just name) boptsflags
-    , Map.findWithDefault Map.empty Nothing boptsflags
-    , Map.findWithDefault Map.empty name (bcFlags bconfig)
-    ]
-
 -- | Add in necessary packages to extra dependencies
 --
 -- Originally part of https://github.com/commercialhaskell/stack/issues/272,
 -- this was then superseded by
 -- https://github.com/commercialhaskell/stack/issues/651
-extendExtraDeps :: (MonadThrow m, MonadReader env m, HasBuildConfig env)
-                => Map PackageName Version -- ^ original extra deps
-                -> Map PackageName Version -- ^ package identifiers from the command line
-                -> Set PackageName -- ^ all packages added on the command line
-                -> Map PackageName Version -- ^ latest versions in indices
-                -> m (Map PackageName Version) -- ^ new extradeps
-extendExtraDeps extraDeps0 cliExtraDeps unknowns latestVersion
-    | null errs = return $ Map.unions $ extraDeps1 : unknowns'
-    | otherwise = do
-        bconfig <- asks getBuildConfig
-        throwM $ UnknownTargets
-            (Set.fromList errs)
-            Map.empty -- TODO check the cliExtraDeps for presence in index
-            (bcStackYaml bconfig)
+extendExtraDeps
+    :: (HasBuildConfig env, MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env, MonadBaseControl IO m, MonadMask m)
+    => Map PackageName Version -- ^ original extra deps
+    -> Map PackageName Version -- ^ package identifiers from the command line
+    -> Set PackageName -- ^ all packages added on the command line
+    -> m (Map PackageName Version) -- ^ new extradeps
+extendExtraDeps extraDeps0 cliExtraDeps unknowns = do
+    (errs, unknowns') <- fmap partitionEithers $ mapM addUnknown $ Set.toList unknowns
+    case errs of
+        [] -> return $ Map.unions $ extraDeps1 : unknowns'
+        _ -> do
+            bconfig <- asks getBuildConfig
+            throwM $ UnknownTargets
+                (Set.fromList errs)
+                Map.empty -- TODO check the cliExtraDeps for presence in index
+                (bcStackYaml bconfig)
   where
     extraDeps1 = Map.union extraDeps0 cliExtraDeps
-
-    (errs, unknowns') = partitionEithers $ map addUnknown $ Set.toList unknowns
-    addUnknown pn =
+    addUnknown pn = do
         case Map.lookup pn extraDeps1 of
-            Just _ -> Right Map.empty
-            Nothing ->
-                case Map.lookup pn latestVersion of
-                    Just v -> Right $ Map.singleton pn v
-                    Nothing -> Left pn
+            Just _ -> return (Right Map.empty)
+            Nothing -> do
+                mlatestVersion <- getLatestVersion pn
+                case mlatestVersion of
+                    Just v -> return (Right $ Map.singleton pn v)
+                    Nothing -> return (Left pn)
+    getLatestVersion pn = do
+        vs <- getPackageVersions pn
+        return (fmap fst (Set.maxView vs))
 
 -- | Compare the current filesystem state to the cached information, and
 -- determine (1) if the files are dirty, and (2) the new cache values.
-checkBuildCache :: MonadIO m
+checkBuildCache :: forall m. (MonadIO m)
                 => Map FilePath FileCacheInfo -- ^ old cache
-                -> [FilePath] -- ^ files in package
+                -> [Path Abs File] -- ^ files in package
                 -> m (Set FilePath, Map FilePath FileCacheInfo)
-checkBuildCache oldCache files = liftIO $ do
-    (dirtyFiles, m) <- mconcat <$> mapM go files
-    return (dirtyFiles, m)
+checkBuildCache oldCache files = do
+    fileTimes <- liftM Map.fromList $ forM files $ \fp -> do
+        mmodTime <- liftIO (getModTimeMaybe (toFilePath fp))
+        return (toFilePath fp, mmodTime)
+    liftM (mconcat . Map.elems) $ sequence $
+        Map.mergeWithKey
+            (\fp mmodTime fci -> Just (go fp mmodTime (Just fci)))
+            (Map.mapWithKey (\fp mmodTime -> go fp mmodTime Nothing))
+            (Map.mapWithKey (\fp fci -> go fp Nothing (Just fci)))
+            fileTimes
+            oldCache
   where
-    go fp = do
-        mmodTime <- getModTimeMaybe fp
-        case mmodTime of
-            Nothing -> return (Set.empty, Map.empty)
-            Just modTime' -> do
-                (isDirty, newFci) <-
-                    case Map.lookup fp oldCache of
-                        Just fci
-                            | fciModTime fci == modTime' -> return (False, fci)
-                            | otherwise -> do
-                                newFci <- calcFci modTime' fp
-                                let isDirty =
-                                        fciSize fci /= fciSize newFci ||
-                                        fciHash fci /= fciHash newFci
-                                return (isDirty, newFci)
-                        Nothing -> do
-                            newFci <- calcFci modTime' fp
-                            return (True, newFci)
-                return (if isDirty then Set.singleton fp else Set.empty, Map.singleton fp newFci)
+    go :: FilePath -> Maybe ModTime -> Maybe FileCacheInfo -> m (Set FilePath, Map FilePath FileCacheInfo)
+    -- Filter out the cabal_macros file to avoid spurious recompilations
+    go fp _ _ | takeFileName fp == "cabal_macros.h" = return (Set.empty, Map.empty)
+    -- Common case where it's in the cache and on the filesystem.
+    go fp (Just modTime') (Just fci)
+        | fciModTime fci == modTime' = return (Set.empty, Map.empty)
+        | otherwise = do
+            newFci <- calcFci modTime' fp
+            let isDirty =
+                    fciSize fci /= fciSize newFci ||
+                    fciHash fci /= fciHash newFci
+                newDirty = if isDirty then Set.singleton fp else Set.empty
+            return (newDirty, Map.singleton fp newFci)
+    -- Missing file. Add it to dirty files, but no FileCacheInfo.
+    go fp Nothing _ = return (Set.singleton fp, Map.empty)
+    -- Missing cache. Add it to dirty files and compute FileCacheInfo.
+    go fp (Just modTime') Nothing = do
+        newFci <- calcFci modTime' fp
+        return (Set.singleton fp, Map.singleton fp newFci)
 
 -- | Returns entries to add to the build cache for any newly found unlisted modules
 addUnlistedToBuildCache
-    :: (MonadIO m, MonadReader env m, MonadCatch m, MonadLogger m, HasEnvConfig env)
+    :: (MonadIO m, MonadReader env m, MonadMask m, MonadLogger m, HasEnvConfig env)
     => ModTime
     -> Package
     -> Path Abs File
@@ -516,7 +547,7 @@ addUnlistedToBuildCache preBuildTime pkg cabalFP buildCache = do
 
 -- | Gets list of Paths for files in a package
 getPackageFilesSimple
-    :: (MonadIO m, MonadReader env m, MonadCatch m, MonadLogger m, HasEnvConfig env)
+    :: (MonadIO m, MonadReader env m, MonadMask m, MonadLogger m, HasEnvConfig env)
     => Package -> Path Abs File -> m (Set (Path Abs File), [PackageWarning])
 getPackageFilesSimple pkg cabalFP = do
     (_,compFiles,cabalFiles,warnings) <-
@@ -532,7 +563,7 @@ getModTimeMaybe fp =
         (catch
              (liftM
                   (Just . modTime)
-                  (getModificationTime fp))
+                  (D.getModificationTime fp))
              (\e ->
                    if isDoesNotExistError e
                        then return Nothing
@@ -564,18 +595,35 @@ checkComponentsBuildable lps =
         , c <- Set.toList (lpUnbuildable lp)
         ]
 
--- | Get 'PackageConfig' for package given its name.
-getPackageConfig :: (MonadIO m, MonadThrow m, MonadCatch m, MonadLogger m, MonadReader env m, HasEnvConfig env)
-  => BuildOpts
-  -> PackageName
-  -> m PackageConfig
-getPackageConfig bopts name = do
+getDefaultPackageConfig :: (MonadIO m, MonadReader env m, HasEnvConfig env)
+  => m PackageConfig
+getDefaultPackageConfig = do
   econfig <- asks getEnvConfig
   bconfig <- asks getBuildConfig
   return PackageConfig
     { packageConfigEnableTests = False
     , packageConfigEnableBenchmarks = False
-    , packageConfigFlags = localFlags (boptsFlags bopts) bconfig name
+    , packageConfigFlags = M.empty
+    , packageConfigGhcOptions = []
+    , packageConfigCompilerVersion = envConfigCompilerVersion econfig
+    , packageConfigPlatform = configPlatform $ getConfig bconfig
+    }
+
+-- | Get 'PackageConfig' for package given its name.
+getPackageConfig :: (MonadIO m, MonadReader env m, HasEnvConfig env)
+  => BuildOptsCLI
+  -> PackageName
+  -> Bool
+  -> Bool
+  -> m PackageConfig
+getPackageConfig boptsCli name isTarget isLocal = do
+  econfig <- asks getEnvConfig
+  bconfig <- asks getBuildConfig
+  return PackageConfig
+    { packageConfigEnableTests = False
+    , packageConfigEnableBenchmarks = False
+    , packageConfigFlags = getLocalFlags bconfig boptsCli name
+    , packageConfigGhcOptions = getGhcOptions bconfig boptsCli name isTarget isLocal
     , packageConfigCompilerVersion = envConfigCompilerVersion econfig
     , packageConfigPlatform = configPlatform $ getConfig bconfig
     }
