@@ -20,21 +20,25 @@ module Stack.Build.Target
     ) where
 
 import           Control.Applicative
-import           Control.Arrow          (second)
-import           Control.Monad.Catch    (MonadThrow, throwM)
+import           Control.Arrow (second)
+import           Control.Monad.Catch (MonadCatch, throwM)
 import           Control.Monad.IO.Class
-import           Data.Either            (partitionEithers)
-import           Data.Map               (Map)
-import qualified Data.Map               as Map
-import           Data.Maybe             (mapMaybe)
-import           Data.Monoid
-import           Data.Set               (Set)
-import qualified Data.Set               as Set
-import           Data.Text              (Text)
-import qualified Data.Text              as T
+import           Data.Either (partitionEithers)
+import           Data.Foldable
+import           Data.List.Extra (groupSort)
+import           Data.List.NonEmpty (NonEmpty((:|)))
+import qualified Data.List.NonEmpty as NonEmpty
+import           Data.Map (Map)
+import qualified Data.Map as Map
+import           Data.Maybe (mapMaybe)
+import           Data.Set (Set)
+import qualified Data.Set as Set
+import           Data.Text (Text)
+import qualified Data.Text as T
 import           Path
+import           Path.Extra (rejectMissingDir)
 import           Path.IO
-import           Prelude -- Fix redundant import warnings
+import           Prelude hiding (concat, concatMap) -- Fix redundant import warnings
 import           Stack.Types
 
 -- | The name of a component, which applies to executables, test suites, and benchmarks
@@ -101,11 +105,11 @@ data LocalPackageView = LocalPackageView
     , lpvRoot       :: !(Path Abs Dir)
     , lpvCabalFP    :: !(Path Abs File)
     , lpvComponents :: !(Set NamedComponent)
-    , lpvExtraDep   :: !Bool
+    , lpvExtraDep   :: !TreatLikeExtraDep
     }
 
 -- | Same as @parseRawTarget@, but also takes directories into account.
-parseRawTargetDirs :: (MonadIO m, MonadThrow m)
+parseRawTargetDirs :: (MonadIO m, MonadCatch m)
                    => Path Abs Dir -- ^ current directory
                    -> Map PackageName LocalPackageView
                    -> Text
@@ -114,7 +118,8 @@ parseRawTargetDirs root locals t =
     case parseRawTarget t of
         Just rt -> return $ Right [(ri, rt)]
         Nothing -> do
-            mdir <- resolveDirMaybe root $ T.unpack t
+            mdir <- forgivingAbsence (resolveDir root (T.unpack t))
+              >>= rejectMissingDir
             case mdir of
                 Nothing -> return $ Left $ "Directory not found: " `T.append` t
                 Just dir ->
@@ -147,31 +152,24 @@ resolveIdents _ _ _ (ri, RTPackageComponent x y) = Right ((ri, RTPackageComponen
 resolveIdents _ _ _ (ri, RTComponent x) = Right ((ri, RTComponent x), Map.empty)
 resolveIdents _ _ _ (ri, RTPackage x) = Right ((ri, RTPackage x), Map.empty)
 resolveIdents snap extras locals (ri, RTPackageIdentifier (PackageIdentifier name version)) =
-    case mfound of
-        Just (foundPlace, foundVersion) | foundVersion /= version -> Left $ T.pack $ concat
-            [ "Specified target version "
-            , versionString version
-            , " for package "
-            , packageNameString name
-            , " does not match "
-            , foundPlace
-            , " version "
-            , versionString foundVersion
-            ]
-        _ -> Right
-            ( (ri, RTPackage name)
-            , case mfound of
-                -- Add to extra deps since we didn't have it already
-                Nothing -> Map.singleton name version
-                -- Already had it, don't add to extra deps
-                Just _ -> Map.empty
-            )
+    fmap ((ri, RTPackage name), ) newExtras
   where
-    mfound = mlocal <|> mextra <|> msnap
-
-    mlocal = (("local", ) . lpvVersion) <$> Map.lookup name locals
-    mextra = ("extra-deps", ) <$> Map.lookup name extras
-    msnap = ("snapshot", ) <$> Map.lookup name snap
+    newExtras =
+        case (Map.lookup name locals, mfound) of
+            -- Error if it matches a local package, pkg idents not
+            -- supported for local.
+            (Just _, _) -> Left $ T.concat
+                [ packageNameText name
+                , " target has a specific version number, but it is a local package."
+                , "\nTo avoid confusion, we will not install the specified version or build the local one."
+                , "\nTo build the local package, specify the target without an explicit version."
+                ]
+            -- If the found version matches, no need for an extra-dep.
+            (_, Just foundVersion) | foundVersion == version -> Right Map.empty
+            -- Otherwise, if there is no specified version or a
+            -- mismatch, add an extra-dep.
+            _ -> Right $ Map.singleton name version
+    mfound = asum (map (Map.lookup name) [extras, snap])
 
 resolveRawTarget :: Map PackageName Version -- ^ snapshot
                  -> Map PackageName Version -- ^ extra deps
@@ -247,24 +245,26 @@ isCompNamed t1 (CBench t2) = t1 == t2
 simplifyTargets :: [(PackageName, (RawInput, SimpleTarget))]
                 -> ([Text], Map PackageName SimpleTarget)
 simplifyTargets =
-    mconcat . map go . Map.toList . Map.fromListWith (++) . fmap (second return)
+    foldMap go . collect
   where
-    go :: (PackageName, [(RawInput, SimpleTarget)])
+    go :: (PackageName, NonEmpty (RawInput, SimpleTarget))
        -> ([Text], Map PackageName SimpleTarget)
-    go (_, []) = error "Stack.Build.Target.simplifyTargets: the impossible happened"
-    go (name, [(_, st)]) = ([], Map.singleton name st)
+    go (name, (_, st) :| []) = ([], Map.singleton name st)
     go (name, pairs) =
-        case partitionEithers $ map (getLocalComp . snd) pairs of
+        case partitionEithers $ map (getLocalComp . snd) (NonEmpty.toList pairs) of
             ([], comps) -> ([], Map.singleton name $ STLocalComps $ Set.unions comps)
             _ ->
                 let err = T.pack $ concat
                         [ "Overlapping targets provided for package "
                         , packageNameString name
                         , ": "
-                        , show $ map (unRawInput . fst) pairs
+                        , show $ map (unRawInput . fst) (NonEmpty.toList pairs)
                         ]
                  in ([err], Map.empty)
 
+    collect :: Ord a => [(a, b)] -> [(a, NonEmpty b)]
+    collect = map (second NonEmpty.fromList) . groupSort
+
     getLocalComp (STLocalComps comps) = Right comps
     getLocalComp _ = Left ()
 
@@ -273,7 +273,7 @@ data NeedTargets
     = NeedTargets
     | AllowNoTargets
 
-parseTargets :: (MonadThrow m, MonadIO m)
+parseTargets :: (MonadCatch m, MonadIO m)
              => NeedTargets -- ^ need at least one target
              -> Bool -- ^ using implicit global project?
              -> Map PackageName Version -- ^ snapshot
@@ -283,9 +283,10 @@ parseTargets :: (MonadThrow m, MonadIO m)
              -> [Text] -- ^ command line targets
              -> m (Map PackageName Version, Map PackageName SimpleTarget)
 parseTargets needTargets implicitGlobal snap extras locals currDir textTargets' = do
-    let textTargets =
+    let nonExtraDeps = Map.keys $ Map.filter (not . lpvExtraDep) locals
+        textTargets =
             if null textTargets'
-                then map (T.pack . packageNameString) $ Map.keys $ Map.filter (not . lpvExtraDep) locals
+                then map (T.pack . packageNameString) nonExtraDeps
                 else textTargets'
     erawTargets <- mapM (parseRawTargetDirs currDir locals) textTargets
 
@@ -302,10 +303,12 @@ parseTargets needTargets implicitGlobal snap extras locals currDir textTargets'
                  then case needTargets of
                         AllowNoTargets ->
                             return (Map.empty, Map.empty)
-                        NeedTargets ->
-                            throwM $ TargetParseException
-                              $ if implicitGlobal
-                                  then ["The specified targets matched no packages.\nPerhaps you need to run 'stack init'?"]
-                                  else ["The specified targets matched no packages"]
+                        NeedTargets
+                            | null textTargets' && implicitGlobal -> throwM $ TargetParseException
+                                ["The specified targets matched no packages.\nPerhaps you need to run 'stack init'?"]
+                            | null textTargets' && null nonExtraDeps -> throwM $ TargetParseException
+                                ["The project contains no local packages (packages not marked with 'extra-dep')"]
+                            | otherwise -> throwM $ TargetParseException
+                                ["The specified targets matched no packages"]
                  else return (Map.unions newExtras, targets)
         else throwM $ TargetParseException errs
