@@ -10,7 +10,7 @@ module Stack.ConfigCmd
        ,cfgCmdSetName
        ,cfgCmdName) where
 
-import           Control.Monad.Catch (MonadMask, throwM, MonadThrow)
+import           Control.Monad.Catch (MonadMask, throwM)
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
 import           Control.Monad.Reader (MonadReader, asks)
@@ -21,7 +21,7 @@ import qualified Data.Yaml as Yaml
 import           Network.HTTP.Client.Conduit (HasHttpManager)
 import           Path
 import           Stack.BuildPlan
-import           Stack.Init
+import           Stack.Config (makeConcreteResolver)
 import           Stack.Types
 
 data ConfigCmdSet = ConfigCmdSetResolver AbstractResolver
@@ -30,11 +30,9 @@ cfgCmdSet :: ( MonadIO m
              , MonadBaseControl IO m
              , MonadMask m
              , MonadReader env m
-             , HasConfig env
              , HasBuildConfig env
              , HasHttpManager env
              , HasGHCVariant env
-             , MonadThrow m
              , MonadLogger m)
              => ConfigCmdSet -> m ()
 cfgCmdSet (ConfigCmdSetResolver newResolver) = do
@@ -45,6 +43,7 @@ cfgCmdSet (ConfigCmdSetResolver newResolver) = do
     (projectYamlConfig :: Yaml.Object) <-
         liftIO (Yaml.decodeFileEither stackYamlFp) >>=
         either throwM return
+    -- TODO: custom snapshot support?
     newResolverText <- fmap resolverName (makeConcreteResolver newResolver)
     -- We checking here that the snapshot actually exists
     snap <- parseSnapName newResolverText
