@@ -4,26 +4,23 @@
 -- | Constants used throughout the project.
 
 module Stack.Constants
-    (builtConfigFileFromDir
-    ,builtFileFromDir
-    ,buildPlanDir
-    ,configuredFileFromDir
-    ,defaultShakeThreads
+    (buildPlanDir
     ,distDirFromDir
+    ,workDirFromDir
     ,distRelativeDir
     ,haskellModuleExts
     ,imageStagingDir
     ,projectDockerSandboxDir
-    ,rawGithubUrl
     ,stackDotYaml
     ,stackRootEnvVar
-    ,userDocsDir
+    ,stackRootOptionName
+    ,deprecatedStackRootOptionName
+    ,inContainerEnvVar
     ,configCacheFile
     ,configCabalMod
     ,buildCacheFile
     ,testSuccessFile
     ,testBuiltFile
-    ,benchBuiltFile
     ,stackProgName
     ,stackProgNameUpper
     ,wiredInPackages
@@ -49,7 +46,6 @@ import           Data.Char (toUpper)
 import           Data.HashSet (HashSet)
 import qualified Data.HashSet as HashSet
 import           Data.Text (Text)
-import qualified Data.Text as T
 import           Path as FL
 import           Prelude
 import           Stack.Types.Compiler
@@ -69,58 +65,6 @@ haskellFileExts = ["hs", "hsc", "lhs"]
 haskellPreprocessorExts :: [Text]
 haskellPreprocessorExts = ["gc", "chs", "hsc", "x", "y", "ly", "cpphs"]
 
--- | The filename used for completed build indicators.
-builtFileFromDir :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
-                 => Path Abs Dir
-                 -> m (Path Abs File)
-builtFileFromDir fp = do
-  dist <- distDirFromDir fp
-  return (dist </> $(mkRelFile "stack.gen"))
-
--- | The filename used for completed configure indicators.
-configuredFileFromDir :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
-                      => Path Abs Dir
-                      -> m (Path Abs File)
-configuredFileFromDir fp = do
-  dist <- distDirFromDir fp
-  return (dist </> $(mkRelFile "setup-config"))
-
--- | The filename used for completed build indicators.
-builtConfigFileFromDir :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
-                       => Path Abs Dir
-                       -> m (Path Abs File)
-builtConfigFileFromDir fp =
-    liftM (fp </>) builtConfigRelativeFile
-
--- | Relative location of completed build indicators.
-builtConfigRelativeFile :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
-                        => m (Path Rel File)
-builtConfigRelativeFile = do
-  dist <- distRelativeDir
-  return (dist </> $(mkRelFile "stack.config"))
-
--- | Default shake thread count for parallel builds.
-defaultShakeThreads :: Int
-defaultShakeThreads = 4
-
--- -- | Hoogle database file.
--- hoogleDatabaseFile :: Path Abs Dir -> Path Abs File
--- hoogleDatabaseFile docLoc =
---   docLoc </>
---   $(mkRelFile "default.hoo")
-
--- -- | Extension for hoogle databases.
--- hoogleDbExtension :: String
--- hoogleDbExtension = "hoo"
-
--- -- | Extension of haddock files
--- haddockExtension :: String
--- haddockExtension = "haddock"
-
--- | User documentation directory.
-userDocsDir :: Config -> Path Abs Dir
-userDocsDir config = configStackRoot config </> $(mkRelDir "doc/")
-
 -- | Output .o/.hi directory.
 objectInterfaceDir :: (MonadReader env m, HasConfig env)
   => BuildConfig -> m (Path Abs Dir)
@@ -129,7 +73,7 @@ objectInterfaceDir bconfig = do
   return (bcwd </> $(mkRelDir "odir/"))
 
 -- | The filename used for dirtiness check of source files.
-buildCacheFile :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
+buildCacheFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                => Path Abs Dir      -- ^ Package directory.
                -> m (Path Abs File)
 buildCacheFile dir =
@@ -138,7 +82,7 @@ buildCacheFile dir =
         (distDirFromDir dir)
 
 -- | The filename used to mark tests as having succeeded
-testSuccessFile :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
+testSuccessFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                 => Path Abs Dir -- ^ Package directory
                 -> m (Path Abs File)
 testSuccessFile dir =
@@ -147,7 +91,7 @@ testSuccessFile dir =
         (distDirFromDir dir)
 
 -- | The filename used to mark tests as having built
-testBuiltFile :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
+testBuiltFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
               => Path Abs Dir -- ^ Package directory
               -> m (Path Abs File)
 testBuiltFile dir =
@@ -155,17 +99,8 @@ testBuiltFile dir =
         (</> $(mkRelFile "stack-test-built"))
         (distDirFromDir dir)
 
--- | The filename used to mark benchmarks as having built
-benchBuiltFile :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
-               => Path Abs Dir -- ^ Package directory
-               -> m (Path Abs File)
-benchBuiltFile dir =
-    liftM
-        (</> $(mkRelFile "stack-bench-built"))
-        (distDirFromDir dir)
-
 -- | The filename used for dirtiness check of config.
-configCacheFile :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
+configCacheFile :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                 => Path Abs Dir      -- ^ Package directory.
                 -> m (Path Abs File)
 configCacheFile dir =
@@ -174,7 +109,7 @@ configCacheFile dir =
         (distDirFromDir dir)
 
 -- | The filename used for modification check of .cabal
-configCabalMod :: (MonadThrow m, MonadReader env m, HasPlatform env,HasEnvConfig env)
+configCabalMod :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                => Path Abs Dir      -- ^ Package directory.
                -> m (Path Abs File)
 configCabalMod dir =
@@ -184,31 +119,38 @@ configCabalMod dir =
 
 -- | Directory for HPC work.
 hpcDirFromDir
-    :: (MonadThrow m, MonadReader env m, HasPlatform env, HasEnvConfig env)
+    :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
     => Path Abs Dir  -- ^ Package directory.
     -> m (Path Abs Dir)
 hpcDirFromDir fp =
     liftM (fp </>) hpcRelativeDir
 
 -- | Relative location of directory for HPC work.
-hpcRelativeDir :: (MonadThrow m, MonadReader env m, HasPlatform env, HasEnvConfig env)
+hpcRelativeDir :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                => m (Path Rel Dir)
 hpcRelativeDir =
     liftM (</> $(mkRelDir "hpc")) distRelativeDir
 
 -- | Package's build artifacts directory.
-distDirFromDir :: (MonadThrow m, MonadReader env m, HasPlatform env, HasEnvConfig env)
+distDirFromDir :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                => Path Abs Dir
                -> m (Path Abs Dir)
 distDirFromDir fp =
     liftM (fp </>) distRelativeDir
 
+-- | Package's working directory.
+workDirFromDir :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
+               => Path Abs Dir
+               -> m (Path Abs Dir)
+workDirFromDir fp =
+    liftM (fp </>) getWorkDir
+
 -- | Directory for project templates.
 templatesDir :: Config -> Path Abs Dir
 templatesDir config = configStackRoot config </> $(mkRelDir "templates")
 
 -- | Relative location of build artifacts.
-distRelativeDir :: (MonadThrow m, MonadReader env m, HasPlatform env, HasEnvConfig env)
+distRelativeDir :: (MonadThrow m, MonadReader env m, HasEnvConfig env)
                 => m (Path Rel Dir)
 distRelativeDir = do
     cabalPkgVer <- asks (envConfigCabalVersion . getEnvConfig)
@@ -227,37 +169,6 @@ distRelativeDir = do
         $(mkRelDir "dist") </>
         platformAndCabal
 
--- | Get a URL for a raw file on Github
-rawGithubUrl :: Text -- ^ user/org name
-             -> Text -- ^ repo name
-             -> Text -- ^ branch name
-             -> Text -- ^ filename
-             -> Text
-rawGithubUrl org repo branch file = T.concat
-    [ "https://raw.githubusercontent.com/"
-    , org
-    , "/"
-    , repo
-    , "/"
-    , branch
-    , "/"
-    , file
-    ]
-
--- -- | Hoogle database file.
--- hoogleDatabaseFile :: Path Abs Dir -> Path Abs File
--- hoogleDatabaseFile docLoc =
---   docLoc </>
---   $(mkRelFile "default.hoo")
-
--- -- | Extension for hoogle databases.
--- hoogleDbExtension :: String
--- hoogleDbExtension = "hoo"
-
--- -- | Extension of haddock files
--- haddockExtension :: String
--- haddockExtension = "haddock"
-
 -- | Docker sandbox from project root.
 projectDockerSandboxDir :: (MonadReader env m, HasConfig env)
   => Path Abs Dir      -- ^ Project root
@@ -267,12 +178,14 @@ projectDockerSandboxDir projectRoot = do
   return $ projectRoot </> workDir </> $(mkRelDir "docker/")
 
 -- | Image staging dir from project root.
-imageStagingDir :: (MonadReader env m, HasConfig env)
+imageStagingDir :: (MonadReader env m, HasConfig env, MonadThrow m)
   => Path Abs Dir      -- ^ Project root
+  -> Int               -- ^ Index of image
   -> m (Path Abs Dir)  -- ^ Docker sandbox
-imageStagingDir projectRoot = do
+imageStagingDir projectRoot imageIdx = do
   workDir <- getWorkDir
-  return $ projectRoot </> workDir </> $(mkRelDir "image/")
+  idxRelDir <- parseRelDir (show imageIdx)
+  return $ projectRoot </> workDir </> $(mkRelDir "image") </> idxRelDir
 
 -- | Name of the 'stack' program, uppercased
 stackProgNameUpper :: String
@@ -290,6 +203,23 @@ stackDotYaml = $(mkRelFile "stack.yaml")
 stackRootEnvVar :: String
 stackRootEnvVar = "STACK_ROOT"
 
+-- | Option name for the global stack root.
+stackRootOptionName :: String
+stackRootOptionName = "stack-root"
+
+-- | Deprecated option name for the global stack root.
+--
+-- Deprecated since stack-1.1.0.
+--
+-- TODO: Remove occurences of this variable and use 'stackRootOptionName' only
+-- after an appropriate deprecation period.
+deprecatedStackRootOptionName :: String
+deprecatedStackRootOptionName = "global-stack-root"
+
+-- | Environment variable used to indicate stack is running in container.
+inContainerEnvVar :: String
+inContainerEnvVar = stackProgNameUpper ++ "_IN_CONTAINER"
+
 -- See https://downloads.haskell.org/~ghc/7.10.1/docs/html/libraries/ghc/src/Module.html#integerPackageKey
 wiredInPackages :: HashSet PackageName
 wiredInPackages =
