@@ -4,6 +4,7 @@
 {-# LANGUAGE OverloadedStrings     #-}
 {-# LANGUAGE TemplateHaskell       #-}
 {-# LANGUAGE FlexibleInstances     #-}
+{-# LANGUAGE LambdaCase            #-}
 -- | Generate HPC (Haskell Program Coverage) reports
 module Stack.Coverage
     ( deleteHpcReports
@@ -18,7 +19,7 @@ module Stack.Coverage
 import           Control.Applicative
 import           Control.Exception.Lifted
 import           Control.Monad (liftM, when, unless, void)
-import           Control.Monad.Catch (MonadCatch)
+import           Control.Monad.Catch (MonadMask)
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
 import           Control.Monad.Reader (MonadReader, asks)
@@ -37,7 +38,6 @@ import qualified Data.Text.Encoding as T
 import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy as LT
 import           Data.Traversable (forM)
-import           Network.HTTP.Download (HasHttpManager)
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO
@@ -47,29 +47,28 @@ import           Stack.Build.Target
 import           Stack.Constants
 import           Stack.Package
 import           Stack.Types
-import qualified System.Directory as D
 import           System.FilePath (isPathSeparator)
 import           System.Process.Read
 import           Text.Hastache (htmlEscape)
 import           Trace.Hpc.Tix
 
 -- | Invoked at the beginning of running with "--coverage"
-deleteHpcReports :: (MonadIO m, MonadThrow m, MonadReader env m, HasEnvConfig env)
+deleteHpcReports :: (MonadIO m, MonadMask m, MonadReader env m, HasEnvConfig env)
                  => m ()
 deleteHpcReports = do
     hpcDir <- hpcReportDir
-    removeTreeIfExists hpcDir
+    ignoringAbsence (removeDirRecur hpcDir)
 
 -- | Move a tix file into a sub-directory of the hpc report directory. Deletes the old one if one is
 -- present.
-updateTixFile :: (MonadIO m,MonadReader env m,HasConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,HasEnvConfig env)
+updateTixFile :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
             => PackageName -> Path Abs File -> String -> m ()
 updateTixFile pkgName tixSrc testName = do
-    exists <- fileExists tixSrc
+    exists <- doesFileExist tixSrc
     when exists $ do
         tixDest <- tixFilePath pkgName testName
-        removeFileIfExists tixDest
-        createTree (parent tixDest)
+        ignoringAbsence (removeFile tixDest)
+        ensureDir (parent tixDest)
         -- Remove exe modules because they are problematic. This could be revisited if there's a GHC
         -- version that fixes https://ghc.haskell.org/trac/ghc/ticket/1853
         mtix <- readTixOrLog tixSrc
@@ -77,10 +76,10 @@ updateTixFile pkgName tixSrc testName = do
             Nothing -> $logError $ "Failed to read " <> T.pack (toFilePath tixSrc)
             Just tix -> do
                 liftIO $ writeTix (toFilePath tixDest) (removeExeModules tix)
-                removeFileIfExists tixSrc
+                ignoringAbsence (removeFile tixSrc)
 
 -- | Get the directory used for hpc reports for the given pkgId.
-hpcPkgPath :: (MonadIO m,MonadReader env m,HasConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,HasEnvConfig env)
+hpcPkgPath :: (MonadIO m,MonadReader env m,MonadMask m,HasEnvConfig env)
             => PackageName -> m (Path Abs Dir)
 hpcPkgPath pkgName = do
     outputDir <- hpcReportDir
@@ -89,7 +88,7 @@ hpcPkgPath pkgName = do
 
 -- | Get the tix file location, given the name of the file (without extension), and the package
 -- identifier string.
-tixFilePath :: (MonadIO m,MonadReader env m,HasConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,HasEnvConfig env)
+tixFilePath :: (MonadIO m,MonadReader env m,MonadMask m,HasEnvConfig env)
             => PackageName -> String ->  m (Path Abs File)
 tixFilePath pkgName testName = do
     pkgPath <- hpcPkgPath pkgName
@@ -97,7 +96,7 @@ tixFilePath pkgName testName = do
     return (pkgPath </> tixRel)
 
 -- | Generates the HTML coverage report and shows a textual coverage summary for a package.
-generateHpcReport :: (MonadIO m,MonadReader env m,HasConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,HasEnvConfig env)
+generateHpcReport :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
                   => Path Abs Dir -> Package -> [Text] -> m ()
 generateHpcReport pkgDir package tests = do
     -- If we're using > GHC 7.10, the hpc 'include' parameter must specify a ghc package key. See
@@ -134,11 +133,11 @@ generateHpcReport pkgDir package tests = do
                         Nothing -> []
                 generateHpcReportInternal tixSrc reportDir report extraArgs extraArgs
 
-generateHpcReportInternal :: (MonadIO m,MonadReader env m,HasConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,HasEnvConfig env)
+generateHpcReportInternal :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
                           => Path Abs File -> Path Abs Dir -> Text -> [String] -> [String] -> m ()
 generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArgs = do
     -- If a .tix file exists, move it to the HPC output directory and generate a report for it.
-    tixFileExists <- fileExists tixSrc
+    tixFileExists <- doesFileExist tixSrc
     if not tixFileExists
         then $logError $ T.concat
             [ "Didn't find .tix for "
@@ -163,7 +162,8 @@ generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArg
                     ["--hpcdir", toFilePathNoTrailingSep hpcRelDir, "--reset-hpcdirs"]
             menv <- getMinimalEnvOverride
             $logInfo $ "Generating " <> report
-            outputLines <- liftM S8.lines $ readProcessStdout Nothing menv "hpc"
+            outputLines <- liftM (map (S8.filter (/= '\r')) . S8.lines) $
+                readProcessStdout Nothing menv "hpc"
                 ( "report"
                 : toFilePath tixSrc
                 : (args ++ extraReportArgs)
@@ -186,7 +186,7 @@ generateHpcReportInternal tixSrc reportDir report extraMarkupArgs extraReportArg
                     generateHpcErrorReport reportDir (msg True)
                 else do
                     -- Print output, stripping @\r@ characters because Windows.
-                    forM_ outputLines ($logInfo . T.decodeUtf8 . S8.filter (/= '\r'))
+                    forM_ outputLines ($logInfo . T.decodeUtf8)
                     $logInfo
                         ("The " <> report <> " is available at " <>
                          T.pack (toFilePath (reportDir </> $(mkRelFile "hpc_index.html"))))
@@ -204,7 +204,7 @@ data HpcReportOpts = HpcReportOpts
     , hroptsDestDir :: Maybe String
     } deriving (Show)
 
-generateHpcReportForTargets :: (MonadIO m, HasHttpManager env, MonadReader env m, MonadBaseControl IO m, MonadCatch m, MonadLogger m, HasEnvConfig env)
+generateHpcReportForTargets :: (MonadIO m, MonadReader env m, MonadBaseControl IO m, MonadMask m, MonadLogger m, HasEnvConfig env)
                             => HpcReportOpts -> m ()
 generateHpcReportForTargets opts = do
     let (tixFiles, targetNames) = partition (".tix" `T.isSuffixOf`) (hroptsInputs opts)
@@ -218,9 +218,8 @@ generateHpcReportForTargets opts = do
                  $logWarn $ "Since --all is used, it is redundant to specify these targets: " <> T.pack (show targetNames)
              (_,_,targets) <- parseTargetsFromBuildOpts
                  AllowNoTargets
-                 defaultBuildOpts
-                 { boptsTargets = if hroptsAll opts then [] else targetNames
-                 }
+                 defaultBuildOptsCLI
+                    { boptsCLITargets = if hroptsAll opts then [] else targetNames }
              liftM concat $ forM (Map.toList targets) $ \(name, target) ->
                  case target of
                      STUnknown -> fail $
@@ -241,32 +240,33 @@ generateHpcReportForTargets opts = do
                                      " is used with a non test-suite target)"
                      STLocalAll -> do
                          pkgPath <- hpcPkgPath name
-                         exists <- dirExists pkgPath
+                         exists <- doesDirExist pkgPath
                          if exists
                              then do
-                                 (_, files) <- listDirectory pkgPath
+                                 (_, files) <- listDir pkgPath
                                  return (filter ((".tix" `isSuffixOf`) . toFilePath) files)
                              else return []
-    tixPaths <- liftM (++ targetTixFiles) $ mapM (parseRelAsAbsFile . T.unpack) tixFiles
+    tixPaths <- liftM (++ targetTixFiles) $ mapM (resolveFile' . T.unpack) tixFiles
     when (null tixPaths) $
         fail "Not generating combined report, because no targets or tix files are specified."
     reportDir <- case hroptsDestDir opts of
         Nothing -> liftM (</> $(mkRelDir "combined/custom")) hpcReportDir
         Just destDir -> do
-            liftIO $ D.createDirectoryIfMissing True destDir
-            parseRelAsAbsDir destDir
+            dest <- resolveDir' destDir
+            ensureDir dest
+            return dest
     generateUnionReport "combined report" reportDir tixPaths
 
-generateHpcUnifiedReport :: (MonadIO m,MonadReader env m,HasConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,HasEnvConfig env)
+generateHpcUnifiedReport :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
                          => m ()
 generateHpcUnifiedReport = do
     outputDir <- hpcReportDir
-    createTree outputDir
-    (dirs, _) <- listDirectory outputDir
+    ensureDir outputDir
+    (dirs, _) <- listDir outputDir
     tixFiles <- liftM (concat . concat) $ forM (filter (("combined" /=) . dirnameString) dirs) $ \dir -> do
-        (dirs', _) <- listDirectory dir
+        (dirs', _) <- listDir dir
         forM dirs' $ \dir' -> do
-            (_, files) <- listDirectory dir'
+            (_, files) <- listDir dir'
             return (filter ((".tix" `isSuffixOf`) . toFilePath) files)
     let reportDir = outputDir </> $(mkRelDir "combined/all")
     if length tixFiles < 2
@@ -278,7 +278,7 @@ generateHpcUnifiedReport = do
             ]
         else generateUnionReport "unified report" reportDir tixFiles
 
-generateUnionReport :: (MonadIO m,MonadReader env m,HasConfig env,MonadLogger m,MonadBaseControl IO m,MonadCatch m,HasEnvConfig env)
+generateUnionReport :: (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadMask m,HasEnvConfig env)
                     => Text -> Path Abs Dir -> [Path Abs File] -> m ()
 generateUnionReport report reportDir tixFiles = do
     (errs, tix) <- fmap (unionTixes . map removeExeModules) (mapMaybeM readTixOrLog tixFiles)
@@ -287,14 +287,14 @@ generateUnionReport report reportDir tixFiles = do
         "The following modules are left out of the " : report : " due to version mismatches: " :
         intersperse ", " (map T.pack errs)
     tixDest <- liftM (reportDir </>) $ parseRelFile (dirnameString reportDir ++ ".tix")
-    createTree (parent tixDest)
+    ensureDir (parent tixDest)
     liftIO $ writeTix (toFilePath tixDest) tix
     generateHpcReportInternal tixDest reportDir report [] []
 
 readTixOrLog :: (MonadLogger m, MonadIO m, MonadBaseControl IO m) => Path b File -> m (Maybe Tix)
 readTixOrLog path = do
-    mtix <- liftIO (readTix (toFilePath path)) `catch` \(ErrorCall err) -> do
-        $logError $ "Error while reading tix: " <> T.pack err
+    mtix <- liftIO (readTix (toFilePath path)) `catch` \errorCall -> do
+        $logError $ "Error while reading tix: " <> T.pack (show (errorCall :: ErrorCall))
         return Nothing
     when (isNothing mtix) $
         $logError $ "Failed to read tix file " <> T.pack (toFilePath path)
@@ -315,18 +315,18 @@ unionTixes tixes = (Map.keys errs, Tix (Map.elems outputs))
         | hash1 == hash2 && len1 == len2 = Right (TixModule k hash1 len1 (zipWith (+) tix1 tix2))
     merge _ _ = Left ()
 
-generateHpcMarkupIndex :: (MonadIO m,MonadReader env m,MonadLogger m,MonadCatch m,HasEnvConfig env)
+generateHpcMarkupIndex :: (MonadIO m,MonadReader env m,MonadLogger m,MonadMask m,HasEnvConfig env)
                        => m ()
 generateHpcMarkupIndex = do
     outputDir <- hpcReportDir
     let outputFile = outputDir </> $(mkRelFile "index.html")
-    createTree outputDir
-    (dirs, _) <- listDirectory outputDir
+    ensureDir outputDir
+    (dirs, _) <- listDir outputDir
     rows <- liftM (catMaybes . concat) $ forM dirs $ \dir -> do
-        (subdirs, _) <- listDirectory dir
+        (subdirs, _) <- listDir dir
         forM subdirs $ \subdir -> do
             let indexPath = subdir </> $(mkRelFile "hpc_index.html")
-            exists' <- fileExists indexPath
+            exists' <- doesFileExist indexPath
             if not exists' then return Nothing else do
                 relPath <- stripDir outputDir indexPath
                 let package = dirname dir
@@ -371,7 +371,7 @@ generateHpcMarkupIndex = do
 
 generateHpcErrorReport :: MonadIO m => Path Abs Dir -> Text -> m ()
 generateHpcErrorReport dir err = do
-    createTree dir
+    ensureDir dir
     liftIO $ T.writeFile (toFilePath (dir </> $(mkRelFile "hpc_index.html"))) $ T.concat
         [ "<html><head><meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\"></head><body>"
         , "<h1>HPC Report Generation Error</h1>"
@@ -396,7 +396,7 @@ findPackageKeyForBuiltPackage pkgDir pkgId = do
     distDir <- distDirFromDir pkgDir
     path <- liftM (distDir </>) $
         parseRelFile ("package.conf.inplace/" ++ packageIdentifierString pkgId ++ "-inplace.conf")
-    exists <- fileExists path
+    exists <- doesFileExist path
     if exists
         then do
             contents <- liftIO $ T.readFile (toFilePath path)
