@@ -56,22 +56,24 @@ import           GHC.Exts (sortWith)
 import           Network.HTTP.Client.Conduit (HasHttpManager)
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
-import           Path.IO
+import           Path.IO hiding (canonicalizePath)
 import qualified Paths_stack as Meta
 import           Prelude -- Fix redundant import warnings
+import           Stack.Config (getInContainer)
 import           Stack.Constants
 import           Stack.Docker.GlobalDB
 import           Stack.Types
 import           Stack.Types.Internal
 import           Stack.Setup (ensureDockerStackExe)
-import           System.Directory (canonicalizePath,getModificationTime)
-import           System.Environment (getEnv,getProgName,getArgs,getExecutablePath,lookupEnv)
+import           System.Directory (canonicalizePath,getHomeDirectory)
+import           System.Environment (getEnv,getEnvironment,getProgName,getArgs,getExecutablePath)
 import           System.Exit (exitSuccess, exitWith)
 import qualified System.FilePath as FP
 import           System.IO (stderr,stdin,stdout,hIsTerminalDevice)
 import           System.IO.Error (isDoesNotExistError)
 import           System.IO.Unsafe (unsafePerformIO)
 import qualified System.PosixCompat.User as User
+import qualified System.PosixCompat.Files as Files
 import           System.Process.PagerEditor (editByteString)
 import           System.Process.Read
 import           System.Process.Run
@@ -82,6 +84,7 @@ import           Text.Printf (printf)
 import           Control.Concurrent (threadDelay)
 import           Control.Monad.Trans.Control (liftBaseWith)
 import           System.Posix.Signals
+import qualified System.Posix.User as PosixUser
 #endif
 
 -- | If Docker is enabled, re-runs the currently running OS command in a Docker container.
@@ -104,10 +107,16 @@ reexecWithOptionalContainer mprojectRoot =
   where
     getCmdArgs docker envOverride imageInfo isRemoteDocker = do
         config <- asks getConfig
-        deUidGid <-
+        deUser <-
             if fromMaybe (not isRemoteDocker) (dockerSetUser docker)
-                then liftIO $ Just <$>
-                  ((,) <$> User.getEffectiveUserID <*> User.getEffectiveGroupID)
+                then liftIO $ do
+                  duUid <- User.getEffectiveUserID
+                  duGid <- User.getEffectiveGroupID
+                  duGroups <- User.getGroups
+                  duUmask <- Files.setFileCreationMask 0o022
+                  -- Only way to get old umask seems to be to change it, so set it back afterward
+                  _ <- Files.setFileCreationMask duUmask
+                  return (Just DockerUser{..})
                 else return Nothing
         args <-
             fmap
@@ -133,7 +142,7 @@ reexecWithOptionalContainer mprojectRoot =
                   (exePath,exeTimestamp,misCompatible) <-
                       liftIO $
                       do exePath <- liftIO getExecutablePath
-                         exeTimestamp <- liftIO (getModificationTime exePath)
+                         exeTimestamp <- resolveFile' exePath >>= getModificationTime
                          isKnown <-
                              liftIO $
                              getDockerImageExe
@@ -229,10 +238,6 @@ preventInContainer inner =
         then throwM OnlyOnHostException
         else inner
 
--- | 'True' if we are currently running inside a Docker container.
-getInContainer :: (MonadIO m) => m Bool
-getInContainer = liftIO (isJust <$> lookupEnv inContainerEnvVar)
-
 -- | Run a command in a new Docker container, then exit the process.
 runContainerAndExit :: M env m
   => GetCmdArgs env m
@@ -248,20 +253,24 @@ runContainerAndExit getCmdArgs
      let docker = configDocker config
      envOverride <- getEnvOverride (configPlatform config)
      checkDockerVersion envOverride docker
-     (dockerHost,dockerCertPath,bamboo,jenkins) <-
-       liftIO ((,,,) <$> lookupEnv "DOCKER_HOST"
-                     <*> lookupEnv "DOCKER_CERT_PATH"
-                     <*> lookupEnv "bamboo_buildKey"
-                     <*> lookupEnv "JENKINS_HOME")
-     let isRemoteDocker = maybe False (isPrefixOf "tcp://") dockerHost
+     (env,isStdinTerminal,isStderrTerminal,homeDir) <- liftIO $
+       (,,,)
+       <$> getEnvironment
+       <*> hIsTerminalDevice stdin
+       <*> hIsTerminalDevice stderr
+       <*> (parseAbsDir =<< getHomeDirectory)
      isStdoutTerminal <- asks getTerminal
-     (isStdinTerminal,isStderrTerminal) <-
-       liftIO ((,) <$> hIsTerminalDevice stdin
-                   <*> hIsTerminalDevice stderr)
+     let dockerHost = lookup "DOCKER_HOST" env
+         dockerCertPath = lookup "DOCKER_CERT_PATH" env
+         bamboo = lookup "bamboo_buildKey" env
+         jenkins = lookup "JENKINS_HOME" env
+         msshAuthSock = lookup "SSH_AUTH_SOCK" env
+         muserEnv = lookup "USER" env
+         isRemoteDocker = maybe False (isPrefixOf "tcp://") dockerHost
+         image = dockerImage docker
      when (isRemoteDocker &&
            maybe False (isInfixOf "boot2docker") dockerCertPath)
           ($logWarn "Warning: Using boot2docker is NOT supported, and not likely to perform well.")
-     let image = dockerImage docker
      maybeImageInfo <- inspect envOverride image
      imageInfo@Inspect{..} <- case maybeImageInfo of
        Just ii -> return ii
@@ -294,10 +303,23 @@ runContainerAndExit getCmdArgs
                                             </> $(mkRelDir ".local/bin")]
                       (T.pack <$> lookupImageEnv "PATH" imageEnvVars)
      (cmnd,args,envVars,extraMount) <- getCmdArgs docker envOverride imageInfo isRemoteDocker
-     pwd <- getWorkingDir
+     pwd <- getCurrentDir
      liftIO
        (do updateDockerImageLastUsed config iiId (toFilePath projectRoot)
-           mapM_ createTree [sandboxHomeDir, stackRoot])
+           mapM_ (ensureDir) [sandboxHomeDir, stackRoot])
+     -- Since $HOME is now mounted in the same place in the container we can
+     -- just symlink $HOME/.ssh to the right place for the stack docker user
+     let sshDir = homeDir </> sshRelDir
+     sshDirExists <- doesDirExist sshDir
+     sshSandboxDirExists <-
+         liftIO
+             (Files.fileExist
+                 (toFilePathNoTrailingSep (sandboxHomeDir </> sshRelDir)))
+     when (sshDirExists && not sshSandboxDirExists)
+         (liftIO
+             (Files.createSymbolicLink
+                 (toFilePathNoTrailingSep sshDir)
+                 (toFilePathNoTrailingSep (sandboxHomeDir </> sshRelDir))))
      containerID <- (trim . decodeUtf8) <$> readDockerProcess
        envOverride
        (concat
@@ -308,10 +330,20 @@ runContainerAndExit getCmdArgs
           ,"-e",platformVariantEnvVar ++ "=dk" ++ platformVariant
           ,"-e","HOME=" ++ toFilePathNoTrailingSep sandboxHomeDir
           ,"-e","PATH=" ++ T.unpack newPathEnv
+          ,"-e","PWD=" ++ toFilePathNoTrailingSep pwd
+          ,"-v",toFilePathNoTrailingSep homeDir ++ ":" ++ toFilePathNoTrailingSep homeDir
           ,"-v",toFilePathNoTrailingSep stackRoot ++ ":" ++ toFilePathNoTrailingSep stackRoot
           ,"-v",toFilePathNoTrailingSep projectRoot ++ ":" ++ toFilePathNoTrailingSep projectRoot
           ,"-v",toFilePathNoTrailingSep sandboxHomeDir ++ ":" ++ toFilePathNoTrailingSep sandboxHomeDir
           ,"-w",toFilePathNoTrailingSep pwd]
+         ,case muserEnv of
+            Nothing -> []
+            Just userEnv -> ["-e","USER=" ++ userEnv]
+         ,case msshAuthSock of
+            Nothing -> []
+            Just sshAuthSock ->
+              ["-e","SSH_AUTH_SOCK=" ++ sshAuthSock
+              ,"-v",sshAuthSock ++ ":" ++ sshAuthSock]
            -- Disable the deprecated entrypoint in FP Complete-generated images
          ,["--entrypoint=/usr/bin/env"
              | isJust (lookupImageEnv oldSandboxIdEnvVar imageEnvVars) &&
@@ -378,6 +410,7 @@ runContainerAndExit getCmdArgs
         _ -> Nothing
     mountArg (Mount host container) = ["-v",host ++ ":" ++ container]
     projectRoot = fromMaybeProjectRoot mprojectRoot
+    sshRelDir = $(mkRelDir ".ssh/")
 
 -- | Clean-up old docker images and containers.
 cleanup :: M env m
@@ -607,7 +640,7 @@ cleanup opts =
     containerStr = "container"
 
 -- | Inspect Docker image or container.
-inspect :: (MonadIO m,MonadThrow m,MonadLogger m,MonadBaseControl IO m,MonadCatch m)
+inspect :: (MonadIO m,MonadLogger m,MonadBaseControl IO m,MonadCatch m)
         => EnvOverride -> String -> m (Maybe Inspect)
 inspect envOverride image =
   do results <- inspects envOverride [image]
@@ -617,7 +650,7 @@ inspect envOverride image =
        _ -> throwM (InvalidInspectOutputException "expect a single result")
 
 -- | Inspect multiple Docker images and/or containers.
-inspects :: (MonadIO m, MonadThrow m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+inspects :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
          => EnvOverride -> [String] -> m (Map String Inspect)
 inspects _ [] = return Map.empty
 inspects envOverride images =
@@ -665,7 +698,7 @@ pullImage envOverride docker image =
 
 -- | Check docker version (throws exception if incorrect)
 checkDockerVersion
-    :: (MonadIO m, MonadThrow m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
+    :: (MonadIO m, MonadLogger m, MonadBaseControl IO m, MonadCatch m)
     => EnvOverride -> DockerOpts -> m ()
 checkDockerVersion envOverride docker =
   do dockerExists <- doesExecutableExist envOverride "docker"
@@ -673,7 +706,7 @@ checkDockerVersion envOverride docker =
      dockerVersionOut <- readDockerProcess envOverride ["--version"]
      case words (decodeUtf8 dockerVersionOut) of
        (_:_:v:_) ->
-         case parseVersionFromString (dropWhileEnd (not . isDigit) v) of
+         case parseVersionFromString (stripVersion v) of
            Just v'
              | v' < minimumDockerVersion ->
                throwM (DockerTooOldException minimumDockerVersion v')
@@ -687,6 +720,7 @@ checkDockerVersion envOverride docker =
        _ -> throwM InvalidVersionOutputException
   where minimumDockerVersion = $(mkVersion "1.6.0")
         prohibitedDockerVersions = []
+        stripVersion v = fst $ break (== '-') $ dropWhileEnd (not . isDigit) v
 
 -- | Remove the project's Docker sandbox.
 reset :: (MonadIO m, MonadReader env m, HasConfig env)
@@ -713,10 +747,10 @@ entrypoint config@Config{..} DockerEntrypoint{..} =
       estackUserEntry0 <- liftIO $ tryJust (guard . isDoesNotExistError) $
         User.getUserEntryForName stackUserName
       -- Switch UID/GID if needed, and update user's home directory
-      case deUidGid of
+      case deUser of
         Nothing -> return ()
-        Just (0,_) -> return ()
-        Just (uid,gid) -> updateOrCreateStackUser envOverride estackUserEntry0 homeDir uid gid
+        Just (DockerUser 0 _ _ _) -> return ()
+        Just du -> updateOrCreateStackUser envOverride estackUserEntry0 homeDir du
       case estackUserEntry0 of
         Left _ -> return ()
         Right ue -> do
@@ -724,19 +758,19 @@ entrypoint config@Config{..} DockerEntrypoint{..} =
           -- its original home directory to the host's stack root, to avoid needing to download them
           origStackHomeDir <- parseAbsDir (User.homeDirectory ue)
           let origStackRoot = origStackHomeDir </> $(mkRelDir ("." ++ stackProgName))
-          buildPlanDirExists <- dirExists (buildPlanDir origStackRoot)
+          buildPlanDirExists <- doesDirExist (buildPlanDir origStackRoot)
           when buildPlanDirExists $ do
-            (_, buildPlans) <- listDirectory (buildPlanDir origStackRoot)
+            (_, buildPlans) <- listDir (buildPlanDir origStackRoot)
             forM_ buildPlans $ \srcBuildPlan -> do
               let destBuildPlan = buildPlanDir configStackRoot </> filename srcBuildPlan
-              exists <- fileExists destBuildPlan
+              exists <- doesFileExist destBuildPlan
               unless exists $ do
-                createTree (parent destBuildPlan)
+                ensureDir (parent destBuildPlan)
                 copyFile srcBuildPlan destBuildPlan
           forM_ configPackageIndices $ \pkgIdx -> do
             msrcIndex <- flip runReaderT (config{configStackRoot = origStackRoot}) $ do
                srcIndex <- configPackageIndex (indexName pkgIdx)
-               exists <- fileExists srcIndex
+               exists <- doesFileExist srcIndex
                return $ if exists
                  then Just srcIndex
                  else Nothing
@@ -745,41 +779,51 @@ entrypoint config@Config{..} DockerEntrypoint{..} =
               Just srcIndex -> do
                 flip runReaderT config $ do
                   destIndex <- configPackageIndex (indexName pkgIdx)
-                  exists <- fileExists destIndex
+                  exists <- doesFileExist destIndex
                   unless exists $ do
-                    createTree (parent destIndex)
+                    ensureDir (parent destIndex)
                     copyFile srcIndex destIndex
     return True
   where
-    updateOrCreateStackUser envOverride estackUserEntry homeDir uid gid = do
+    updateOrCreateStackUser envOverride estackUserEntry homeDir DockerUser{..} = do
       case estackUserEntry of
         Left _ -> do
           -- If no 'stack' user in image, create one with correct UID/GID and home directory
           readProcessNull Nothing envOverride "groupadd"
             ["-o"
-            ,"--gid",show gid
+            ,"--gid",show duGid
             ,stackUserName]
           readProcessNull Nothing envOverride "useradd"
             ["-oN"
-            ,"--uid",show uid
-            ,"--gid",show gid
+            ,"--uid",show duUid
+            ,"--gid",show duGid
             ,"--home",toFilePathNoTrailingSep homeDir
             ,stackUserName]
         Right _ -> do
-          -- If there is already a 'stack' user in thr image, adjust its UID/GID and home directory
+          -- If there is already a 'stack' user in the image, adjust its UID/GID and home directory
           readProcessNull Nothing envOverride "usermod"
             ["-o"
-            ,"--uid",show uid
+            ,"--uid",show duUid
             ,"--home",toFilePathNoTrailingSep homeDir
             ,stackUserName]
           readProcessNull Nothing envOverride "groupmod"
             ["-o"
-            ,"--gid",show gid
+            ,"--gid",show duGid
             ,stackUserName]
+      forM_ duGroups $ \gid -> do
+        readProcessNull Nothing envOverride "groupadd"
+          ["-o"
+          ,"--gid",show gid
+          ,"group" ++ show gid]
       -- 'setuid' to the wanted UID and GID
       liftIO $ do
-        User.setGroupID gid
-        User.setUserID uid
+        User.setGroupID duGid
+#ifndef WINDOWS
+        PosixUser.setGroups duGroups
+#endif
+        User.setUserID duUid
+        _ <- Files.setFileCreationMask duUmask
+        return ()
     stackUserName = "stack"::String
 
 -- | MVar used to ensure the Docker entrypoint is performed exactly once
@@ -795,12 +839,12 @@ removeDirectoryContents :: Path Abs Dir -- ^ Directory to remove contents of
                         -> [Path Rel File] -- ^ Top-level file names to exclude from removal
                         -> IO ()
 removeDirectoryContents path excludeDirs excludeFiles =
-  do isRootDir <- dirExists path
+  do isRootDir <- doesDirExist path
      when isRootDir
-          (do (lsd,lsf) <- listDirectory path
+          (do (lsd,lsf) <- listDir path
               forM_ lsd
                     (\d -> unless (dirname d `elem` excludeDirs)
-                                  (removeTree d))
+                                  (removeDirRecur d))
               forM_ lsf
                     (\f -> unless (filename f `elem` excludeFiles)
                                   (removeFile f)))
@@ -838,10 +882,6 @@ fromMaybeProjectRoot = fromMaybe (throw CannotDetermineProjectRootException)
 oldSandboxIdEnvVar :: String
 oldSandboxIdEnvVar = "DOCKER_SANDBOX_ID"
 
--- | Environment variable used to indicate stack is running in container.
-inContainerEnvVar :: String
-inContainerEnvVar = stackProgNameUpper ++ "_IN_CONTAINER"
-
 -- | Command-line argument for "docker"
 dockerCmdName :: String
 dockerCmdName = "docker"
@@ -909,8 +949,8 @@ instance FromJSON ImageConfig where
   parseJSON v =
     do o <- parseJSON v
        ImageConfig
-         <$> o .:? "Env" .!= []
-         <*> o .:? "Entrypoint" .!= []
+         <$> fmap join (o .:? "Env") .!= []
+         <*> fmap join (o .:? "Entrypoint") .!= []
 
 -- | Exceptions thrown by Stack.Docker.
 data StackDockerException
@@ -1050,6 +1090,5 @@ type GetCmdArgs env m
   -> Bool
   -> m (FilePath,[String],[(String,String)],[Mount])
 
-
 type M env m = (MonadIO m,MonadReader env m,MonadLogger m,MonadBaseControl IO m,MonadCatch m
                ,HasConfig env,HasTerminal env,HasReExec env,HasHttpManager env,MonadMask m)
