@@ -16,7 +16,7 @@ import           Control.Monad.Catch (MonadCatch,MonadMask)
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger (MonadLogger)
 import           Control.Monad.Reader (MonadReader)
-import           Control.Monad.Trans.Control (MonadBaseControl)
+import           Control.Monad.Trans.Unlift (MonadBaseUnlift)
 import qualified Data.Foldable as F
 import qualified Data.HashSet as HashSet
 import           Data.Map (Map)
@@ -55,7 +55,7 @@ data DotOpts = DotOpts
 dot :: (HasEnvConfig env
        ,HasHttpManager env
        ,HasLogLevel env
-       ,MonadBaseControl IO m
+       ,MonadBaseUnlift IO m
        ,MonadCatch m
        ,MonadLogger m
        ,MonadIO m
@@ -81,15 +81,14 @@ createDependencyGraph :: (HasEnvConfig env
                          ,HasHttpManager env
                          ,HasLogLevel env
                          ,MonadLogger m
-                         ,MonadBaseControl IO m
-                         ,MonadCatch m
+                         ,MonadBaseUnlift IO m
                          ,MonadIO m
                          ,MonadMask m
                          ,MonadReader env m)
                        => DotOpts
                        -> m (Map PackageName (Set PackageName, Maybe Version))
 createDependencyGraph dotOpts = do
-  (_,_,locals,_,sourceMap) <- loadSourceMap NeedTargets defaultBuildOpts
+  (_,_,locals,_,sourceMap) <- loadSourceMap NeedTargets defaultBuildOptsCLI
   let graph = Map.fromList (localDependencies dotOpts locals)
   menv <- getMinimalEnvOverride
   installedMap <- fmap snd . fst4 <$> getInstalled menv
@@ -99,11 +98,11 @@ createDependencyGraph dotOpts = do
     let depLoader =
           createDepLoader sourceMap
                           installedMap
-                          (fmap3 (packageAllDeps &&& (Just . packageVersion)) loader)
+                          (fmap4 (packageAllDeps &&& (Just . packageVersion)) loader)
     liftIO $ resolveDependencies (dotDependencyDepth dotOpts) graph depLoader)
   where -- fmap a function over the result of a function with 3 arguments
-        fmap3 :: Functor f => (d -> e) -> (a -> b -> c -> f d) -> a -> b -> c -> f e
-        fmap3 f g a b c = f <$> g a b c
+        fmap4 :: Functor f => (r -> r') -> (a -> b -> c -> d -> f r) -> a -> b -> c -> d -> f r'
+        fmap4 f g a b c d = f <$> g a b c d
 
         fst4 :: (a,b,c,d) -> a
         fst4 (x,_,_,_) = x
@@ -111,8 +110,7 @@ createDependencyGraph dotOpts = do
 listDependencies :: (HasEnvConfig env
                     ,HasHttpManager env
                     ,HasLogLevel env
-                    ,MonadBaseControl IO m
-                    ,MonadCatch m
+                    ,MonadBaseUnlift IO m
                     ,MonadLogger m
                     ,MonadMask m
                     ,MonadIO m
@@ -183,13 +181,13 @@ resolveDependencies limit graph loadPackageDeps = do
 createDepLoader :: Applicative m
                 => Map PackageName PackageSource
                 -> Map PackageName Installed
-                -> (PackageName -> Version -> Map FlagName Bool -> m (Set PackageName,Maybe Version))
+                -> (PackageName -> Version -> Map FlagName Bool -> [Text] -> m (Set PackageName,Maybe Version))
                 -> PackageName
                 -> m (Set PackageName, Maybe Version)
 createDepLoader sourceMap installed loadPackageDeps pkgName =
   case Map.lookup pkgName sourceMap of
     Just (PSLocal lp) -> pure ((packageAllDeps &&& (Just . packageVersion)) (lpPackage lp))
-    Just (PSUpstream version _ flags) -> loadPackageDeps pkgName version flags
+    Just (PSUpstream version _ flags ghcOptions _) -> loadPackageDeps pkgName version flags ghcOptions
     Nothing -> pure (Set.empty, fmap installedVersion (Map.lookup pkgName installed))
 
 -- | Resolve the direct (depth 0) external dependencies of the given local packages
@@ -232,14 +230,14 @@ printLocalNodes dotOpts locals = liftIO $ Text.putStrLn (Text.intercalate "\n" l
         lpNodes = map (applyStyle . nodeName) (F.toList locals)
 
 -- | Print nodes without dependencies
-printLeaves :: (Applicative m, MonadIO m)
+printLeaves :: MonadIO m
             => Map PackageName (Set PackageName,Maybe Version)
             -> m ()
-printLeaves = F.traverse_ printLeaf . Map.keysSet . Map.filter Set.null . fmap fst
+printLeaves = F.mapM_ printLeaf . Map.keysSet . Map.filter Set.null . fmap fst
 
 -- | @printDedges p ps@ prints an edge from p to every ps
-printEdges :: (Applicative m, MonadIO m) => PackageName -> Set PackageName -> m ()
-printEdges package deps = F.for_ deps (printEdge package)
+printEdges :: MonadIO m => PackageName -> Set PackageName -> m ()
+printEdges package deps = F.forM_ deps (printEdge package)
 
 -- | Print an edge between the two package names
 printEdge :: MonadIO m => PackageName -> PackageName -> m ()
