@@ -11,22 +11,26 @@
 module Stack.Ghci
     ( GhciOpts(..)
     , GhciPkgInfo(..)
+    , GhciException(..)
     , ghciSetup
     , ghci
     ) where
 
+import           Control.Applicative
 import           Control.Exception.Enclosed (tryAny)
+import           Control.Monad
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
 import           Control.Monad.RWS.Strict
 import           Control.Monad.State.Strict
-import           Control.Monad.Trans.Resource
+import           Control.Monad.Trans.Unlift (MonadBaseUnlift)
 import qualified Data.ByteString.Char8 as S8
 import           Data.Either
 import           Data.Function
 import           Data.List
 import           Data.List.Extra (nubOrd)
+import           Data.List.Split (splitOn)
 import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as M
 import           Data.Maybe
@@ -35,6 +39,7 @@ import           Data.Set (Set)
 import qualified Data.Set as S
 import           Data.Text (Text)
 import qualified Data.Text as T
+import           Data.Typeable (Typeable)
 import           Distribution.ModuleName (ModuleName)
 import           Distribution.Text (display)
 import           Network.HTTP.Client.Conduit
@@ -51,7 +56,8 @@ import           Stack.Exec
 import           Stack.Package
 import           Stack.Types
 import           Stack.Types.Internal
-import           System.Directory (getTemporaryDirectory)
+import           System.FilePath (takeBaseName)
+import           Text.Read (readMaybe)
 
 #ifndef WINDOWS
 import qualified System.Posix.Files as Posix
@@ -65,9 +71,10 @@ data GhciOpts = GhciOpts
     , ghciNoLoadModules      :: !Bool
     , ghciAdditionalPackages :: ![String]
     , ghciMainIs             :: !(Maybe Text)
+    , ghciLoadLocalDeps      :: !Bool
     , ghciSkipIntermediate   :: !Bool
     , ghciHidePackages       :: !Bool
-    , ghciBuildOpts          :: !BuildOpts
+    , ghciBuildOptsCLI       :: !BuildOptsCLI
     } deriving Show
 
 -- | Necessary information to load a package or its components.
@@ -82,21 +89,32 @@ data GhciPkgInfo = GhciPkgInfo
     , ghciPkgPackage :: !Package
     } deriving Show
 
+data GhciException
+    = InvalidPackageOption String
+    | LoadingDuplicateModules
+    deriving (Typeable)
+
+instance Exception GhciException
+
+instance Show GhciException where
+    show (InvalidPackageOption name) =
+        "Failed to parse --package option " ++ name
+    show LoadingDuplicateModules = unlines
+        [ "Not attempting to start ghci due to these duplicate modules."
+        , "Use --no-load to try to start it anyway, without loading any modules (but these are still likely to cause errors)"
+        ]
+
 -- | Launch a GHCi session for the given local package targets with the
 -- given options and configure it with the load paths and extensions
 -- of those targets.
 ghci
-    :: (HasConfig r, HasBuildConfig r, HasHttpManager r, MonadMask m, HasLogLevel r, HasTerminal r, HasEnvConfig r, MonadReader r m, MonadIO m, MonadThrow m, MonadLogger m, MonadCatch m, MonadBaseControl IO m)
+    :: (HasBuildConfig r, HasHttpManager r, MonadMask m, HasLogLevel r, HasTerminal r, HasEnvConfig r, MonadReader r m, MonadIO m, MonadLogger m, MonadBaseUnlift IO m)
     => GhciOpts -> m ()
-ghci GhciOpts{..} = do
-    let bopts = ghciBuildOpts
-            { boptsTestOpts = (boptsTestOpts ghciBuildOpts) { toDisableRun = True }
-            , boptsBenchmarkOpts = (boptsBenchmarkOpts ghciBuildOpts) { beoDisableRun = True }
-            }
-    (targets,mainIsTargets,pkgs) <- ghciSetup bopts ghciNoBuild ghciSkipIntermediate ghciMainIs
+ghci opts@GhciOpts{..} = do
+    bopts <- asks (configBuild . getConfig)
+    (targets,mainIsTargets,pkgs) <- ghciSetup opts
     config <- asks getConfig
     bconfig <- asks getBuildConfig
-    mainFile <- figureOutMainFile bopts mainIsTargets targets pkgs
     wc <- getWhichCompiler
     let pkgopts = hidePkgOpt ++ genOpts ++ ghcOpts
         hidePkgOpt = if null pkgs || not ghciHidePackages then [] else ["-hide-all-packages"]
@@ -109,19 +127,33 @@ ghci GhciOpts{..} = do
             getUserOptions Nothing ++
             concatMap (getUserOptions . Just . ghciPkgName) pkgs
         getUserOptions mpkg =
-            map T.unpack (M.findWithDefault [] mpkg (configGhcOptions config))
+            map T.unpack (M.findWithDefault [] mpkg (unGhcOptions (configGhcOptions config)))
         badForGhci x =
             isPrefixOf "-O" x || elem x (words "-debug -threaded -ticky -static -Werror")
     unless (null omittedOpts) $
         $logWarn
             ("The following GHC options are incompatible with GHCi and have not been passed to it: " <>
              T.unwords (map T.pack (nubOrd omittedOpts)))
+    allModules <- checkForDuplicateModules ghciNoLoadModules pkgs
     oiDir <- objectInterfaceDir bconfig
-    let modulesToLoad = nubOrd $
-            concatMap (map display . S.toList . ghciPkgModules) pkgs
-        thingsToLoad =
-            maybe [] (return . toFilePath) mainFile <> modulesToLoad
-        odir =
+    (modulesToLoad, mainFile) <- if ghciNoLoadModules then return ([], Nothing) else do
+        mmainFile <- figureOutMainFile bopts mainIsTargets targets pkgs
+        modulesToLoad <- case mmainFile of
+            Just mainFile -> do
+                let (_, mfDirs, mfName) = filePathPieces mainFile
+                    mainPathPieces = map toFilePath mfDirs ++ [takeBaseName (toFilePath mfName)]
+                liftM catMaybes $ forM allModules $ \mn -> do
+                    let matchesModule = splitOn "." mn `isSuffixOf` mainPathPieces
+                    if matchesModule
+                        then do
+                            $logWarn $ "Warning: Omitting load of module " <> T.pack mn <>
+                               ", because it matches the filepath of the Main target, " <>
+                               T.pack (toFilePath mainFile)
+                            return Nothing
+                        else return (Just mn)
+            Nothing -> return allModules
+        return (modulesToLoad, mmainFile)
+    let odir =
             [ "-odir=" <> toFilePathNoTrailingSep oiDir
             , "-hidir=" <> toFilePathNoTrailingSep oiDir ]
     $logInfo
@@ -129,15 +161,14 @@ ghci GhciOpts{..} = do
          T.intercalate ", " (map (packageNameText . ghciPkgName) pkgs))
     let execGhci extras = do
             menv <- liftIO $ configEnvOverride config defaultEnvSettings
-            exec menv
+            execSpawn menv
                  (fromMaybe (compilerExeName wc) ghciGhcCommand)
                  ("--interactive" :
                  -- This initial "-i" resets the include directories to not
                  -- include CWD.
                   "-i" :
                   odir <> pkgopts <> ghciArgs <> extras)
-    tmp <- liftIO getTemporaryDirectory
-    withCanonicalizedTempDirectory tmp "ghci" $ \tmpDir -> do
+    withSystemTempDir "ghci" $ \tmpDir -> do
         let macrosFile = tmpDir </> $(mkRelFile "cabal_macros.h")
         macrosOpts <- preprocessCabalMacros pkgs macrosFile
         if ghciNoLoadModules
@@ -145,9 +176,10 @@ ghci GhciOpts{..} = do
             else do
                 let scriptPath = tmpDir </> $(mkRelFile "ghci-script")
                     fp = toFilePath scriptPath
-                    loadModules = ":load " <> unwords (map show thingsToLoad)
+                    loadModules = ":add " <> unwords (map show modulesToLoad)
+                    addMainFile = maybe "" ((":add " <>) . toFilePath) mainFile
                     bringIntoScope = ":module + " <> unwords modulesToLoad
-                liftIO (writeFile fp (unlines [loadModules,bringIntoScope]))
+                liftIO (writeFile fp (unlines [loadModules,addMainFile,bringIntoScope]))
                 setScriptPerms fp
                 execGhci (macrosOpts ++ ["-ghci-script=" <> fp])
 
@@ -155,7 +187,7 @@ ghci GhciOpts{..} = do
 -- is none, sometimes it's unambiguous, sometimes it's
 -- ambiguous. Warns and returns nothing if it's ambiguous.
 figureOutMainFile
-    :: (Monad m, MonadLogger m)
+    :: (MonadLogger m, MonadIO m)
     => BuildOpts
     -> Maybe (Map PackageName SimpleTarget)
     -> Map PackageName SimpleTarget
@@ -166,18 +198,22 @@ figureOutMainFile bopts mainIsTargets targets0 packages =
         [] -> return Nothing
         [c@(_,_,fp)] -> do $logInfo ("Using main module: " <> renderCandidate c)
                            return (Just fp)
-        candidate:_ -> borderedWarning $ do
+        candidate:_ -> do
+          borderedWarning $ do
             $logWarn "The main module to load is ambiguous. Candidates are: "
             forM_ (map renderCandidate candidates) $logWarn
             $logWarn
-                "None will be loaded. You can specify which one to pick by: "
+                "You can specify which one to pick by: "
             $logWarn
-                (" 1) Specifying targets to stack ghci e.g. stack ghci " <>
+                (" * Specifying targets to stack ghci e.g. stack ghci " <>
                  sampleTargetArg candidate)
             $logWarn
-                (" 2) Specifying what the main is e.g. stack ghci " <>
+                (" * Specifying what the main is e.g. stack ghci " <>
                  sampleMainIsArg candidate)
-            return Nothing
+            $logWarn
+                (" * Choosing from the candidate above [1.." <>
+                T.pack (show $ length candidates) <> "]")
+          liftIO userOption
   where
     targets = fromMaybe targets0 mainIsTargets
     candidates = do
@@ -194,11 +230,35 @@ figureOutMainFile bopts mainIsTargets targets0 packages =
               where
                 wantedComponents =
                     wantedPackageComponents bopts target (ghciPkgPackage pkg)
-    renderCandidate (pkgName,namedComponent,mainIs) =
-        "Package `" <> packageNameText pkgName <> "' component " <>
-        renderComp namedComponent <>
-        " with main-is file: " <>
-        T.pack (toFilePath mainIs)
+    renderCandidate c@(pkgName,namedComponent,mainIs) =
+        let candidateIndex = T.pack . show . (+1) . fromMaybe 0 . elemIndex c
+        in  candidateIndex candidates <> ". Package `" <>
+            packageNameText pkgName <>
+            "' component " <>
+            renderComp namedComponent <>
+            " with main-is file: " <>
+            T.pack (toFilePath mainIs)
+    candidateIndices = take (length candidates) [1 :: Int ..]
+    userOption = do
+      putStr "Specify main module to use (press enter to load none): "
+      option <- getLine
+      let selected = fromMaybe
+                      ((+1) $ length candidateIndices)
+                      (readMaybe option :: Maybe Int)
+      case elemIndex selected candidateIndices  of
+        Nothing -> do
+            putStrLn
+              "Not loading any main modules, as no valid module selected"
+            putStrLn ""
+            return Nothing
+        Just op -> do
+            let (_,_,fp) = candidates !! op
+            putStrLn
+              ("Loading main module from cadidate " <>
+              show (op + 1) <> ", --main-is " <>
+              toFilePath fp)
+            putStrLn ""
+            return $ Just fp
     renderComp c =
         case c of
             CLib -> "lib"
@@ -213,22 +273,34 @@ figureOutMainFile bopts mainIsTargets targets0 packages =
 -- | Create a list of infos for each target containing necessary
 -- information to load that package/components.
 ghciSetup
-    :: (HasConfig r, HasHttpManager r, HasBuildConfig r, MonadMask m, HasTerminal r, HasLogLevel r, HasEnvConfig r, MonadReader r m, MonadIO m, MonadThrow m, MonadLogger m, MonadCatch m, MonadBaseControl IO m)
-    => BuildOpts
-    -> Bool
-    -> Bool
-    -> Maybe Text
+    :: (HasHttpManager r, HasBuildConfig r, MonadMask m, HasTerminal r, HasLogLevel r, HasEnvConfig r, MonadReader r m, MonadIO m, MonadLogger m, MonadBaseUnlift IO m)
+    => GhciOpts
     -> m (Map PackageName SimpleTarget, Maybe (Map PackageName SimpleTarget), [GhciPkgInfo])
-ghciSetup bopts noBuild skipIntermediate mainIs = do
-    (_,_,targets) <- parseTargetsFromBuildOpts AllowNoTargets bopts
+ghciSetup GhciOpts{..} = do
+    (_,_,targets) <- parseTargetsFromBuildOpts AllowNoTargets ghciBuildOptsCLI
     mainIsTargets <-
-        case mainIs of
+        case ghciMainIs of
             Nothing -> return Nothing
             Just target -> do
-                (_,_,targets') <- parseTargetsFromBuildOpts AllowNoTargets bopts { boptsTargets = [target] }
+                (_,_,targets') <- parseTargetsFromBuildOpts AllowNoTargets ghciBuildOptsCLI { boptsCLITargets = [target] }
                 return (Just targets')
+    addPkgs <- forM ghciAdditionalPackages $ \name -> do
+        let mres = (packageIdentifierName <$> parsePackageIdentifierFromString name)
+                <|> parsePackageNameFromString name
+        maybe (throwM $ InvalidPackageOption name) return mres
+    let boptsCli = ghciBuildOptsCLI
+            { boptsCLITargets = boptsCLITargets ghciBuildOptsCLI ++ map T.pack ghciAdditionalPackages
+            }
+    -- Try to build, but optimistically launch GHCi anyway if it fails (#1065)
+    unless ghciNoBuild $ do
+        eres <- tryAny $ build (const (return ())) Nothing boptsCli
+        case eres of
+            Right () -> return ()
+            Left err -> do
+                $logError $ T.pack (show err)
+                $logWarn "Warning: build failed, but optimistically launching GHCi anyway"
     econfig <- asks getEnvConfig
-    (realTargets,_,_,_,sourceMap) <- loadSourceMap AllowNoTargets bopts
+    (realTargets,_,_,_,sourceMap) <- loadSourceMap AllowNoTargets boptsCli
     menv <- getMinimalEnvOverride
     (installedMap, _, _, _) <- getInstalled
         menv
@@ -239,41 +311,40 @@ ghciSetup bopts noBuild skipIntermediate mainIs = do
         sourceMap
     directlyWanted <-
         forMaybeM (M.toList (envConfigPackages econfig)) $
-        \(dir,validWanted) ->
-             do cabalfp <- getCabalFileName dir
+        \(dir,treatLikeExtraDep) ->
+             do cabalfp <- findOrGenerateCabalFile dir
                 name <- parsePackageNameFromFilePath cabalfp
-                if validWanted
-                    then case M.lookup name targets of
+                if treatLikeExtraDep
+                    then return Nothing
+                    else case M.lookup name targets of
                              Just simpleTargets ->
                                  return (Just (name, (cabalfp, simpleTargets)))
                              Nothing -> return Nothing
-                    else return Nothing
-    let intermediateDeps = getIntermediateDeps sourceMap directlyWanted
+    let extraLoadDeps = getExtraLoadDeps ghciLoadLocalDeps sourceMap directlyWanted
     wanted <-
-        if skipIntermediate || null intermediateDeps
+        if (ghciSkipIntermediate && not ghciLoadLocalDeps) || null extraLoadDeps
             then return directlyWanted
             else do
-                $logInfo $ T.concat
-                    [ "The following libraries will also be loaded into GHCi because "
-                    , "they are intermediate dependencies of your targets:\n    "
-                    , T.intercalate ", " (map (packageNameText . fst) intermediateDeps)
-                    , "\n(Use --skip-intermediate-deps to omit these)"
-                    ]
-                return (directlyWanted ++ intermediateDeps)
-    -- Try to build, but optimistically launch GHCi anyway if it fails (#1065)
-    unless noBuild $ do
-        eres <- tryAny $ build (const (return ())) Nothing bopts
-        case eres of
-            Right () -> return ()
-            Left err -> do
-                $logError $ T.pack (show err)
-                $logWarn "Warning: build failed, but optimistically launching GHCi anyway"
+                let extraList = T.intercalate ", " (map (packageNameText . fst) extraLoadDeps)
+                if ghciLoadLocalDeps
+                    then $logInfo $ T.concat
+                        [ "The following libraries will also be loaded into GHCi because "
+                        , "they are local dependencies of your targets, and you specified --load-local-deps:\n    "
+                        , extraList
+                        ]
+                    else $logInfo $ T.concat
+                        [ "The following libraries will also be loaded into GHCi because "
+                        , "they are intermediate dependencies of your targets:\n    "
+                        , extraList
+                        , "\n(Use --skip-intermediate-deps to omit these)"
+                        ]
+                return (directlyWanted ++ extraLoadDeps)
     -- Load the list of modules _after_ building, to catch changes in unlisted dependencies (#1180)
-    let localLibs = [name | (name, (_, target)) <- wanted , hasLocalComp isCLib target]
+    let localLibs = [name | (name, (_, target)) <- wanted, hasLocalComp isCLib target]
     infos <-
         forM wanted $
         \(name,(cabalfp,target)) ->
-             makeGhciPkgInfo bopts sourceMap installedMap localLibs name cabalfp target
+             makeGhciPkgInfo boptsCli sourceMap installedMap localLibs addPkgs name cabalfp target
     checkForIssues infos
     return (realTargets, mainIsTargets, infos)
   where
@@ -286,28 +357,31 @@ ghciSetup bopts noBuild skipIntermediate mainIs = do
 -- | Make information necessary to load the given package in GHCi.
 makeGhciPkgInfo
     :: (MonadReader r m, HasEnvConfig r, MonadLogger m, MonadIO m, MonadCatch m)
-    => BuildOpts
+    => BuildOptsCLI
     -> SourceMap
     -> InstalledMap
     -> [PackageName]
+    -> [PackageName]
     -> PackageName
     -> Path Abs File
     -> SimpleTarget
     -> m GhciPkgInfo
-makeGhciPkgInfo bopts sourceMap installedMap locals name cabalfp target = do
+makeGhciPkgInfo boptsCli sourceMap installedMap locals addPkgs name cabalfp target = do
+    bopts <- asks (configBuild . getConfig)
     econfig <- asks getEnvConfig
     bconfig <- asks getBuildConfig
     let config =
             PackageConfig
             { packageConfigEnableTests = True
             , packageConfigEnableBenchmarks = True
-            , packageConfigFlags = localFlags (boptsFlags bopts) bconfig name
+            , packageConfigFlags = getLocalFlags bconfig boptsCli name
+            , packageConfigGhcOptions = getGhcOptions bconfig boptsCli name True True
             , packageConfigCompilerVersion = envConfigCompilerVersion econfig
             , packageConfigPlatform = configPlatform (getConfig bconfig)
             }
     (warnings,pkg) <- readPackage config cabalfp
     mapM_ (printCabalFileWarning cabalfp) warnings
-    (mods,files,opts) <- getPackageOpts (packageOpts pkg) sourceMap installedMap locals cabalfp
+    (mods,files,opts) <- getPackageOpts (packageOpts pkg) sourceMap installedMap locals addPkgs cabalfp
     let filteredOpts = filterWanted opts
         filterWanted = M.filterWithKey (\k _ -> k `S.member` allWanted)
         allWanted = wantedPackageComponents bopts target pkg
@@ -407,15 +481,35 @@ borderedWarning f = do
     $logWarn ""
     return x
 
+checkForDuplicateModules :: (MonadThrow m, MonadLogger m) => Bool -> [GhciPkgInfo] -> m [String]
+checkForDuplicateModules noLoadModules pkgs = do
+    unless (null duplicates) $ do
+        borderedWarning $ do
+            $logWarn "The following modules are present in multiple packages:"
+            forM_ duplicates $ \(mn, pns) -> do
+                $logWarn (" * " <> T.pack mn <> " (in " <> T.intercalate ", " (map packageNameText pns) <> ")")
+        unless noLoadModules $ throwM LoadingDuplicateModules
+    return (map fst allModules)
+  where
+    duplicates, allModules :: [(String, [PackageName])]
+    duplicates = filter (not . null . tail . snd) allModules
+    allModules =
+        M.toList $ M.fromListWith (++) $
+        concatMap (\pkg -> map (, [ghciPkgName pkg]) (map display (S.toList (ghciPkgModules pkg)))) pkgs
+
 -- Adds in intermediate dependencies between ghci targets. Note that it
 -- will return a Lib component for these intermediate dependencies even
 -- if they don't have a library (but that's fine for the usage within
 -- this module).
-getIntermediateDeps
-    :: SourceMap
+--
+-- If 'True' is passed for loadAllDeps, this loads all local deps, even
+-- if they aren't intermediate.
+getExtraLoadDeps
+    :: Bool
+    -> SourceMap
     -> [(PackageName, (Path Abs File, SimpleTarget))]
     -> [(PackageName, (Path Abs File, SimpleTarget))]
-getIntermediateDeps sourceMap targets =
+getExtraLoadDeps loadAllDeps sourceMap targets =
     M.toList $
     (\mp -> foldl' (flip M.delete) mp (map fst targets)) $
     M.mapMaybe id $
@@ -432,26 +526,26 @@ getIntermediateDeps sourceMap targets =
         cache <- get
         case (M.lookup name cache, M.lookup name sourceMap) of
             (Just (Just _), _) -> return True
-            (Just Nothing, _) -> return False
+            (Just Nothing, _) | not loadAllDeps -> return False
             (_, Just (PSLocal lp)) -> do
                 let deps = M.keys (packageDeps (lpPackage lp))
-                isIntermediate <- liftM or $ mapM go deps
-                if isIntermediate
+                shouldLoad <- liftM or $ mapM go deps
+                if shouldLoad
                     then do
                         modify (M.insert name (Just (lpCabalFile lp, STLocalComps (S.singleton CLib))))
                         return True
                     else do
                         modify (M.insert name Nothing)
                         return False
-            (_, Just PSUpstream{}) -> return False
-            (Nothing, Nothing) -> return False
+            (_, Just PSUpstream{}) -> return loadAllDeps
+            (_, _) -> return False
 
 preprocessCabalMacros :: MonadIO m => [GhciPkgInfo] -> Path Abs File -> m [String]
 preprocessCabalMacros pkgs out = liftIO $ do
     let fps = nubOrd (concatMap (mapMaybe (bioCabalMacros . snd) . ghciPkgOpts) pkgs)
     files <- mapM (S8.readFile . toFilePath) fps
     if null files then return [] else do
-        S8.writeFile (toFilePath out) $ S8.intercalate "\n#undef CURRENT_PACKAGE_KEY\n" files
+        S8.writeFile (toFilePath out) $ S8.concat $ map (<> "\n#undef CURRENT_PACKAGE_KEY\n") files
         return ["-optP-include", "-optP" <> toFilePath out]
 
 setScriptPerms :: MonadIO m => FilePath -> m ()
@@ -467,3 +561,10 @@ setScriptPerms fp = do
         , Posix.otherReadMode
         ]
 #endif
+
+filePathPieces :: Path Abs File -> (Path Abs Dir, [Path Rel Dir], Path Rel File)
+filePathPieces x0 = go (parent x0, [], filename x0)
+  where
+    go (x, dirs, fp)
+        | parent x == x = (x, dirs, fp)
+        | otherwise = (parent x, dirname x : dirs, fp)
