@@ -1,5 +1,6 @@
 {-# LANGUAGE ScopedTypeVariables #-}
 {-# LANGUAGE DeriveDataTypeable #-}
+{-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
 
@@ -16,7 +17,6 @@ module Stack.New
     , listTemplates)
     where
 
-import           Control.Applicative
 import           Control.Monad
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
@@ -29,6 +29,7 @@ import qualified Data.ByteString.Lazy as LB
 import qualified Data.ByteString.Lazy.Char8 as L8
 import           Data.Conduit
 import           Data.Foldable (asum)
+import qualified Data.HashMap.Strict as HM
 import           Data.List
 import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as M
@@ -42,7 +43,10 @@ import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import qualified Data.Text.IO as T
 import qualified Data.Text.Lazy as LT
+import           Data.Time.Calendar
+import           Data.Time.Clock
 import           Data.Typeable
+import qualified Data.Yaml as Yaml
 import           Network.HTTP.Client.Conduit hiding (path)
 import           Network.HTTP.Download
 import           Network.HTTP.Types.Status
@@ -55,6 +59,7 @@ import           Stack.Types.TemplateName
 import           System.Process.Run
 import           Text.Hastache
 import           Text.Hastache.Context
+import           Text.Printf
 import           Text.ProjectTemplate
 
 --------------------------------------------------------------------------------
@@ -74,15 +79,15 @@ data NewOpts = NewOpts
 
 -- | Create a new project with the given options.
 new
-    :: (HasConfig r, MonadReader r m, MonadLogger m, MonadCatch m, MonadThrow m, MonadIO m, HasHttpManager r, Functor m, Applicative m)
-    => NewOpts -> m (Path Abs Dir)
-new opts = do
-    pwd <- getWorkingDir
+    :: (HasConfig r, MonadReader r m, MonadLogger m, MonadCatch m, MonadIO m, HasHttpManager r)
+    => NewOpts -> Bool -> m (Path Abs Dir)
+new opts forceOverwrite = do
+    pwd <- getCurrentDir
     absDir <- if bare then return pwd
                       else do relDir <- parseRelDir (packageNameString project)
                               liftM (pwd </>) (return relDir)
-    exists <- dirExists absDir
-    configTemplate <- configDefaultTemplate <$> asks getConfig
+    exists <- doesDirExist absDir
+    configTemplate <- asks (configDefaultTemplate . getConfig)
     let template = fromMaybe defaultTemplateName $ asum [ cliOptionTemplate
                                                         , configTemplate
                                                         ]
@@ -97,6 +102,7 @@ new opts = do
                     (newOptsNonceParams opts)
                     absDir
                     templateText
+            when (not forceOverwrite && bare) $ checkForOverwrite (M.keys files)
             writeTemplateFiles files
             runTemplateInits absDir
             return absDir
@@ -123,10 +129,10 @@ data TemplateFrom = LocalTemp | RemoteTemp
 -- | Download and read in a template's text content.
 loadTemplate
     :: forall m r.
-       (HasConfig r, HasHttpManager r, MonadReader r m, MonadIO m, MonadThrow m, MonadCatch m, MonadLogger m, Functor m, Applicative m)
+       (HasConfig r, HasHttpManager r, MonadReader r m, MonadIO m, MonadThrow m, MonadCatch m, MonadLogger m)
     => TemplateName -> (TemplateFrom -> m ()) -> m Text
 loadTemplate name logIt = do
-    templateDir <- templatesDir <$> asks getConfig
+    templateDir <- asks (templatesDir . getConfig)
     case templatePath name of
         AbsPath absFile -> logIt LocalTemp >> loadLocalFile absFile
         UrlPath s -> do
@@ -137,7 +143,9 @@ loadTemplate name logIt = do
             downloadTemplate req (templateDir </> rel)
         RelPath relFile ->
             catch
-                (loadLocalFile relFile <* logIt LocalTemp)
+                (do f <- loadLocalFile relFile
+                    logIt LocalTemp
+                    return f)
                 (\(e :: NewException) ->
                       case relRequest relFile of
                         Just req -> downloadTemplate req
@@ -149,7 +157,7 @@ loadTemplate name logIt = do
     loadLocalFile path = do
         $logDebug ("Opening local template: \"" <> T.pack (toFilePath path)
                                                 <> "\"")
-        exists <- fileExists path
+        exists <- doesFileExist path
         if exists
             then liftIO (T.readFile (toFilePath path))
             else throwM (FailedToLoadTemplate name (toFilePath path))
@@ -167,7 +175,7 @@ loadTemplate name logIt = do
 
 -- | Apply and unpack a template into a directory.
 applyTemplate
-    :: (MonadIO m, MonadThrow m, MonadCatch m, MonadReader r m, HasConfig r, MonadLogger m)
+    :: (MonadIO m, MonadCatch m, MonadReader r m, HasConfig r, MonadLogger m)
     => PackageName
     -> TemplateName
     -> Map Text Text
@@ -176,9 +184,14 @@ applyTemplate
     -> m (Map (Path Abs File) LB.ByteString)
 applyTemplate project template nonceParams dir templateText = do
     config <- asks getConfig
-    let context = M.union (M.union nonceParams name) configParams
+    currentYear <- do
+      now <- liftIO getCurrentTime
+      (year, _, _) <- return $ toGregorian . utctDay $ now
+      return $ T.pack . show $ year
+    let context = M.union (M.union nonceParams extraParams) configParams
           where
-            name = M.fromList [("name", packageNameText project)]
+            extraParams = M.fromList [ ("name", packageNameText project)
+                                     , ("year", currentYear) ]
             configParams = configTemplateParams config
     (applied,missingKeys) <-
         runWriterT
@@ -187,7 +200,7 @@ applyTemplate project template nonceParams dir templateText = do
                  templateText
                  (mkStrContextM (contextFunction context)))
     unless (S.null missingKeys)
-         ($logInfo (T.pack (show (MissingParameters project template missingKeys (configUserConfigPath config)))))
+         ($logInfo ("\n" <> T.pack (show (MissingParameters project template missingKeys (configUserConfigPath config))) <> "\n"))
     files :: Map FilePath LB.ByteString <-
         catch (execWriterT $
                yield (T.encodeUtf8 (LT.toStrict applied)) $$
@@ -197,9 +210,11 @@ applyTemplate project template nonceParams dir templateText = do
                    throwM (InvalidTemplate template (show e)))
     when (M.null files) $
          throwM (InvalidTemplate template "Template does not contain any files")
-    unless (any (".cabal" `isSuffixOf`) . M.keys $ files) $
-         throwM (InvalidTemplate template "Template does not contain a .cabal\
-                                          \ file")
+
+    let isPkgSpec f = ".cabal" `isSuffixOf` f || f == "package.yaml"
+    unless (any isPkgSpec . M.keys $ files) $
+         throwM (InvalidTemplate template "Template does not contain a .cabal \
+                                          \or package.yaml file")
     liftM
         M.fromList
         (mapM
@@ -223,6 +238,12 @@ applyTemplate project template nonceParams dir templateText = do
                 return MuNothing
             Just value -> return (MuVariable value)
 
+-- | Check if we're going to overwrite any existing files.
+checkForOverwrite :: (MonadIO m, MonadThrow m) => [Path Abs File] -> m ()
+checkForOverwrite files = do
+    overwrites <- filterM doesFileExist files
+    unless (null overwrites) $ throwM (AttemptedOverwrites overwrites)
+
 -- | Write files to the new project directory.
 writeTemplateFiles
     :: MonadIO m
@@ -231,7 +252,7 @@ writeTemplateFiles files =
     forM_
         (M.toList files)
         (\(fp,bytes) ->
-              do createTree (parent fp)
+              do ensureDir (parent fp)
                  liftIO (LB.writeFile (toFilePath fp) bytes))
 
 -- | Run any initialization functions, such as Git.
@@ -248,19 +269,28 @@ runTemplateInits dir = do
                   (\(_ :: ProcessExitedUnsuccessfully) ->
                          $logInfo "git init failed to run, ignoring ...")
 
---------------------------------------------------------------------------------
--- Getting templates list
-
+-- | Display the set of templates accompanied with description if available.
 listTemplates
-    :: (MonadIO m, MonadThrow m, MonadReader r m, HasHttpManager r, MonadCatch m, MonadLogger m)
+    :: (MonadIO m, MonadReader r m, HasHttpManager r, MonadCatch m)
     => m ()
 listTemplates = do
     templates <- getTemplates
-    mapM_ ($logInfo . templateName) (S.toList templates)
+    templateInfo <- getTemplateInfo
+    if not . M.null $ templateInfo then do
+      let keySizes  = map (T.length . templateName) $ S.toList templates
+          padWidth  = show $ maximum keySizes
+          outputfmt = "%-" <> padWidth <> "s %s\n"
+          headerfmt = "%-" <> padWidth <> "s   %s\n"
+      liftIO $ printf headerfmt ("Template"::String) ("Description"::String)
+      forM_ (S.toList templates) (\x -> do
+           let name = templateName x
+               desc = fromMaybe "" $ liftM (mappend "- ") (M.lookup name templateInfo >>= description)
+           liftIO $ printf outputfmt (T.unpack name) (T.unpack desc))
+      else mapM_ (liftIO . T.putStrLn . templateName) (S.toList templates)
 
 -- | Get the set of templates.
 getTemplates
-    :: (MonadIO m, MonadThrow m, MonadReader r m, HasHttpManager r, MonadCatch m)
+    :: (MonadIO m, MonadReader r m, HasHttpManager r, MonadCatch m)
     => m (Set TemplateName)
 getTemplates = do
     req <- liftM addHeaders (parseUrl defaultTemplatesList)
@@ -272,13 +302,36 @@ getTemplates = do
                 Left err -> throwM (BadTemplatesJSON err (responseBody resp))
                 Right value -> return value
         code -> throwM (BadTemplatesResponse code)
+
+getTemplateInfo
+    :: (MonadIO m, MonadReader r m, HasHttpManager r, MonadCatch m)
+    => m (Map Text TemplateInfo)
+getTemplateInfo = do
+  req <- liftM addHeaders (parseUrl defaultTemplateInfoUrl)
+  resp <- catch (liftM Right $ httpLbs req) (\(ex :: HttpException) -> return . Left $ "Failed to download template info. The HTTP error was: " <> show ex)
+  case resp >>= is200 of
+    Left err -> do
+      liftIO . putStrLn $ err
+      return M.empty
+    Right resp' ->
+      case Yaml.decodeEither (LB.toStrict $ responseBody resp') :: Either String Object of
+        Left err ->
+          throwM $ BadTemplateInfo err
+        Right o ->
+          return (M.mapMaybe (Yaml.parseMaybe Yaml.parseJSON) (M.fromList . HM.toList $ o) :: Map Text TemplateInfo)
   where
-    addHeaders req =
-        req
-        { requestHeaders = [ ("User-Agent", "The Haskell Stack")
-                           , ("Accept", "application/vnd.github.v3+json")] <>
-          requestHeaders req
-        }
+    is200 resp =
+      if statusCode (responseStatus resp) == 200
+        then return resp
+        else Left $ "Unexpected status code while retrieving templates info: " <> show (statusCode $ responseStatus resp)
+
+addHeaders :: Request -> Request
+addHeaders req =
+  req
+    { requestHeaders = [ ("User-Agent", "The Haskell Stack")
+                       , ("Accept", "application/vnd.github.v3+json")] <>
+      requestHeaders req
+    }
 
 -- | Parser the set of templates from the JSON.
 parseTemplateSet :: Value -> Parser (Set TemplateName)
@@ -308,6 +361,11 @@ defaultTemplateUrl :: String
 defaultTemplateUrl =
     "https://raw.githubusercontent.com/commercialhaskell/stack-templates/master"
 
+-- | Default web URL to get a yaml file containing template metadata.
+defaultTemplateInfoUrl :: String
+defaultTemplateInfoUrl =
+    "https://raw.githubusercontent.com/commercialhaskell/stack-templates/master/template-info.yaml"
+
 -- | Default web URL to list the repo contents.
 defaultTemplatesList :: String
 defaultTemplatesList =
@@ -328,6 +386,10 @@ data NewException
     | AlreadyExists !(Path Abs Dir)
     | MissingParameters !PackageName !TemplateName !(Set String) !(Path Abs File)
     | InvalidTemplate !TemplateName !String
+    | AttemptedOverwrites [Path Abs File]
+    | FailedToDownloadTemplateInfo !HttpException
+    | BadTemplateInfo !String
+    | BadTemplateInfoResponse !Int
     deriving (Typeable)
 
 instance Exception NewException
@@ -386,3 +448,13 @@ instance Show NewException where
         "The template \"" <> T.unpack (templateName name) <>
         "\" is invalid and could not be used. " <>
         "The error was: \"" <> why <> "\""
+    show (AttemptedOverwrites fps) =
+        "The template would create the following files, but they already exist:\n" <>
+        unlines (map (("  " ++) . toFilePath) fps) <>
+        "Use --force to ignore this, and overwite these files."
+    show (FailedToDownloadTemplateInfo ex) =
+        "Failed to download templates info. The HTTP error was: " <> show ex
+    show (BadTemplateInfo err) =
+        "Template info couldn't be parsed: " <> err
+    show (BadTemplateInfoResponse code) =
+        "Unexpected status code while retrieving templates info: " <> show code
