@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE LambdaCase #-}
 {-# LANGUAGE TupleSections #-}
 {-# LANGUAGE ScopedTypeVariables #-}
@@ -9,7 +10,7 @@
 {-# LANGUAGE RankNTypes #-}
 {-# LANGUAGE TemplateHaskell #-}
 {-# LANGUAGE ViewPatterns #-}
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE RecordWildCards #-}
 
 -- | Dealing with Cabal.
 
@@ -20,7 +21,8 @@ module Stack.Package
   ,readPackageUnresolved
   ,readPackageUnresolvedBS
   ,resolvePackage
-  ,getCabalFileName
+  ,findOrGenerateCabalFile
+  ,hpack
   ,Package(..)
   ,GetPackageFiles(..)
   ,GetPackageOpts(..)
@@ -30,29 +32,25 @@ module Stack.Package
   ,resolvePackageDescription
   ,packageToolDependencies
   ,packageDependencies
-  ,packageIdentifier
   ,autogenDir
   ,checkCabalFileName
   ,printCabalFileWarning
   ,cabalFilePackageId)
   where
 
-#if __GLASGOW_HASKELL__ < 710
-import           Control.Applicative (Applicative, (<$>), (<*>))
-#endif
+import Prelude ()
+import Prelude.Compat
 
 import           Control.Arrow ((&&&))
 import           Control.Exception hiding (try,catch)
-import           Control.Monad
+import           Control.Monad (liftM, liftM2, (<=<), when, forM, forM_)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
-import           Control.Monad.Logger (MonadLogger,logWarn)
-import           Control.Monad.Reader
+import           Control.Monad.Logger
+import           Control.Monad.Reader (MonadReader,runReaderT,ask,asks)
 import qualified Data.ByteString as BS
 import qualified Data.ByteString.Char8 as C8
-import           Data.Either
-import           Data.Function
-import           Data.List
+import           Data.List.Compat
 import           Data.List.Extra (nubOrd)
 import           Data.Map.Strict (Map)
 import qualified Data.Map.Strict as M
@@ -69,10 +67,10 @@ import           Data.Version (showVersion)
 import           Distribution.Compiler
 import           Distribution.ModuleName (ModuleName)
 import qualified Distribution.ModuleName as Cabal
-import           Distribution.Package hiding (Package,PackageName,packageName,packageVersion,PackageIdentifier)
 import qualified Distribution.Package as D
-import           Distribution.PackageDescription hiding (FlagName)
+import           Distribution.Package hiding (Package,PackageName,packageName,packageVersion,PackageIdentifier)
 import qualified Distribution.PackageDescription as D
+import           Distribution.PackageDescription hiding (FlagName)
 import           Distribution.PackageDescription.Parse
 import qualified Distribution.PackageDescription.Parse as D
 import           Distribution.ParseUtils
@@ -80,27 +78,21 @@ import           Distribution.Simple.Utils
 import           Distribution.System (OS (..), Arch, Platform (..))
 import           Distribution.Text (display, simpleParse)
 import qualified Distribution.Verbosity as D
+import qualified Hpack
+import qualified Hpack.Config as Hpack
 import           Path as FL
 import           Path.Extra
 import           Path.Find
-import           Path.IO
-import           Prelude
+import           Path.IO hiding (findFiles)
 import           Safe (headDef, tailSafe)
 import           Stack.Build.Installed
 import           Stack.Constants
 import           Stack.Types
-import qualified Stack.Types.PackageIdentifier
-import           System.Directory (doesFileExist, getDirectoryContents)
+import qualified System.Directory as D
 import           System.FilePath (splitExtensions, replaceExtension)
 import qualified System.FilePath as FilePath
 import           System.IO.Error
 
-packageIdentifier :: Package -> Stack.Types.PackageIdentifier.PackageIdentifier
-packageIdentifier pkg =
-    Stack.Types.PackageIdentifier.PackageIdentifier
-        (packageName pkg)
-        (packageVersion pkg)
-
 -- | Read the raw, unresolved package information.
 readPackageUnresolved :: (MonadIO m, MonadThrow m)
                       => Path Abs File
@@ -126,7 +118,7 @@ readPackageUnresolvedBS mcabalfp bs =
     dropBOM t = fromMaybe t $ T.stripPrefix "\xFEFF" t
 
 -- | Reads and exposes the package information
-readPackage :: (MonadLogger m, MonadIO m, MonadThrow m, MonadCatch m)
+readPackage :: (MonadLogger m, MonadIO m, MonadCatch m)
             => PackageConfig
             -> Path Abs File
             -> m ([PWarning],Package)
@@ -145,12 +137,12 @@ readPackageBS packageConfig bs =
 
 -- | Get 'GenericPackageDescription' and 'PackageDescription' reading info
 -- from given directory.
-readPackageDescriptionDir :: (MonadLogger m, MonadIO m, MonadThrow m, MonadCatch m)
+readPackageDescriptionDir :: (MonadLogger m, MonadIO m, MonadCatch m)
   => PackageConfig
   -> Path Abs Dir
   -> m (GenericPackageDescription, PackageDescription)
 readPackageDescriptionDir config pkgDir = do
-    cabalfp <- getCabalFileName pkgDir
+    cabalfp <- findOrGenerateCabalFile pkgDir
     gdesc   <- liftM snd (readPackageUnresolved cabalfp)
     return (gdesc, resolvePackageDescription config gdesc)
 
@@ -193,41 +185,57 @@ resolvePackage packageConfig gpkg =
     , packageDeps = deps
     , packageFiles = pkgFiles
     , packageTools = packageDescTools pkg
+    , packageGhcOptions = packageConfigGhcOptions packageConfig
     , packageFlags = packageConfigFlags packageConfig
+    , packageDefaultFlags = M.fromList
+      [(fromCabalFlagName (flagName flag), flagDefault flag) | flag <- genPackageFlags gpkg]
     , packageAllDeps = S.fromList (M.keys deps)
     , packageHasLibrary = maybe False (buildable . libBuildInfo) (library pkg)
     , packageTests = M.fromList
       [(T.pack (testName t), testInterface t) | t <- testSuites pkg
                                               , buildable (testBuildInfo t)]
     , packageBenchmarks = S.fromList
-      [T.pack (benchmarkName b) | b <- benchmarks pkg
-                                , buildable (benchmarkBuildInfo b)]
+      [T.pack (benchmarkName biBuildInfo) | biBuildInfo <- benchmarks pkg
+                                          , buildable (benchmarkBuildInfo biBuildInfo)]
     , packageExes = S.fromList
-      [T.pack (exeName b) | b <- executables pkg
-                          , buildable (buildInfo b)]
+      [T.pack (exeName biBuildInfo) | biBuildInfo <- executables pkg
+                                    , buildable (buildInfo biBuildInfo)]
     , packageOpts = GetPackageOpts $
-      \sourceMap installedMap omitPkgs cabalfp ->
+      \sourceMap installedMap omitPkgs addPkgs cabalfp ->
            do (componentsModules,componentFiles,_,_) <- getPackageFiles pkgFiles cabalfp
               componentsOpts <-
-                  generatePkgDescOpts sourceMap installedMap omitPkgs cabalfp pkg componentFiles
+                  generatePkgDescOpts sourceMap installedMap omitPkgs addPkgs cabalfp pkg componentFiles
               return (componentsModules,componentFiles,componentsOpts)
     , packageHasExposedModules = maybe
           False
           (not . null . exposedModules)
           (library pkg)
     , packageSimpleType = buildType (packageDescription gpkg) == Just Simple
-    , packageDefinedFlags = S.fromList $
-      map (fromCabalFlagName . flagName) $ genPackageFlags gpkg
     }
   where
     pkgFiles = GetPackageFiles $
         \cabalfp ->
-             do distDir <- distDirFromDir (parent cabalfp)
+             do let pkgDir = parent cabalfp
+                distDir <- distDirFromDir pkgDir
                 (componentModules,componentFiles,dataFiles',warnings) <-
                     runReaderT
                         (packageDescModulesAndFiles pkg)
                         (cabalfp, buildDir distDir)
-                return (componentModules, componentFiles, S.insert cabalfp dataFiles', warnings)
+                setupFiles <-
+                    if buildType pkg `elem` [Nothing, Just Custom]
+                    then do
+                        let setupHsPath = pkgDir </> $(mkRelFile "Setup.hs")
+                            setupLhsPath = pkgDir </> $(mkRelFile "Setup.lhs")
+                        setupHsExists <- doesFileExist setupHsPath
+                        if setupHsExists then return (S.singleton setupHsPath) else do
+                            setupLhsExists <- doesFileExist setupLhsPath
+                            if setupLhsExists then return (S.singleton setupLhsPath) else return S.empty
+                    else return S.empty
+                buildFiles <- liftM (S.insert cabalfp . S.union setupFiles) $ do
+                    let hpackPath = pkgDir </> $(mkRelFile Hpack.packageConfig)
+                    hpackExists <- doesFileExist hpackPath
+                    return $ if hpackExists then S.singleton hpackPath else S.empty
+                return (componentModules, componentFiles, buildFiles <> dataFiles', warnings)
     pkgId = package (packageDescription gpkg)
     name = fromCabalPackageName (pkgName pkgId)
     pkg = resolvePackageDescription packageConfig gpkg
@@ -237,34 +245,41 @@ resolvePackage packageConfig gpkg =
 -- options which apply generally to the package, not one specific
 -- component.
 generatePkgDescOpts
-    :: (HasEnvConfig env, HasPlatform env, MonadThrow m, MonadReader env m, MonadIO m)
+    :: (HasEnvConfig env, MonadThrow m, MonadReader env m, MonadIO m)
     => SourceMap
     -> InstalledMap
     -> [PackageName] -- ^ Packages to omit from the "-package" / "-package-id" flags
+    -> [PackageName] -- ^ Packages to add to the "-package" flags
     -> Path Abs File
     -> PackageDescription
     -> Map NamedComponent (Set DotCabalPath)
     -> m (Map NamedComponent BuildInfoOpts)
-generatePkgDescOpts sourceMap installedMap omitPkgs cabalfp pkg componentPaths = do
+generatePkgDescOpts sourceMap installedMap omitPkgs addPkgs cabalfp pkg componentPaths = do
+    config <- asks getConfig
     distDir <- distDirFromDir cabalDir
     let cabalMacros = autogenDir distDir </> $(mkRelFile "cabal_macros.h")
-    exists <- fileExists cabalMacros
+    exists <- doesFileExist cabalMacros
     let mcabalMacros =
             if exists
                 then Just cabalMacros
                 else Nothing
     let generate namedComponent binfo =
             ( namedComponent
-            , generateBuildInfoOpts
-                  sourceMap
-                  installedMap
-                  mcabalMacros
-                  cabalDir
-                  distDir
-                  omitPkgs
-                  binfo
-                  (fromMaybe mempty (M.lookup namedComponent componentPaths))
-                  namedComponent)
+            , generateBuildInfoOpts BioInput
+                { biSourceMap = sourceMap
+                , biInstalledMap = installedMap
+                , biCabalMacros = mcabalMacros
+                , biCabalDir = cabalDir
+                , biDistDir = distDir
+                , biOmitPackages = omitPkgs
+                , biAddPackages = addPkgs
+                , biBuildInfo = binfo
+                , biDotCabalPaths = fromMaybe mempty (M.lookup namedComponent componentPaths)
+                , biConfigLibDirs = configExtraLibDirs config
+                , biConfigIncludeDirs = configExtraIncludeDirs config
+                , biComponentName = namedComponent
+                }
+            )
     return
         ( M.fromList
               (concat
@@ -293,21 +308,26 @@ generatePkgDescOpts sourceMap installedMap omitPkgs cabalfp pkg componentPaths =
   where
     cabalDir = parent cabalfp
 
+data BioInput = BioInput
+    { biSourceMap :: !SourceMap
+    , biInstalledMap :: !InstalledMap
+    , biCabalMacros :: !(Maybe (Path Abs File))
+    , biCabalDir :: !(Path Abs Dir)
+    , biDistDir :: !(Path Abs Dir)
+    , biOmitPackages :: ![PackageName]
+    , biAddPackages :: ![PackageName]
+    , biBuildInfo :: !BuildInfo
+    , biDotCabalPaths :: !(Set DotCabalPath)
+    , biConfigLibDirs :: !(Set Text)
+    , biConfigIncludeDirs :: !(Set Text)
+    , biComponentName :: !NamedComponent
+    }
+
 -- | Generate GHC options for the target.
-generateBuildInfoOpts
-    :: SourceMap
-    -> InstalledMap
-    -> Maybe (Path Abs File)
-    -> Path Abs Dir
-    -> Path Abs Dir
-    -> [PackageName]
-    -> BuildInfo
-    -> Set DotCabalPath
-    -> NamedComponent
-    -> BuildInfoOpts
-generateBuildInfoOpts sourceMap installedMap mcabalMacros cabalDir distDir omitPkgs b dotCabalPaths componentName =
+generateBuildInfoOpts :: BioInput -> BuildInfoOpts
+generateBuildInfoOpts BioInput {..} =
     BuildInfoOpts
-        { bioOpts = ghcOpts b ++ cppOptions b
+        { bioOpts = ghcOpts ++ cppOptions biBuildInfo
         -- NOTE for future changes: Due to this use of nubOrd (and other uses
         -- downstream), these generated options must not rely on multiple
         -- argument sequences.  For example, ["--main-is", "Foo.hs", "--main-
@@ -316,65 +336,72 @@ generateBuildInfoOpts sourceMap installedMap mcabalMacros cabalDir distDir omitP
         --
         -- See https://github.com/commercialhaskell/stack/issues/1255
         , bioOneWordOpts = nubOrd $ concat
-            [extOpts b, srcOpts, includeOpts, extra b, extraDirs, fworks b, cObjectFiles]
+            [extOpts, srcOpts, includeOpts, libOpts, fworks, cObjectFiles]
         , bioPackageFlags = deps
-        , bioCabalMacros = mcabalMacros
+        , bioCabalMacros = biCabalMacros
         }
   where
     cObjectFiles =
         mapMaybe (fmap toFilePath .
-                  makeObjectFilePathFromC cabalDir componentName distDir)
+                  makeObjectFilePathFromC biCabalDir biComponentName biDistDir)
                  cfiles
-    cfiles = mapMaybe dotCabalCFilePath (S.toList dotCabalPaths)
+    cfiles = mapMaybe dotCabalCFilePath (S.toList biDotCabalPaths)
+    -- Generates: -package=base -package=base16-bytestring-0.1.1.6 ...
     deps =
         concat
-            [ case M.lookup (fromCabalPackageName name) installedMap of
+            [ case M.lookup name biInstalledMap of
                 Just (_, Stack.Types.Library _ident ipid) -> ["-package-id=" <> ghcPkgIdString ipid]
-                _ -> ["-package=" <> display name <>
+                _ -> ["-package=" <> packageNameString name <>
                  maybe "" -- This empty case applies to e.g. base.
-                     ((("-" <>) . versionString) . sourceVersion)
-                     (M.lookup (fromCabalPackageName name) sourceMap)]
-            | Dependency name _ <- targetBuildDepends b
-            , name `notElem` fmap toCabalPackageName omitPkgs]
-        -- Generates: -package=base -package=base16-bytestring-0.1.1.6 ...
-    sourceVersion (PSUpstream ver _ _) = ver
-    sourceVersion (PSLocal localPkg) = packageVersion (lpPackage localPkg)
-    ghcOpts = concatMap snd . filter (isGhc . fst) . options
+                     ((("-" <>) . versionString) . piiVersion)
+                     (M.lookup name biSourceMap)]
+            | name <- pkgs]
+    pkgs =
+        biAddPackages ++
+        [ name
+        | Dependency cname _ <- targetBuildDepends biBuildInfo
+        , let name = fromCabalPackageName cname
+        , name `notElem` biOmitPackages]
+    ghcOpts = concatMap snd . filter (isGhc . fst) $ options biBuildInfo
       where
         isGhc GHC = True
         isGhc _ = False
-    extOpts = map (("-X" ++) . display) . usedExtensions
+    extOpts = map (("-X" ++) . display) (usedExtensions biBuildInfo)
     srcOpts =
         map
             (("-i" <>) . toFilePathNoTrailingSep)
-            ([cabalDir | null (hsSourceDirs b)] <>
-             mapMaybe toIncludeDir (hsSourceDirs b) <>
-             [autogenDir distDir,buildDir distDir] <>
-             [makeGenDir (buildDir distDir)
-             | Just makeGenDir <- [fileGenDirFromComponentName componentName]]) ++
-        ["-stubdir=" ++ toFilePathNoTrailingSep (buildDir distDir)]
-    toIncludeDir "." = Just cabalDir
-    toIncludeDir x = fmap (cabalDir </>) (parseRelDir x)
+            ([biCabalDir | null (hsSourceDirs biBuildInfo)] <>
+             mapMaybe toIncludeDir (hsSourceDirs biBuildInfo) <>
+             [autogenDir biDistDir,buildDir biDistDir] <>
+             [makeGenDir (buildDir biDistDir)
+             | Just makeGenDir <- [fileGenDirFromComponentName biComponentName]]) ++
+        ["-stubdir=" ++ toFilePathNoTrailingSep (buildDir biDistDir)]
+    toIncludeDir "." = Just biCabalDir
+    toIncludeDir x = fmap (biCabalDir </>) (parseRelDir x)
     includeOpts =
-        [ "-I" <> toFilePathNoTrailingSep absDir
-        | dir <- includeDirs b
-        , absDir <- case (parseAbsDir dir, parseRelDir dir) of
-          (Just ab, _       ) -> [ab]
-          (_      , Just rel) -> [cabalDir </> rel]
-          (Nothing, Nothing ) -> []
+        map ("-I" <>) (configExtraIncludeDirs <> pkgIncludeOpts)
+    configExtraIncludeDirs =
+        map T.unpack (S.toList biConfigIncludeDirs)
+    pkgIncludeOpts =
+        [ toFilePathNoTrailingSep absDir
+        | dir <- includeDirs biBuildInfo
+        , absDir <- handleDir dir
         ]
-    extra
-        = map ("-l" <>)
-        . extraLibs
-    extraDirs =
-        [ "-L" <> toFilePathNoTrailingSep absDir
-        | dir <- extraLibDirs b
-        , absDir <- case (parseAbsDir dir, parseRelDir dir) of
-          (Just ab, _       ) -> [ab]
-          (_      , Just rel) -> [cabalDir </> rel]
-          (Nothing, Nothing ) -> []
+    libOpts =
+        map ("-l" <>) (extraLibs biBuildInfo) <>
+        map ("-L" <>) (configExtraLibDirs <> pkgLibDirs)
+    configExtraLibDirs =
+        map T.unpack (S.toList biConfigLibDirs)
+    pkgLibDirs =
+        [ toFilePathNoTrailingSep absDir
+        | dir <- extraLibDirs biBuildInfo
+        , absDir <- handleDir dir
         ]
-    fworks = map (\fwk -> "-framework=" <> fwk) . frameworks
+    handleDir dir = case (parseAbsDir dir, parseRelDir dir) of
+       (Just ab, _       ) -> [ab]
+       (_      , Just rel) -> [biCabalDir </> rel]
+       (Nothing, Nothing ) -> []
+    fworks = map (\fwk -> "-framework=" <> fwk) (frameworks biBuildInfo)
 
 -- | Make the .o path from the .c file path for a component. Example:
 --
@@ -484,7 +511,7 @@ allBuildInfo' pkg_descr = [ bi | Just lib <- [library pkg_descr]
 
 -- | Get all files referenced by the package.
 packageDescModulesAndFiles
-    :: (MonadLogger m, MonadIO m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m, MonadCatch m)
+    :: (MonadLogger m, MonadIO m, MonadReader (Path Abs File, Path Abs Dir) m, MonadCatch m)
     => PackageDescription
     -> m (Map NamedComponent (Set ModuleName), Map NamedComponent (Set DotCabalPath), Set (Path Abs File), [PackageWarning])
 packageDescModulesAndFiles pkg = do
@@ -527,7 +554,7 @@ packageDescModulesAndFiles pkg = do
     foldTuples = foldl' (<>) (M.empty, M.empty, [])
 
 -- | Resolve globbing of files (e.g. data files) to absolute paths.
-resolveGlobFiles :: (MonadLogger m,MonadIO m,MonadThrow m,MonadReader (Path Abs File, Path Abs Dir) m,MonadCatch m)
+resolveGlobFiles :: (MonadLogger m,MonadIO m,MonadReader (Path Abs File, Path Abs Dir) m,MonadCatch m)
                  => [String] -> m (Set (Path Abs File))
 resolveGlobFiles =
     liftM (S.fromList . catMaybes . concat) .
@@ -546,7 +573,7 @@ resolveGlobFiles =
         mapM resolveFileOrWarn names
     matchDirFileGlob' dir glob =
         catch
-            (liftIO (matchDirFileGlob_ dir glob))
+            (matchDirFileGlob_ dir glob)
             (\(e :: IOException) ->
                   if isUserError e
                       then do
@@ -571,26 +598,32 @@ resolveGlobFiles =
 -- ["test/package-dump/ghc-7.8.txt","test/package-dump/ghc-7.10.txt"]
 -- @
 --
-matchDirFileGlob_ :: String -> String -> IO [String]
+matchDirFileGlob_ :: (MonadLogger m, MonadIO m) => String -> String -> m [String]
 matchDirFileGlob_ dir filepath = case parseFileGlob filepath of
-  Nothing -> die $ "invalid file glob '" ++ filepath
-                ++ "'. Wildcards '*' are only allowed in place of the file"
-                ++ " name, not in the directory name or file extension."
-                ++ " If a wildcard is used it must be with an file extension."
+  Nothing -> liftIO $ die $
+      "invalid file glob '" ++ filepath
+      ++ "'. Wildcards '*' are only allowed in place of the file"
+      ++ " name, not in the directory name or file extension."
+      ++ " If a wildcard is used it must be with an file extension."
   Just (NoGlob filepath') -> return [filepath']
   Just (FileGlob dir' ext) -> do
-    files <- getDirectoryContents (dir FilePath.</> dir')
-    case   [ dir' FilePath.</> file
-           | file <- files
-           , let (name, ext') = splitExtensions file
-           , not (null name) && isSuffixOf ext ext' ] of
-      []      -> die $ "filepath wildcard '" ++ filepath
-                    ++ "' does not match any files."
-      matches -> return matches
+    efiles <- liftIO $ try $ D.getDirectoryContents (dir FilePath.</> dir')
+    let matches =
+            case efiles of
+                Left (_ :: IOException) -> []
+                Right files ->
+                    [ dir' FilePath.</> file
+                    | file <- files
+                    , let (name, ext') = splitExtensions file
+                    , not (null name) && isSuffixOf ext ext'
+                    ]
+    when (null matches) $
+        $logWarn $ "WARNING: filepath wildcard '" <> T.pack filepath <> "' does not match any files."
+    return matches
 
 -- | Get all files referenced by the benchmark.
 benchmarkFiles
-    :: (MonadLogger m, MonadIO m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m)
+    :: (MonadLogger m, MonadIO m, MonadCatch m, MonadReader (Path Abs File, Path Abs Dir) m)
     => Benchmark -> m (Set ModuleName, Set DotCabalPath, [PackageWarning])
 benchmarkFiles bench = do
     dirs <- mapMaybeM resolveDirOrWarn (hsSourceDirs build)
@@ -613,7 +646,7 @@ benchmarkFiles bench = do
 
 -- | Get all files referenced by the test.
 testFiles
-    :: (MonadLogger m, MonadIO m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m)
+    :: (MonadLogger m, MonadIO m, MonadCatch m, MonadReader (Path Abs File, Path Abs Dir) m)
     => TestSuite
     -> m (Set ModuleName, Set DotCabalPath, [PackageWarning])
 testFiles test = do
@@ -638,7 +671,7 @@ testFiles test = do
 
 -- | Get all files referenced by the executable.
 executableFiles
-    :: (MonadLogger m, MonadIO m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m)
+    :: (MonadLogger m, MonadIO m, MonadCatch m, MonadReader (Path Abs File, Path Abs Dir) m)
     => Executable
     -> m (Set ModuleName, Set DotCabalPath, [PackageWarning])
 executableFiles exe = do
@@ -658,7 +691,7 @@ executableFiles exe = do
 
 -- | Get all files referenced by the library.
 libraryFiles
-    :: (MonadLogger m, MonadIO m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m)
+    :: (MonadLogger m, MonadIO m, MonadCatch m, MonadReader (Path Abs File, Path Abs Dir) m)
     => Library -> m (Set ModuleName, Set DotCabalPath, [PackageWarning])
 libraryFiles lib = do
     dirs <- mapMaybeM resolveDirOrWarn (hsSourceDirs build)
@@ -678,7 +711,7 @@ libraryFiles lib = do
     build = libBuildInfo lib
 
 -- | Get all C sources and extra source files in a build.
-buildOtherSources :: (MonadLogger m,MonadIO m,MonadThrow m,MonadReader (Path Abs File, Path Abs Dir) m)
+buildOtherSources :: (MonadLogger m,MonadIO m,MonadCatch m,MonadReader (Path Abs File, Path Abs Dir) m)
            => BuildInfo -> m (Set DotCabalPath)
 buildOtherSources build =
     do csources <- liftM
@@ -822,36 +855,45 @@ depRange (Dependency _ r) = r
 -- extensions, plus find any of their module and TemplateHaskell
 -- dependencies.
 resolveFilesAndDeps
-    :: (MonadIO m, MonadLogger m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m)
+    :: (MonadIO m, MonadLogger m, MonadCatch m, MonadReader (Path Abs File, Path Abs Dir) m)
     => Maybe String         -- ^ Package component name
     -> [Path Abs Dir]       -- ^ Directories to look in.
     -> [DotCabalDescriptor] -- ^ Base names.
-    -> [Text]               -- ^ Extentions.
+    -> [Text]               -- ^ Extensions.
     -> m (Set ModuleName,Set DotCabalPath,[PackageWarning])
 resolveFilesAndDeps component dirs names0 exts = do
-    (dotCabalPaths,foundModules) <- loop names0 S.empty
-    warnings <- warnUnlisted foundModules
+    (dotCabalPaths, foundModules, missingModules) <- loop names0 S.empty
+    warnings <- liftM2 (++) (warnUnlisted foundModules) (warnMissing missingModules)
     return (foundModules, dotCabalPaths, warnings)
   where
-    loop [] doneModules = return (S.empty, doneModules)
+    loop [] _ = return (S.empty, S.empty, [])
     loop names doneModules0 = do
-        resolvedFiles <- resolveFiles dirs names exts
-        pairs <- mapM (getDependencies component) resolvedFiles
-        let doneModules' =
+        resolved <- resolveFiles dirs names exts
+        let foundFiles = mapMaybe snd resolved
+            (foundModules', missingModules') = partition (isJust . snd) resolved
+            foundModules = mapMaybe (dotCabalModule . fst) foundModules'
+            missingModules = mapMaybe (dotCabalModule . fst) missingModules'
+        pairs <- mapM (getDependencies component) foundFiles
+        let doneModules =
                 S.union
                     doneModules0
                     (S.fromList (mapMaybe dotCabalModule names))
             moduleDeps = S.unions (map fst pairs)
             thDepFiles = concatMap snd pairs
-            modulesRemaining = S.difference moduleDeps doneModules'
-        (resolvedFiles',doneModules'') <-
-            loop (map DotCabalModule (S.toList modulesRemaining)) doneModules'
+            modulesRemaining = S.difference moduleDeps doneModules
+        -- Ignore missing modules discovered as dependencies - they may
+        -- have been deleted.
+        (resolvedFiles, resolvedModules, _) <-
+            loop (map DotCabalModule (S.toList modulesRemaining)) doneModules
         return
             ( S.union
                   (S.fromList
-                       (resolvedFiles <> map DotCabalFilePath thDepFiles))
-                  resolvedFiles'
-            , doneModules'')
+                       (foundFiles <> map DotCabalFilePath thDepFiles))
+                  resolvedFiles
+            , S.union
+                  (S.fromList foundModules)
+                  resolvedModules
+            , missingModules)
     warnUnlisted foundModules = do
         let unlistedModules =
                 foundModules `S.difference`
@@ -864,10 +906,23 @@ resolveFilesAndDeps component dirs names0 exts = do
                            cabalfp
                            component
                            (S.toList unlistedModules)]
+    warnMissing _missingModules = do
+        return []
+        {- FIXME: the issue with this is it's noisy for modules like Paths_*
+        cabalfp <- asks fst
+        return $
+            if null missingModules
+               then []
+               else [ MissingModulesWarning
+                           cabalfp
+                           component
+                           missingModules]
+        -}
+
 
 -- | Get the dependencies of a Haskell module file.
 getDependencies
-    :: (MonadReader (Path Abs File, Path Abs Dir) m, MonadIO m)
+    :: (MonadReader (Path Abs File, Path Abs Dir) m, MonadIO m, MonadCatch m, MonadLogger m)
     => Maybe String -> DotCabalPath -> m (Set ModuleName, [Path Abs File])
 getDependencies component dotCabalPath =
     case dotCabalPath of
@@ -886,7 +941,7 @@ getDependencies component dotCabalPath =
                         FilePath.replaceExtension
                             (toFilePath (dumpHIDir </> fileRel))
                             ".dump-hi"
-                dumpHIExists <- liftIO $ doesFileExist dumpHIPath
+                dumpHIExists <- liftIO $ D.doesFileExist dumpHIPath
                 if dumpHIExists
                     then parseDumpHI dumpHIPath
                     else return (S.empty, [])
@@ -896,7 +951,7 @@ getDependencies component dotCabalPath =
 
 -- | Parse a .dump-hi file into a set of modules and files.
 parseDumpHI
-    :: (MonadReader (Path Abs File, void) m, MonadIO m)
+    :: (MonadReader (Path Abs File, void) m, MonadIO m, MonadCatch m, MonadLogger m)
     => FilePath -> m (Set ModuleName, [Path Abs File])
 parseDumpHI dumpHIPath = do
     dir <- asks (parent . fst)
@@ -914,17 +969,17 @@ parseDumpHI dumpHIPath = do
             -- The dependent file path is surrounded by quotes but is not escaped.
             -- It can be an absolute or relative path.
             mapMaybe
-                (parseAbsOrRelFile dir <=<
-                 (fmap T.unpack .
+                ((fmap T.unpack .
                   (T.stripSuffix "\"" <=< T.stripPrefix "\"") .
                   T.dropWhileEnd (== '\r') . decodeUtf8 . C8.dropWhile (/= '"'))) $
             filter ("addDependentFile \"" `C8.isPrefixOf`) dumpHI
-    return (moduleDeps, thDeps)
-  where
-    parseAbsOrRelFile dir fp =
-        case parseRelFile fp of
-            Just rel -> Just (dir </> rel)
-            Nothing -> parseAbsFile fp
+    thDepsResolved <- liftM catMaybes $ forM thDeps $ \x -> do
+        mresolved <- forgivingAbsence (resolveFile dir x) >>= rejectMissingFile
+        when (isNothing mresolved) $
+            $logWarn $ "Warning: qAddDepedency path listed in " <> T.pack dumpHIPath <>
+                " does not exist: " <> T.pack x
+        return mresolved
+    return (moduleDeps, thDepsResolved)
 
 -- | Try to resolve the list of base names in the given directory by
 -- looking for unique instances of base names applied with the given
@@ -933,10 +988,10 @@ resolveFiles
     :: (MonadIO m, MonadLogger m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m)
     => [Path Abs Dir] -- ^ Directories to look in.
     -> [DotCabalDescriptor] -- ^ Base names.
-    -> [Text] -- ^ Extentions.
-    -> m [DotCabalPath]
+    -> [Text] -- ^ Extensions.
+    -> m [(DotCabalDescriptor, Maybe DotCabalPath)]
 resolveFiles dirs names exts =
-    forMaybeM names (findCandidate dirs exts)
+    forM names (\name -> liftM (name, ) (findCandidate dirs exts name))
 
 -- | Find a candidate for the given module-or-filename from the list
 -- of directories and given extensions.
@@ -969,29 +1024,29 @@ findCandidate dirs exts name = do
             DotCabalCFile{} -> DotCabalCFilePath
     paths_pkg pkg = "Paths_" ++ packageNameString pkg
     makeNameCandidates =
-        liftM (nubOrd . rights . concat) (mapM makeDirCandidates dirs)
+        liftM (nubOrd . concat) (mapM makeDirCandidates dirs)
     makeDirCandidates :: Path Abs Dir
-                      -> IO [Either ResolveException (Path Abs File)]
+                      -> IO [Path Abs File]
     makeDirCandidates dir =
         case name of
-            DotCabalMain fp -> liftM return (try (resolveFile' dir fp))
-            DotCabalFile fp -> liftM return (try (resolveFile' dir fp))
-            DotCabalCFile fp -> liftM return (try (resolveFile' dir fp))
+            DotCabalMain fp -> resolveCandidate dir fp
+            DotCabalFile fp -> resolveCandidate dir fp
+            DotCabalCFile fp -> resolveCandidate dir fp
             DotCabalModule mn ->
-                mapM
+                liftM concat
+                $ mapM
                   ((\ ext ->
-                     try (resolveFile' dir (Cabal.toFilePath mn ++ "." ++ ext)))
+                     resolveCandidate dir (Cabal.toFilePath mn ++ "." ++ ext))
                    . T.unpack)
                    exts
-    resolveFile'
+    resolveCandidate
         :: (MonadIO m, MonadThrow m)
-        => Path Abs Dir -> FilePath.FilePath -> m (Path Abs File)
-    resolveFile' x y = do
+        => Path Abs Dir -> FilePath.FilePath -> m [Path Abs File]
+    resolveCandidate x y = do
+        -- The standard canonicalizePath does not work for this case
         p <- parseCollapsedAbsFile (toFilePath x FilePath.</> y)
-        exists <- fileExists p
-        if exists
-            then return p
-            else throwM $ ResolveFileFailed x y (toFilePath p)
+        exists <- doesFileExist p
+        return $ if exists then [p] else []
 
 -- | Warn the user that multiple candidates are available for an
 -- entry, but that we picked one anyway and continued.
@@ -1036,7 +1091,7 @@ logPossibilities dirs mn = do
     makePossibilities name =
         mapM
             (\dir ->
-                  do (_,files) <- listDirectory dir
+                  do (_,files) <- listDir dir
                      return
                          (map
                               filename
@@ -1050,40 +1105,78 @@ logPossibilities dirs mn = do
 --
 -- If no .cabal file is present, or more than one is present, an exception is
 -- thrown via 'throwM'.
-getCabalFileName
-    :: (MonadThrow m, MonadIO m)
+--
+-- If the directory contains a file named package.yaml, hpack is used to
+-- generate a .cabal file from it.
+findOrGenerateCabalFile
+    :: forall m. (MonadThrow m, MonadIO m, MonadLogger m)
     => Path Abs Dir -- ^ package directory
     -> m (Path Abs File)
-getCabalFileName pkgDir = do
-    files <- liftIO $ findFiles
-        pkgDir
-        (flip hasExtension "cabal" . FL.toFilePath)
-        (const False)
-    case files of
-        [] -> throwM $ PackageNoCabalFileFound pkgDir
-        [x] -> return x
-        _:_ -> throwM $ PackageMultipleCabalFilesFound pkgDir files
-  where hasExtension fp x = FilePath.takeExtension fp == "." ++ x
+findOrGenerateCabalFile pkgDir = do
+    hpack pkgDir
+    findCabalFile
+  where
+    findCabalFile :: m (Path Abs File)
+    findCabalFile = findCabalFile' >>= either throwM return
+
+    findCabalFile' :: m (Either PackageException (Path Abs File))
+    findCabalFile' = do
+        files <- liftIO $ findFiles
+            pkgDir
+            (flip hasExtension "cabal" . FL.toFilePath)
+            (const False)
+        return $ case files of
+            [] -> Left $ PackageNoCabalFileFound pkgDir
+            [x] -> Right x
+            -- If there are multiple files, ignore files that start with
+            -- ".". On unixlike environments these are hidden, and this
+            -- character is not valid in package names. The main goal is
+            -- to ignore emacs lock files - see
+            -- https://github.com/commercialhaskell/stack/issues/1897.
+            (filter (not . ("." `isPrefixOf`) . toFilePath . filename) -> [x]) -> Right x
+            _:_ -> Left $ PackageMultipleCabalFilesFound pkgDir files
+      where hasExtension fp x = FilePath.takeExtension fp == "." ++ x
+
+-- | Generate .cabal file from package.yaml, if necessary.
+hpack :: (MonadIO m, MonadLogger m) => Path Abs Dir -> m ()
+hpack pkgDir = do
+    let hpackFile = pkgDir </> $(mkRelFile Hpack.packageConfig)
+    exists <- liftIO $ doesFileExist hpackFile
+    when exists $ do
+        let fpt = T.pack (toFilePath hpackFile)
+        $logDebug $ "Running hpack on " <> fpt
+        r <- liftIO $ Hpack.hpackResult (toFilePath pkgDir)
+        forM_ (Hpack.resultWarnings r) $ \w -> $logWarn ("WARNING: " <> T.pack w)
+        let cabalFile = T.pack (Hpack.resultCabalFile r)
+        case Hpack.resultStatus r of
+            Hpack.Generated -> $logDebug $
+                "hpack generated a modified version of " <> cabalFile
+            Hpack.OutputUnchanged -> $logDebug $
+                "hpack output unchanged in " <> cabalFile
+            -- NOTE: this is 'logInfo' so it will be outputted to the
+            -- user by default.
+            Hpack.AlreadyGeneratedByNewerHpack -> $logWarn $
+                "WARNING: " <> cabalFile <> " was generated with a newer version of hpack, please upgrade and try again."
 
 -- | Path for the package's build log.
 buildLogPath :: (MonadReader env m, HasBuildConfig env, MonadThrow m)
              => Package -> Maybe String -> m (Path Abs File)
 buildLogPath package' msuffix = do
   env <- ask
-  let stack = configProjectWorkDir env
+  let stack = getProjectWorkDir env
   fp <- parseRelFile $ concat $
     packageIdentifierString (packageIdentifier package') :
     maybe id (\suffix -> ("-" :) . (suffix :)) msuffix [".log"]
   return $ stack </> $(mkRelDir "logs") </> fp
 
 -- Internal helper to define resolveFileOrWarn and resolveDirOrWarn
-resolveOrWarn :: (MonadLogger m, MonadIO m, MonadReader (Path Abs File, Path Abs Dir) m)
+resolveOrWarn :: (MonadLogger m, MonadIO m, MonadThrow m, MonadReader (Path Abs File, Path Abs Dir) m)
               => Text
               -> (Path Abs Dir -> String -> m (Maybe a))
               -> FilePath.FilePath
               -> m (Maybe a)
 resolveOrWarn subject resolver path =
-  do cwd <- getWorkingDir
+  do cwd <- getCurrentDir
      file <- asks fst
      dir <- asks (parent . fst)
      result <- resolver dir path
@@ -1096,27 +1189,30 @@ resolveOrWarn subject resolver path =
 
 -- | Resolve the file, if it can't be resolved, warn for the user
 -- (purely to be helpful).
-resolveFileOrWarn :: (MonadThrow m,MonadIO m,MonadLogger m,MonadReader (Path Abs File, Path Abs Dir) m)
+resolveFileOrWarn :: (MonadCatch m,MonadIO m,MonadLogger m,MonadReader (Path Abs File, Path Abs Dir) m)
                   => FilePath.FilePath
                   -> m (Maybe (Path Abs File))
-resolveFileOrWarn = resolveOrWarn "File" resolveFileMaybe
+resolveFileOrWarn = resolveOrWarn "File" f
+  where f p x = forgivingAbsence (resolveFile p x) >>= rejectMissingFile
 
 -- | Resolve the directory, if it can't be resolved, warn for the user
 -- (purely to be helpful).
-resolveDirOrWarn :: (MonadThrow m,MonadIO m,MonadLogger m,MonadReader (Path Abs File, Path Abs Dir) m)
+resolveDirOrWarn :: (MonadCatch m,MonadIO m,MonadLogger m,MonadReader (Path Abs File, Path Abs Dir) m)
                  => FilePath.FilePath
                  -> m (Maybe (Path Abs Dir))
-resolveDirOrWarn = resolveOrWarn "Directory" resolveDirMaybe
+resolveDirOrWarn = resolveOrWarn "Directory" f
+  where f p x = forgivingAbsence (resolveDir p x) >>= rejectMissingDir
 
 -- | Extract the @PackageIdentifier@ given an exploded haskell package
 -- path.
 cabalFilePackageId
-    :: (Applicative m, MonadIO m, MonadThrow m)
+    :: (MonadIO m, MonadThrow m)
     => Path Abs File -> m PackageIdentifier
 cabalFilePackageId fp = do
     pkgDescr <- liftIO (D.readPackageDescription D.silent $ toFilePath fp)
     (toStackPI . D.package . D.packageDescription) pkgDescr
   where
-    toStackPI (D.PackageIdentifier (D.PackageName name) ver) =
-        PackageIdentifier <$> parsePackageNameFromString name <*>
-        parseVersionFromString (showVersion ver)
+    toStackPI (D.PackageIdentifier (D.PackageName name) ver) = do
+        name' <- parsePackageNameFromString name
+        ver' <- parseVersionFromString (showVersion ver)
+        return (PackageIdentifier name' ver')
