@@ -16,8 +16,10 @@ module Stack.Setup
   ( setupEnv
   , ensureCompiler
   , ensureDockerStackExe
+  , getSystemCompiler
   , SetupOpts (..)
   , defaultStackSetupYaml
+  , removeHaskellEnvVars
   ) where
 
 import           Control.Applicative
@@ -36,7 +38,6 @@ import qualified Data.ByteString.Char8 as S8
 import qualified Data.ByteString.Lazy as LBS
 import           Data.Char (isSpace)
 import           Data.Conduit (Conduit, ($$), (=$), await, yield, awaitForever)
-import qualified Data.Conduit.Binary as CB
 import           Data.Conduit.Lift (evalStateC)
 import qualified Data.Conduit.List as CL
 import           Data.Either
@@ -61,12 +62,12 @@ import qualified Data.Yaml as Yaml
 import           Distribution.System (OS, Arch (..), Platform (..))
 import qualified Distribution.System as Cabal
 import           Distribution.Text (simpleParse)
-import           Language.Haskell.TH as TH
+import           Lens.Micro (set)
 import           Network.HTTP.Client.Conduit
 import           Network.HTTP.Download.Verified
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
-import           Path.IO
+import           Path.IO hiding (findExecutable)
 import qualified Paths_stack as Meta
 import           Prelude hiding (concat, elem, any) -- Fix AMP warning
 import           Safe (readMay)
@@ -78,7 +79,7 @@ import           Stack.Fetch
 import           Stack.GhcPkg (createDatabase, getCabalPkgVer, getGlobalDB, mkGhcPackagePath)
 import           Stack.Setup.Installed
 import           Stack.Types
-import           Stack.Types.Internal (HasTerminal, HasReExec, HasLogLevel)
+import           Stack.Types.Internal (HasTerminal, HasReExec, HasLogLevel, envConfigBuildOpts, buildOptsInstallExes)
 import           Stack.Types.StackT
 import qualified System.Directory as D
 import           System.Environment (getExecutablePath)
@@ -156,7 +157,7 @@ instance Show SetupException where
         [ "The GHC located at "
         , toFilePath ghc
         , " failed to compile a sanity check. Please see:\n\n"
-        , "    https://github.com/commercialhaskell/stack/blob/release/doc/install_and_upgrade.md\n\n"
+        , "    http://docs.haskellstack.org/en/stable/install_and_upgrade/\n\n"
         , "for more information. Exception was:\n"
         , show e
         ]
@@ -217,6 +218,7 @@ setupEnv mResolveMissingGHC = do
     menv <- mkEnvOverride platform env
     compilerVer <- getCompilerVersion menv wc
     cabalVer <- getCabalPkgVer menv wc
+    $logDebug "Resolving package entries"
     packages <- mapM
         (resolvePackageEntry menv (bcRoot bconfig))
         (bcPackageEntries bconfig)
@@ -246,7 +248,7 @@ setupEnv mResolveMissingGHC = do
 
     executablePath <- liftIO getExecutablePath
 
-    utf8EnvVars <- getUtf8LocaleVars menv
+    utf8EnvVars <- getUtf8EnvVars menv compilerVer
 
     envRef <- liftIO $ newIORef Map.empty
     let getEnvOverride' es = do
@@ -270,6 +272,13 @@ setupEnv mResolveMissingGHC = do
                                 then Map.union utf8EnvVars
                                 else id)
 
+                        $ case (soptsSkipMsys sopts, platform) of
+                            (False, Platform Cabal.I386   Cabal.Windows)
+                                -> Map.insert "MSYSTEM" "MINGW32"
+                            (False, Platform Cabal.X86_64 Cabal.Windows)
+                                -> Map.insert "MSYSTEM" "MINGW64"
+                            _   -> id
+
                         -- For reasoning and duplication, see: https://github.com/fpco/stack/issues/70
                         $ Map.insert "HASKELL_PACKAGE_SANDBOX" (T.pack $ toFilePathNoTrailingSep deps)
                         $ Map.insert "HASKELL_PACKAGE_SANDBOXES"
@@ -284,6 +293,7 @@ setupEnv mResolveMissingGHC = do
                                         , ""
                                         ])
                         $ Map.insert "HASKELL_DIST_DIR" (T.pack $ toFilePathNoTrailingSep distDir) env
+
                     () <- atomicModifyIORef envRef $ \m' ->
                         (Map.insert es eo m', ())
                     return eo
@@ -327,7 +337,9 @@ ensureCompiler sopts = do
 
     msystem <-
         if soptsUseSystem sopts
-            then getSystemCompiler menv0 wc
+            then do
+                $logDebug "Getting system compiler version"
+                getSystemCompiler menv0 wc
             else return Nothing
 
     Platform expectedArch _ <- asks getPlatform
@@ -444,7 +456,7 @@ ensureDockerStackExe containerPlatform = do
         stackVersion = fromCabalVersion Meta.version
         tool = Tool (PackageIdentifier $(mkPackageName "stack") stackVersion)
     stackExePath <- (</> $(mkRelFile "stack")) <$> installDir programsPath tool
-    stackExeExists <- fileExists stackExePath
+    stackExeExists <- doesFileExist stackExePath
     unless stackExeExists $
         do
            $logInfo $ mconcat ["Downloading Docker-compatible ", T.pack stackProgName, " executable"]
@@ -474,7 +486,7 @@ upgradeCabal :: (MonadIO m, MonadLogger m, MonadReader env m, HasHttpManager env
              -> m ()
 upgradeCabal menv wc = do
     let name = $(mkPackageName "Cabal")
-    rmap <- resolvePackages menv Set.empty (Set.singleton name)
+    rmap <- resolvePackages menv Map.empty (Set.singleton name)
     newest <-
         case Map.keys rmap of
             [] -> error "No Cabal library found in index, cannot upgrade"
@@ -490,7 +502,7 @@ upgradeCabal menv wc = do
             , T.pack $ versionString newest
             , ". I'm not upgrading Cabal."
             ]
-        else withCanonicalizedSystemTempDirectory "stack-cabal-upgrade" $ \tmpdir -> do
+        else withSystemTempDir "stack-cabal-upgrade" $ \tmpdir -> do
             $logInfo $ T.concat
                 [ "Installing Cabal-"
                 , T.pack $ versionString newest
@@ -498,7 +510,8 @@ upgradeCabal menv wc = do
                 , T.pack $ versionString installed
                 ]
             let ident = PackageIdentifier name newest
-            m <- unpackPackageIdents menv tmpdir Nothing (Set.singleton ident)
+            -- Nothing below: use the newest .cabal file revision
+            m <- unpackPackageIdents menv tmpdir Nothing (Map.singleton ident Nothing)
 
             compilerPath <- join $ findExecutable menv (compilerExeName wc)
             newestDir <- parseRelDir $ versionString newest
@@ -541,9 +554,9 @@ getSystemCompiler menv wc = do
             eres <- tryProcessStdout Nothing menv exeName ["--info"]
             let minfo = do
                     Right bs <- Just eres
-                    pairs <- readMay $ S8.unpack bs :: Maybe [(String, String)]
-                    version <- lookup "Project version" pairs >>= parseVersionFromString
-                    arch <- lookup "Target platform" pairs >>= simpleParse . takeWhile (/= '-')
+                    pairs_ <- readMay $ S8.unpack bs :: Maybe [(String, String)]
+                    version <- lookup "Project version" pairs_ >>= parseVersionFromString
+                    arch <- lookup "Target platform" pairs_ >>= simpleParse . takeWhile (/= '-')
                     return (version, arch)
             case (wc, minfo) of
                 (Ghc, Just (version, arch)) -> return (Just (GhcVersion version, arch))
@@ -581,7 +594,7 @@ getSetupInfo stackSetupYaml manager = do
                              responseBody res $$ CL.consume
                     return $ S8.concat bss
                 Nothing -> liftIO $ S.readFile urlOrFile
-        (si,warnings) <- either throwM return (Yaml.decodeEither' bs)
+        WithJSONWarnings si warnings <- either throwM return (Yaml.decodeEither' bs)
         when (urlOrFile /= defaultStackSetupYaml) $
             logJSONWarnings urlOrFile warnings
         return si
@@ -652,7 +665,7 @@ downloadAndInstallCompiler si wanted@(GhcVersion{}) versionCheck mbindistURL = d
             ghcKey <- getGhcKey
             case Map.lookup ghcKey $ siGHCs si of
                 Nothing -> throwM $ UnknownOSKey ghcKey
-                Just pairs -> getWantedCompilerInfo ghcKey versionCheck wanted GhcVersion pairs
+                Just pairs_ -> getWantedCompilerInfo ghcKey versionCheck wanted GhcVersion pairs_
     config <- asks getConfig
     let installer =
             case configPlatform config of
@@ -676,14 +689,11 @@ downloadAndInstallCompiler si wanted versionCheck _mbindistUrl = do
         _ -> throwM GHCJSRequiresStandardVariant
     (selectedVersion, downloadInfo) <- case Map.lookup "source" $ siGHCJSs si of
         Nothing -> throwM $ UnknownOSKey "source"
-        Just pairs -> getWantedCompilerInfo "source" versionCheck wanted id pairs
+        Just pairs_ -> getWantedCompilerInfo "source" versionCheck wanted id pairs_
     $logInfo "Preparing to install GHCJS to an isolated location."
     $logInfo "This will not interfere with any system-level installation."
     let tool = ToolGhcjs selectedVersion
-        installer = installGHCJS $ case selectedVersion of
-            GhcjsVersion version _ -> version
-            _ -> error "Invariant violated: expected ghcjs version in downloadAndInstallCompiler."
-    downloadAndInstallTool (configLocalPrograms config) si downloadInfo tool installer
+    downloadAndInstallTool (configLocalPrograms config) si downloadInfo tool installGHCJS
 
 getWantedCompilerInfo :: (Ord k, MonadThrow m)
                       => Text
@@ -692,17 +702,17 @@ getWantedCompilerInfo :: (Ord k, MonadThrow m)
                       -> (k -> CompilerVersion)
                       -> Map k a
                       -> m (k, a)
-getWantedCompilerInfo key versionCheck wanted toCV pairs =
+getWantedCompilerInfo key versionCheck wanted toCV pairs_ =
     case mpair of
         Just pair -> return pair
-        Nothing -> throwM $ UnknownCompilerVersion key wanted (map toCV (Map.keys pairs))
+        Nothing -> throwM $ UnknownCompilerVersion key wanted (map toCV (Map.keys pairs_))
   where
     mpair =
         listToMaybe $
         sortBy (flip (comparing fst)) $
-        filter (isWantedCompiler versionCheck wanted . toCV . fst) (Map.toList pairs)
+        filter (isWantedCompiler versionCheck wanted . toCV . fst) (Map.toList pairs_)
 
-getGhcKey :: (MonadReader env m, MonadThrow m, HasPlatform env, HasGHCVariant env, MonadLogger m, MonadIO m, MonadCatch m, MonadBaseControl IO m)
+getGhcKey :: (MonadReader env m, HasPlatform env, HasGHCVariant env, MonadCatch m)
           => m Text
 getGhcKey = do
     ghcVariant <- asks getGHCVariant
@@ -710,7 +720,7 @@ getGhcKey = do
     osKey <- getOSKey platform
     return $ osKey <> T.pack (ghcVariantSuffix ghcVariant)
 
-getOSKey :: (MonadReader env m, MonadThrow m, HasPlatform env, MonadLogger m, MonadIO m, MonadCatch m, MonadBaseControl IO m)
+getOSKey :: (MonadThrow m)
          => Platform -> m Text
 getOSKey platform =
     case platform of
@@ -785,7 +795,7 @@ installGHCPosix version _ archiveFile archiveType destDir = do
     $logDebug $ "make: " <> T.pack makeTool
     $logDebug $ "tar: " <> T.pack tarTool
 
-    withCanonicalizedSystemTempDirectory "stack-setup" $ \root -> do
+    withSystemTempDir "stack-setup" $ \root -> do
         dir <-
             liftM (root </>) $
             parseRelDir $
@@ -806,13 +816,12 @@ installGHCPosix version _ archiveFile archiveType destDir = do
         $logDebug $ "GHC installed to " <> T.pack (toFilePath destDir)
 
 installGHCJS :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env m, HasConfig env, HasHttpManager env, HasTerminal env, HasReExec env, HasLogLevel env, MonadBaseControl IO m)
-             => Version
-             -> SetupInfo
+             => SetupInfo
              -> Path Abs File
              -> ArchiveType
              -> Path Abs Dir
              -> m ()
-installGHCJS version si archiveFile archiveType destDir = do
+installGHCJS si archiveFile archiveType destDir = do
     platform <- asks getPlatform
     menv0 <- getMinimalEnvOverride
     -- This ensures that locking is disabled for the invocations of
@@ -833,10 +842,9 @@ installGHCJS version si archiveFile archiveType destDir = do
     -- install cabal-install. This lets us also fix the version of
     -- cabal-install used.
     let unpackDir = destDir </> $(mkRelDir "src")
-    tarComponent <- parseRelDir ("ghcjs-" ++ versionString version)
     runUnpack <- case platform of
         Platform _ Cabal.Windows -> return $
-            withUnpackedTarball7z "GHCJS" si archiveFile archiveType tarComponent unpackDir
+            withUnpackedTarball7z "GHCJS" si archiveFile archiveType Nothing unpackDir
         _ -> do
             zipTool' <-
                 case archiveType of
@@ -850,9 +858,11 @@ installGHCJS version si archiveFile archiveType destDir = do
             $logDebug $ "ziptool: " <> T.pack zipTool
             $logDebug $ "tar: " <> T.pack tarTool
             return $ do
-                removeTreeIfExists unpackDir
+                ignoringAbsence (removeDirRecur destDir)
+                ignoringAbsence (removeDirRecur unpackDir)
                 readInNull destDir tarTool menv ["xf", toFilePath archiveFile] Nothing
-                renameDir (destDir </> tarComponent) unpackDir
+                innerDir <- expectSingleUnpackedDir archiveFile destDir
+                renameDir innerDir unpackDir
 
     $logSticky $ T.concat ["Unpacking GHCJS into ", T.pack . toFilePath $ unpackDir, " ..."]
     $logDebug $ "Unpacking " <> T.pack (toFilePath archiveFile)
@@ -861,32 +871,32 @@ installGHCJS version si archiveFile archiveType destDir = do
     $logSticky "Setting up GHCJS build environment"
     let stackYaml = unpackDir </> $(mkRelFile "stack.yaml")
         destBinDir = destDir </> $(mkRelDir "bin")
-    createTree destBinDir
-    envConfig <- loadGhcjsEnvConfig stackYaml destBinDir
+    ensureDir destBinDir
+    envConfig' <- loadGhcjsEnvConfig stackYaml destBinDir
 
     -- On windows we need to copy options files out of the install dir.  Argh!
     -- This is done before the build, so that if it fails, things fail
     -- earlier.
     mwindowsInstallDir <- case platform of
         Platform _ Cabal.Windows ->
-            liftM Just $ runInnerStackT envConfig installationRootLocal
+            liftM Just $ runInnerStackT envConfig' installationRootLocal
         _ -> return Nothing
 
     $logSticky "Installing GHCJS (this will take a long time) ..."
-    runInnerStackT envConfig $
-        build (\_ -> return ()) Nothing defaultBuildOpts { boptsInstallExes = True }
+    runInnerStackT ((set (envConfigBuildOpts.buildOptsInstallExes) True envConfig')) $
+        (build (\_ -> return ()) Nothing defaultBuildOptsCLI)
     -- Copy over *.options files needed on windows.
     forM_ mwindowsInstallDir $ \dir -> do
-        (_, files) <- listDirectory (dir </> $(mkRelDir "bin"))
+        (_, files) <- listDir (dir </> $(mkRelDir "bin"))
         forM_ (filter ((".options" `isSuffixOf`). toFilePath) files) $ \optionsFile -> do
             let dest = destDir </> $(mkRelDir "bin") </> filename optionsFile
-            removeFileIfExists dest
+            ignoringAbsence (removeFile dest)
             copyFile optionsFile dest
     $logStickyDone "Installed GHCJS."
 
 -- Install the downloaded stack binary distribution
 installDockerStackExe
-    :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env m, HasConfig env, HasHttpManager env, MonadBaseControl IO m)
+    :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env m, HasConfig env, MonadBaseControl IO m)
     => SetupInfo
     -> Path Abs File
     -> ArchiveType
@@ -897,7 +907,7 @@ installDockerStackExe _ archiveFile _ destDir = do
         checkDependencies $
         (,) <$> checkDependency "gzip" <*> checkDependency "tar"
     menv <- getMinimalEnvOverride
-    createTree destDir
+    ensureDir destDir
     readInNull
         destDir
         tarTool
@@ -926,68 +936,88 @@ ensureGhcjsBooted menv cv shouldBoot  = do
                 -- https://github.com/commercialhaskell/stack/issues/749#issuecomment-147382783
                 -- This only affects the case where GHCJS has been
                 -- installed with an older version and not yet booted.
-                stackYamlExists <- fileExists stackYaml
-                actualStackYaml <- if stackYamlExists then return stackYaml
-                    else case cv of
-                        GhcjsVersion version _ ->
-                            liftM ((destDir </> $(mkRelDir "src")) </>) $
-                            parseRelFile $ "ghcjs-" ++ versionString version ++ "/stack.yaml"
+                stackYamlExists <- doesFileExist stackYaml
+                ghcjsVersion <- case cv of
+                        GhcjsVersion version _ -> return version
                         _ -> fail "ensureGhcjsBooted invoked on non GhcjsVersion"
-                actualStackYamlExists <- fileExists actualStackYaml
+                actualStackYaml <- if stackYamlExists then return stackYaml
+                    else
+                        liftM ((destDir </> $(mkRelDir "src")) </>) $
+                        parseRelFile $ "ghcjs-" ++ versionString ghcjsVersion ++ "/stack.yaml"
+                actualStackYamlExists <- doesFileExist actualStackYaml
                 unless actualStackYamlExists $
                     fail "Couldn't find GHCJS stack.yaml in old or new location."
-                bootGhcjs actualStackYaml destDir
+                bootGhcjs ghcjsVersion actualStackYaml destDir
         Left err -> throwM err
 
 bootGhcjs :: (MonadIO m, MonadBaseControl IO m, MonadLogger m, MonadCatch m, HasHttpManager env, HasTerminal env, HasReExec env, HasLogLevel env, MonadReader env m)
-          => Path Abs File -> Path Abs Dir -> m ()
-bootGhcjs stackYaml destDir = do
+          => Version -> Path Abs File -> Path Abs Dir -> m ()
+bootGhcjs ghcjsVersion stackYaml destDir = do
     envConfig <- loadGhcjsEnvConfig stackYaml (destDir </> $(mkRelDir "bin"))
     menv <- liftIO $ configEnvOverride (getConfig envConfig) defaultEnvSettings
     -- Install cabal-install if missing, or if the installed one is old.
     mcabal <- getCabalInstallVersion menv
     shouldInstallCabal <- case mcabal of
         Nothing -> do
-            $logInfo "No cabal-install binary found for use with GHCJS.  Installing a local copy of cabal-install from source."
+            $logInfo "No cabal-install binary found for use with GHCJS."
             return True
         Just v
             | v < $(mkVersion "1.22.4") -> do
                 $logInfo $
-                    "cabal-install found on PATH is too old to be used for booting GHCJS (version " <>
+                    "The cabal-install found on PATH is too old to be used for booting GHCJS (version " <>
                     versionText v <>
-                    ").  Installing a local copy of cabal-install from source."
+                    ")."
+                return True
+            | v >= $(mkVersion "1.23") -> do
+                $logWarn $
+                    "The cabal-install found on PATH is a version stack doesn't know about, version " <>
+                    versionText v <>
+                    ". This may or may not work.\n" <>
+                    "See this issue: https://github.com/ghcjs/ghcjs/issues/470"
+                return False
+            | ghcjsVersion >= $(mkVersion "0.2.0.20160413") && v >= $(mkVersion "1.22.8") -> do
+                $logWarn $
+                    "The cabal-install found on PATH, version " <>
+                    versionText v <>
+                    ", is >= 1.22.8.\n" <>
+                    "That version has a bug preventing ghcjs < 0.2.0.20160413 from booting.\n" <>
+                    "See this issue: https://github.com/ghcjs/ghcjs/issues/470"
                 return True
             | otherwise -> return False
+    let envSettings = defaultEnvSettings { esIncludeGhcPackagePath = False }
+    menv' <- liftIO $ configEnvOverride (getConfig envConfig) envSettings
     when shouldInstallCabal $ do
-        $logSticky "Building cabal-install for use by ghcjs-boot ... "
+        $logInfo "Building a local copy of cabal-install from source."
         runInnerStackT envConfig $
             build (\_ -> return ())
                   Nothing
-                  defaultBuildOpts { boptsTargets = ["cabal-install"] }
+                  defaultBuildOptsCLI { boptsCLITargets = ["cabal-install"] }
+        mcabal' <- getCabalInstallVersion menv'
+        case mcabal' of
+            Nothing ->
+                $logError $
+                    "Failed to get cabal-install version after installing it.\n" <>
+                    "This shouldn't happen, because it gets built to the snapshot bin directory, which should be treated as being on the PATH."
+            Just v | v >= $(mkVersion "1.22.8") && v < $(mkVersion "1.23") ->
+                $logWarn $
+                    "Installed version of cabal-install is in a version range which may not work.\n" <>
+                    "See this issue: https://github.com/ghcjs/ghcjs/issues/470\n" <>
+                    "This version is specified by the stack.yaml file included in the ghcjs tarball.\n"
+            _ -> return ()
     $logSticky "Booting GHCJS (this will take a long time) ..."
-    let envSettings = defaultEnvSettings { esIncludeGhcPackagePath = False }
-    menv' <- liftIO $ configEnvOverride (getConfig envConfig) envSettings
-    runAndLog Nothing "ghcjs-boot" menv' ["--clean"]
+    logProcessStderrStdout Nothing "ghcjs-boot" menv' ["--clean"]
     $logStickyDone "GHCJS booted."
 
--- TODO: something similar is done in Stack.Build.Execute. Create some utilities
--- for this?
-runAndLog :: (MonadIO m, MonadBaseControl IO m, MonadLogger m)
-          => Maybe (Path Abs Dir) -> String -> EnvOverride -> [String] -> m ()
-runAndLog mdir name menv args = liftBaseWith $ \restore -> do
-    let logLines = CB.lines =$ CL.mapM_ (void . restore . monadLoggerLog $(TH.location >>= liftLoc) "" LevelInfo . toLogStr)
-    void $ restore $ sinkProcessStderrStdout mdir menv name args logLines logLines
-
 loadGhcjsEnvConfig :: (MonadIO m, HasHttpManager r, MonadReader r m, HasTerminal r, HasReExec r, HasLogLevel r)
                      => Path Abs File -> Path b t -> m EnvConfig
 loadGhcjsEnvConfig stackYaml binPath = runInnerStackLoggingT $ do
     lc <- loadConfig
         (mempty
-            { configMonoidInstallGHC = Just True
-            , configMonoidLocalBinPath = Just (toFilePath binPath)
+            { configMonoidInstallGHC = First (Just True)
+            , configMonoidLocalBinPath = First (Just (toFilePath binPath))
             })
-        (Just stackYaml)
         Nothing
+        (Just stackYaml)
     bconfig <- lcLoadBuildConfig lc Nothing
     runInnerStackT bconfig $ setupEnv Nothing
 
@@ -1042,7 +1072,7 @@ installGHCWindows :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env m,
                   -> m ()
 installGHCWindows version si archiveFile archiveType destDir = do
     tarComponent <- parseRelDir $ "ghc-" ++ versionString version
-    withUnpackedTarball7z "GHC" si archiveFile archiveType tarComponent destDir
+    withUnpackedTarball7z "GHC" si archiveFile archiveType (Just tarComponent) destDir
     $logInfo $ "GHC installed to " <> T.pack (toFilePath destDir)
 
 installMsys2Windows :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env m, HasConfig env, HasHttpManager env, MonadBaseControl IO m)
@@ -1061,18 +1091,19 @@ installMsys2Windows osKey si archiveFile archiveType destDir = do
         throwM e
 
     msys <- parseRelDir $ "msys" ++ T.unpack (fromMaybe "32" $ T.stripPrefix "windows" osKey)
-    withUnpackedTarball7z "MSYS2" si archiveFile archiveType msys destDir
+    withUnpackedTarball7z "MSYS2" si archiveFile archiveType (Just msys) destDir
 
-    platform <- asks getPlatform
-    menv0 <- getMinimalEnvOverride
-    newEnv <- augmentPathMap [toFilePath $ destDir </> $(mkRelDir "usr")
-                                                   </> $(mkRelDir "bin")]
-                             (unEnvOverride menv0)
-    menv <- mkEnvOverride platform newEnv
 
     -- I couldn't find this officially documented anywhere, but you need to run
-    -- the shell once in order to initialize some pacman stuff. Once that run
-    -- happens, you can just run commands as usual.
+    -- the MSYS shell once in order to initialize some pacman stuff. Once that
+    -- run happens, you can just run commands as usual.
+    platform <- asks getPlatform
+    menv0 <- getMinimalEnvOverride
+    newEnv0 <- modifyEnvOverride menv0 $ Map.insert "MSYSTEM" "MSYS"
+    newEnv <- augmentPathMap [toFilePath $ destDir </> $(mkRelDir "usr")
+                                                   </> $(mkRelDir "bin")]
+                             (unEnvOverride newEnv0)
+    menv <- mkEnvOverride platform newEnv
     runCmd (Cmd (Just destDir) "sh" menv ["--login", "-c", "true"]) Nothing
 
     -- No longer installing git, it's unreliable
@@ -1089,10 +1120,10 @@ withUnpackedTarball7z :: (MonadIO m, MonadMask m, MonadLogger m, MonadReader env
                       -> SetupInfo
                       -> Path Abs File -- ^ Path to archive file
                       -> ArchiveType
-                      -> Path Rel Dir -- ^ Name of directory expected to be in archive.
+                      -> Maybe (Path Rel Dir) -- ^ Name of directory expected in archive.  If Nothing, expects a single folder.
                       -> Path Abs Dir -- ^ Destination directory.
                       -> m ()
-withUnpackedTarball7z name si archiveFile archiveType srcDir destDir = do
+withUnpackedTarball7z name si archiveFile archiveType msrcDir destDir = do
     suffix <-
         case archiveType of
             TarXz -> return ".xz"
@@ -1105,12 +1136,14 @@ withUnpackedTarball7z name si archiveFile archiveType srcDir destDir = do
             Just x -> parseAbsFile $ T.unpack x
     run7z <- setup7z si
     let tmpName = toFilePathNoTrailingSep (dirname destDir) ++ "-tmp"
-    createTree (parent destDir)
-    withCanonicalizedTempDirectory (toFilePath $ parent destDir) tmpName $ \tmpDir -> do
-        let absSrcDir = tmpDir </> srcDir
-        removeTreeIfExists destDir
+    ensureDir (parent destDir)
+    withTempDir (parent destDir) tmpName $ \tmpDir -> do
+        ignoringAbsence (removeDirRecur destDir)
         run7z (parent archiveFile) archiveFile
         run7z tmpDir tarFile
+        absSrcDir <- case msrcDir of
+            Just srcDir -> return $ tmpDir </> srcDir
+            Nothing -> expectSingleUnpackedDir archiveFile tmpDir
         removeFile tarFile `catchIO` \e ->
             $logWarn (T.concat
                 [ "Exception when removing "
@@ -1120,6 +1153,13 @@ withUnpackedTarball7z name si archiveFile archiveType srcDir destDir = do
                 ])
         renameDir absSrcDir destDir
 
+expectSingleUnpackedDir :: (MonadIO m, MonadThrow m) => Path Abs File -> Path Abs Dir -> m (Path Abs Dir)
+expectSingleUnpackedDir archiveFile destDir = do
+    contents <- listDir destDir
+    case contents of
+        ([dir], []) -> return dir
+        _ -> error $ "Expected a single directory within unpacked " ++ toFilePath archiveFile
+
 -- | Download 7z as necessary, and get a function for unpacking things.
 --
 -- Returned function takes an unpack directory and archive.
@@ -1270,13 +1310,12 @@ chunksOverTime diff = do
           else put (lastTime,    acc')
         go
 
-
 -- | Perform a basic sanity check of GHC
 sanityCheck :: (MonadIO m, MonadMask m, MonadLogger m, MonadBaseControl IO m)
             => EnvOverride
             -> WhichCompiler
             -> m ()
-sanityCheck menv wc = withCanonicalizedSystemTempDirectory "stack-sanity-check" $ \dir -> do
+sanityCheck menv wc = withSystemTempDir "stack-sanity-check" $ \dir -> do
     let fp = toFilePath $ dir </> $(mkRelFile "Main.hs")
     liftIO $ writeFile fp $ unlines
         [ "import Distribution.Simple" -- ensure Cabal library is present
@@ -1300,79 +1339,86 @@ removeHaskellEnvVars =
     Map.delete "GHC_PACKAGE_PATH" .
     Map.delete "HASKELL_PACKAGE_SANDBOX" .
     Map.delete "HASKELL_PACKAGE_SANDBOXES" .
-    Map.delete "HASKELL_DIST_DIR"
+    Map.delete "HASKELL_DIST_DIR" .
+    -- https://github.com/commercialhaskell/stack/issues/1460
+    Map.delete "DESTDIR"
 
--- | Get map of environment variables to set to change the locale's encoding to UTF-8
-getUtf8LocaleVars
+-- | Get map of environment variables to set to change the GHC's encoding to UTF-8
+getUtf8EnvVars
     :: forall m env.
        (MonadReader env m, HasPlatform env, MonadLogger m, MonadCatch m, MonadBaseControl IO m, MonadIO m)
-    => EnvOverride -> m (Map Text Text)
-getUtf8LocaleVars menv = do
-    Platform _ os <- asks getPlatform
-    if os == Cabal.Windows
-        then
-             -- On Windows, locale is controlled by the code page, so we don't set any environment
-             -- variables.
-             return
-                 Map.empty
-        else do
-            let checkedVars = map checkVar (Map.toList $ eoTextMap menv)
-                -- List of environment variables that will need to be updated to set UTF-8 (because
-                -- they currently do not specify UTF-8).
-                needChangeVars = concatMap fst checkedVars
-                -- Set of locale-related environment variables that have already have a value.
-                existingVarNames = Set.unions (map snd checkedVars)
-                -- True if a locale is already specified by one of the "global" locale variables.
-                hasAnyExisting =
-                    any (`Set.member` existingVarNames) ["LANG", "LANGUAGE", "LC_ALL"]
-            if null needChangeVars && hasAnyExisting
-                then
-                     -- If no variables need changes and at least one "global" variable is set, no
-                     -- changes to environment need to be made.
-                     return
-                         Map.empty
-                else do
-                    -- Get a list of known locales by running @locale -a@.
-                    elocales <- tryProcessStdout Nothing menv "locale" ["-a"]
-                    let
-                        -- Filter the list to only include locales with UTF-8 encoding.
-                        utf8Locales =
-                            case elocales of
-                                Left _ -> []
-                                Right locales ->
-                                    filter
-                                        isUtf8Locale
-                                        (T.lines $
-                                         T.decodeUtf8With
-                                             T.lenientDecode
-                                             locales)
-                        mfallback = getFallbackLocale utf8Locales
-                    when
-                        (isNothing mfallback)
-                        ($logWarn
-                             "Warning: unable to set locale to UTF-8 encoding; GHC may fail with 'invalid character'")
-                    let
-                        -- Get the new values of variables to adjust.
-                        changes =
-                            Map.unions $
-                            map
-                                (adjustedVarValue utf8Locales mfallback)
-                                needChangeVars
-                        -- Get the values of variables to add.
-                        adds
-                          | hasAnyExisting =
-                              -- If we already have a "global" variable, then nothing needs
-                              -- to be added.
-                              Map.empty
-                          | otherwise =
-                              -- If we don't already have a "global" variable, then set LANG to the
-                              -- fallback.
-                              case mfallback of
-                                  Nothing -> Map.empty
-                                  Just fallback ->
-                                      Map.singleton "LANG" fallback
-                    return (Map.union changes adds)
+    => EnvOverride -> CompilerVersion -> m (Map Text Text)
+getUtf8EnvVars menv compilerVer = do
+    if getGhcVersion compilerVer >= $(mkVersion "7.10.3")
+        -- GHC_CHARENC supported by GHC >=7.10.3
+        then return $ Map.singleton "GHC_CHARENC" "UTF-8"
+        else legacyLocale
   where
+    legacyLocale = do
+        Platform _ os <- asks getPlatform
+        if os == Cabal.Windows
+            then
+                 -- On Windows, locale is controlled by the code page, so we don't set any environment
+                 -- variables.
+                 return
+                     Map.empty
+            else do
+                let checkedVars = map checkVar (Map.toList $ eoTextMap menv)
+                    -- List of environment variables that will need to be updated to set UTF-8 (because
+                    -- they currently do not specify UTF-8).
+                    needChangeVars = concatMap fst checkedVars
+                    -- Set of locale-related environment variables that have already have a value.
+                    existingVarNames = Set.unions (map snd checkedVars)
+                    -- True if a locale is already specified by one of the "global" locale variables.
+                    hasAnyExisting =
+                        any (`Set.member` existingVarNames) ["LANG", "LANGUAGE", "LC_ALL"]
+                if null needChangeVars && hasAnyExisting
+                    then
+                         -- If no variables need changes and at least one "global" variable is set, no
+                         -- changes to environment need to be made.
+                         return
+                             Map.empty
+                    else do
+                        -- Get a list of known locales by running @locale -a@.
+                        elocales <- tryProcessStdout Nothing menv "locale" ["-a"]
+                        let
+                            -- Filter the list to only include locales with UTF-8 encoding.
+                            utf8Locales =
+                                case elocales of
+                                    Left _ -> []
+                                    Right locales ->
+                                        filter
+                                            isUtf8Locale
+                                            (T.lines $
+                                             T.decodeUtf8With
+                                                 T.lenientDecode
+                                                 locales)
+                            mfallback = getFallbackLocale utf8Locales
+                        when
+                            (isNothing mfallback)
+                            ($logWarn
+                                 "Warning: unable to set locale to UTF-8 encoding; GHC may fail with 'invalid character'")
+                        let
+                            -- Get the new values of variables to adjust.
+                            changes =
+                                Map.unions $
+                                map
+                                    (adjustedVarValue utf8Locales mfallback)
+                                    needChangeVars
+                            -- Get the values of variables to add.
+                            adds
+                              | hasAnyExisting =
+                                  -- If we already have a "global" variable, then nothing needs
+                                  -- to be added.
+                                  Map.empty
+                              | otherwise =
+                                  -- If we don't already have a "global" variable, then set LANG to the
+                                  -- fallback.
+                                  case mfallback of
+                                      Nothing -> Map.empty
+                                      Just fallback ->
+                                          Map.singleton "LANG" fallback
+                        return (Map.union changes adds)
     -- Determines whether an environment variable is locale-related and, if so, whether it needs to
     -- be adjusted.
     checkVar
