@@ -1,3 +1,4 @@
+{-# LANGUAGE DeriveGeneric #-}
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE TemplateHaskell #-}
@@ -31,6 +32,8 @@ import qualified Data.Text as T
 import qualified Data.Text.Encoding as T
 import           Distribution.System (Platform (..))
 import qualified Distribution.System as Cabal
+import           GHC.Generics (Generic)
+import           Generics.Deriving.Monoid (mappenddefault, memptydefault)
 import           Path
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Path.IO
@@ -55,7 +58,7 @@ parseToolText (parseCompilerVersion -> Just (cv@GhcjsVersion{})) = Just (ToolGhc
 parseToolText (parsePackageIdentifierFromString . T.unpack -> Just pkgId) = Just (Tool pkgId)
 parseToolText _ = Nothing
 
-markInstalled :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m)
+markInstalled :: (MonadIO m, MonadThrow m)
               => Path Abs Dir
               -> Tool
               -> m ()
@@ -63,20 +66,20 @@ markInstalled programsPath tool = do
     fpRel <- parseRelFile $ toolString tool ++ ".installed"
     liftIO $ writeFile (toFilePath $ programsPath </> fpRel) "installed"
 
-unmarkInstalled :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m)
+unmarkInstalled :: (MonadIO m, MonadCatch m)
                 => Path Abs Dir
                 -> Tool
                 -> m ()
 unmarkInstalled programsPath tool = do
     fpRel <- parseRelFile $ toolString tool ++ ".installed"
-    removeFileIfExists $ programsPath </> fpRel
+    ignoringAbsence (removeFile $ programsPath </> fpRel)
 
-listInstalled :: (MonadIO m, MonadReader env m, HasConfig env, MonadThrow m)
+listInstalled :: (MonadIO m, MonadThrow m)
               => Path Abs Dir
               -> m [Tool]
 listInstalled programsPath = do
-    createTree programsPath
-    (_, files) <- listDirectory programsPath
+    ensureDir programsPath
+    (_, files) <- listDir programsPath
     return $ mapMaybe toTool files
   where
     toTool fp = do
@@ -88,10 +91,12 @@ getCompilerVersion :: (MonadLogger m, MonadCatch m, MonadBaseControl IO m, Monad
 getCompilerVersion menv wc =
     case wc of
         Ghc -> do
+            $logDebug "Asking GHC for its version"
             bs <- readProcessStdout Nothing menv "ghc" ["--numeric-version"]
             let (_, ghcVersion) = versionFromEnd bs
             GhcVersion <$> parseVersion (T.decodeUtf8 ghcVersion)
         Ghcjs -> do
+            $logDebug "Asking GHCJS for its version"
             -- Output looks like
             --
             -- The Glorious Glasgow Haskell Compilation System for JavaScript, version 0.1.0 (GHC 7.10.2)
@@ -117,17 +122,30 @@ extraDirs tool = do
                 , dir </> $(mkRelDir "mingw") </> $(mkRelDir "bin")
                 ]
             }
-        (Platform _ Cabal.Windows, "msys2") -> return mempty
+        (Platform Cabal.I386 Cabal.Windows, "msys2") -> return mempty
             { edBins = goList
-                [ dir </> $(mkRelDir "usr") </> $(mkRelDir "bin")
+                [ dir </> $(mkRelDir "mingw32") </> $(mkRelDir "bin")
+                , dir </> $(mkRelDir "usr") </> $(mkRelDir "bin")
+                , dir </> $(mkRelDir "usr") </> $(mkRelDir "local") </> $(mkRelDir "bin")
+                ]
+            , edInclude = goList
+                [ dir </> $(mkRelDir "mingw32") </> $(mkRelDir "include")
+                ]
+            , edLib = goList
+                [ dir </> $(mkRelDir "mingw32") </> $(mkRelDir "lib")
+                ]
+            }
+        (Platform Cabal.X86_64 Cabal.Windows, "msys2") -> return mempty
+            { edBins = goList
+                [ dir </> $(mkRelDir "mingw64") </> $(mkRelDir "bin")
+                , dir </> $(mkRelDir "usr") </> $(mkRelDir "bin")
+                , dir </> $(mkRelDir "usr") </> $(mkRelDir "local") </> $(mkRelDir "bin")
                 ]
             , edInclude = goList
                 [ dir </> $(mkRelDir "mingw64") </> $(mkRelDir "include")
-                , dir </> $(mkRelDir "mingw32") </> $(mkRelDir "include")
                 ]
             , edLib = goList
                 [ dir </> $(mkRelDir "mingw64") </> $(mkRelDir "lib")
-                , dir </> $(mkRelDir "mingw32") </> $(mkRelDir "lib")
                 ]
             }
         (_, isGHC -> True) -> return mempty
@@ -152,15 +170,12 @@ data ExtraDirs = ExtraDirs
     { edBins :: ![FilePath]
     , edInclude :: ![FilePath]
     , edLib :: ![FilePath]
-    }
+    } deriving (Show, Generic)
 instance Monoid ExtraDirs where
-    mempty = ExtraDirs [] [] []
-    mappend (ExtraDirs a b c) (ExtraDirs x y z) = ExtraDirs
-        (a ++ x)
-        (b ++ y)
-        (c ++ z)
+    mempty = memptydefault
+    mappend = mappenddefault
 
-installDir :: (MonadReader env m, HasConfig env, MonadThrow m, MonadLogger m)
+installDir :: (MonadReader env m, MonadThrow m)
            => Path Abs Dir
            -> Tool
            -> m (Path Abs Dir)
