@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {-# LANGUAGE FlexibleContexts  #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE RankNTypes        #-}
@@ -13,7 +14,10 @@ Stability   : experimental
 Portability : POSIX
 -}
 
-module Stack.Sig.Sign (sign, signTarBytes) where
+module Stack.Sig.Sign (sign, signPackage, signTarBytes) where
+
+import Prelude ()
+import Prelude.Compat
 
 import qualified Codec.Archive.Tar as Tar
 import qualified Codec.Compression.GZip as GZip
@@ -21,34 +25,34 @@ import           Control.Monad (when)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Trans.Control
 import qualified Data.ByteString.Lazy as BS
 import qualified Data.ByteString.Lazy as L
 import           Data.Monoid ((<>))
 import qualified Data.Text as T
-import           Data.UUID (toString)
-import           Data.UUID.V4 (nextRandom)
-import           Network.HTTP.Conduit
-       (Response(..), RequestBody(..), Request(..), httpLbs, newManager,
-        tlsManagerSettings)
+import           Network.HTTP.Conduit (Response(..), RequestBody(..),
+                                       Request(..), httpLbs)
+import           Network.HTTP.Client (Manager)
 import           Network.HTTP.Download
 import           Network.HTTP.Types (status200, methodPut)
 import           Path
 import           Path.IO
 import           Stack.Package
-import qualified Stack.Sig.GPG as GPG
+import           Stack.Sig.GPG
 import           Stack.Types
 import qualified System.FilePath as FP
 
 -- | Sign a haskell package with the given url of the signature
 -- service and a path to a tarball.
 sign
-    :: (MonadCatch m, MonadBaseControl IO m, MonadIO m, MonadMask m, MonadLogger m, MonadThrow m, MonadReader env m, HasConfig env)
-    => Maybe (Path Abs Dir) -> String -> Path Abs File -> m ()
-sign Nothing _ _ = throwM SigNoProjectRootException
-sign (Just projectRoot) url filePath = do
-    withStackWorkTempDir
-        projectRoot
+#if __GLASGOW_HASKELL__ < 710
+    :: (Applicative m, MonadIO m, MonadLogger m, MonadMask m)
+#else
+    :: (MonadIO m, MonadLogger m, MonadMask m)
+#endif
+    => Manager -> String -> Path Abs File -> m Signature
+sign manager url filePath =
+    withSystemTempDir
+        "stack"
         (\tempDir ->
               do bytes <-
                      liftIO
@@ -60,9 +64,9 @@ sign (Just projectRoot) url filePath = do
                      Nothing -> throwM SigInvalidSDistTarBall
                      Just cabalPath -> do
                          pkg <- cabalFilePackageId (tempDir </> cabalPath)
-                         signPackage url pkg filePath)
+                         signPackage manager url pkg filePath)
   where
-    extractCabalFile tempDir (Tar.Next entry entries) = do
+    extractCabalFile tempDir (Tar.Next entry entries) =
         case Tar.entryContent entry of
             (Tar.NormalFile lbs _) ->
                 case FP.splitFileName (Tar.entryPath entry) of
@@ -85,54 +89,42 @@ sign (Just projectRoot) url filePath = do
 -- function will write the bytes to the path in a temp dir and sign
 -- the tarball with GPG.
 signTarBytes
-    :: (MonadCatch m, MonadBaseControl IO m, MonadIO m, MonadMask m, MonadLogger m, MonadThrow m, MonadReader env m, HasConfig env)
-    => Maybe (Path Abs Dir) -> String -> Path Rel File -> L.ByteString -> m ()
-signTarBytes Nothing _ _ _ = throwM SigNoProjectRootException
-signTarBytes (Just projectRoot) url tarPath bs =
-    withStackWorkTempDir
-        projectRoot
+#if __GLASGOW_HASKELL__ < 710
+    :: (Applicative m, MonadIO m, MonadLogger m, MonadMask m)
+#else
+    :: (MonadIO m, MonadLogger m, MonadMask m)
+#endif
+    => Manager -> String -> Path Rel File -> L.ByteString -> m Signature
+signTarBytes manager url tarPath bs =
+    withSystemTempDir
+        "stack"
         (\tempDir ->
               do let tempTarBall = tempDir </> tarPath
                  liftIO (L.writeFile (toFilePath tempTarBall) bs)
-                 sign (Just projectRoot) url tempTarBall)
+                 sign manager url tempTarBall)
 
 -- | Sign a haskell package given the url to the signature service, a
 -- @PackageIdentifier@ and a file path to the package on disk.
 signPackage
-    :: (MonadCatch m, MonadBaseControl IO m, MonadIO m, MonadMask m, MonadLogger m, MonadThrow m)
-    => String -> PackageIdentifier -> Path Abs File -> m ()
-signPackage url pkg filePath = do
-    $logInfo ("GPG signing " <> T.pack (toFilePath filePath))
-    sig@(Signature signature) <- GPG.signPackage filePath
-    let (PackageIdentifier n v) = pkg
-        name = show n
-        version = show v
-    verify <- GPG.verifyFile sig filePath
-    fingerprint <- GPG.fullFingerprint verify
-    req <-
-        parseUrl
-            (url <> "/upload/signature/" <> name <> "/" <> version <> "/" <>
-             T.unpack (fingerprintSample fingerprint))
+    :: (MonadIO m, MonadLogger m, MonadThrow m)
+    => Manager -> String -> PackageIdentifier -> Path Abs File -> m Signature
+signPackage manager url pkg filePath = do
+    sig@(Signature signature) <- gpgSign filePath
+    let (PackageIdentifier name version) = pkg
+    fingerprint <- gpgVerify sig filePath
+    let fullUrl =
+            url <> "/upload/signature/" <> show name <> "/" <> show version <>
+            "/" <>
+            show fingerprint
+    req <- parseUrl fullUrl
     let put =
             req
             { method = methodPut
             , requestBody = RequestBodyBS signature
             }
-    mgr <- liftIO (newManager tlsManagerSettings)
-    res <- liftIO (httpLbs put mgr)
+    res <- liftIO (httpLbs put manager)
     when
         (responseStatus res /= status200)
         (throwM (GPGSignException "unable to sign & upload package"))
-
-withStackWorkTempDir
-    :: (MonadCatch m, MonadIO m, MonadMask m, MonadLogger m, MonadReader env m, HasConfig env)
-    => Path Abs Dir -> (Path Abs Dir -> m ()) -> m ()
-withStackWorkTempDir projectRoot f = do
-    uuid <- liftIO nextRandom
-    uuidPath <- parseRelDir (toString uuid)
-    workDir <- getWorkDir
-    let tempDir = projectRoot </> workDir </> $(mkRelDir "tmp") </> uuidPath
-    bracket
-        (createTree tempDir)
-        (const (removeTree tempDir))
-        (const (f tempDir))
+    $logInfo ("Signature uploaded to " <> T.pack fullUrl)
+    return sig
