@@ -34,6 +34,7 @@ module Stack.Types.Build
     ,ConfigCache(..)
     ,ConstructPlanException(..)
     ,configureOpts
+    ,isStackOpt
     ,BadDependency(..)
     ,wantedLocalPackages
     ,FileCacheInfo (..)
@@ -66,14 +67,15 @@ import           Data.Time.Clock
 import           Distribution.System (Arch)
 import           Distribution.PackageDescription (TestSuiteInterface)
 import           Distribution.Text (display)
-import           GHC.Generics
+import           GHC.Generics (Generic, from, to)
 import           Path (Path, Abs, File, Dir, mkRelDir, toFilePath, parseRelDir, (</>))
 import           Path.Extra (toFilePathNoTrailingSep)
 import           Prelude
-import           Stack.Types.FlagName
-import           Stack.Types.GhcPkgId
+import           Stack.Types.BuildPlan (GitSHA1)
 import           Stack.Types.Compiler
 import           Stack.Types.Config
+import           Stack.Types.FlagName
+import           Stack.Types.GhcPkgId
 import           Stack.Types.Package
 import           Stack.Types.PackageIdentifier
 import           Stack.Types.PackageName
@@ -120,10 +122,11 @@ data StackBuildException
   | InvalidFlagSpecification (Set UnusedFlags)
   | TargetParseException [Text]
   | DuplicateLocalPackageNames [(PackageName, [Path Abs Dir])]
+  | SolverGiveUp String
   | SolverMissingCabalInstall
-  | SolverMissingGHC
-  | SolverNoCabalFiles
   | SomeTargetsNotBuildable [(PackageName, NamedComponent)]
+  | TestSuiteExeMissing Bool String String String
+  | CabalCopyFailed Bool String
   deriving Typeable
 
 data FlagSource = FSCommandLine | FSStackYaml
@@ -322,28 +325,54 @@ instance Show StackBuildException where
             : (packageNameString name ++ " used in:")
             : map goDir dirs
         goDir dir = "- " ++ toFilePath dir
+    show (SolverGiveUp msg) = concat
+        [ "\nSolver could not resolve package dependencies.\n"
+        , "You can try the following:\n"
+        , msg
+        ]
     show SolverMissingCabalInstall = unlines
         [ "Solver requires that cabal be on your PATH"
         , "Try running 'stack install cabal-install'"
         ]
-    show SolverMissingGHC = unlines
-        [ "Solver requires that GHC be on your PATH"
-        , "Try running 'stack setup'"
-        ]
-    show SolverNoCabalFiles = unlines
-        [ "No cabal files provided.  Maybe this is due to not having a stack.yaml file?"
-        , "Try running 'stack init' to create a stack.yaml"
-        ]
     show (SomeTargetsNotBuildable xs) =
         "The following components have 'buildable: False' set in the cabal configuration, and so cannot be targets:\n    " ++
         T.unpack (renderPkgComponents xs) ++
         "\nTo resolve this, either provide flags such that these components are buildable, or only specify buildable targets."
+    show (TestSuiteExeMissing isSimpleBuildType exeName pkgName testName) =
+        missingExeError isSimpleBuildType $ concat
+            [ "Test suite executable \""
+            , exeName
+            , " not found for "
+            , pkgName
+            , ":test:"
+            , testName
+            ]
+    show (CabalCopyFailed isSimpleBuildType innerMsg) =
+        missingExeError isSimpleBuildType $ concat
+            [ "'cabal copy' failed.  Error message:\n"
+            , innerMsg
+            , "\n"
+            ]
+
+missingExeError :: Bool -> String -> String
+missingExeError isSimpleBuildType msg =
+    unlines $ msg :
+        case possibleCauses of
+            [] -> []
+            [cause] -> ["One possible cause of this issue is:\n* " <> cause]
+            _ -> "Possible causes of this issue:" : map ("* " <>) possibleCauses
+  where
+    possibleCauses =
+        "No module named \"Main\". The 'main-is' source file should usually have a header indicating that it's a 'Main' module." :
+        if isSimpleBuildType
+            then []
+            else ["The Setup.hs file is changing the installation target dir."]
 
 instance Exception StackBuildException
 
 data ConstructPlanException
     = DependencyCycleDetected [PackageName]
-    | DependencyPlanFailures PackageIdentifier (Map PackageName (VersionRange, LatestApplicableVersion, BadDependency))
+    | DependencyPlanFailures Package (Map PackageName (VersionRange, LatestApplicableVersion, BadDependency))
     | UnknownPackage PackageName -- TODO perhaps this constructor will be removed, and BadDependency will handle it all
     -- ^ Recommend adding to extra-deps, give a helpful version number?
     deriving (Typeable, Eq)
@@ -364,9 +393,10 @@ instance Show ConstructPlanException where
          (DependencyCycleDetected pNames) ->
            "While checking call stack,\n" ++
            "  dependency cycle detected in packages:" ++ indent (appendLines pNames)
-         (DependencyPlanFailures pIdent (Map.toList -> pDeps)) ->
+         (DependencyPlanFailures pkg (Map.toList -> pDeps)) ->
            "Failure when adding dependencies:" ++ doubleIndent (appendDeps pDeps) ++ "\n" ++
-           "  needed for package: " ++ packageIdentifierString pIdent
+           "  needed for package " ++ packageIdentifierString (packageIdentifier pkg) ++
+           appendFlags (packageFlags pkg)
          (UnknownPackage pName) ->
              "While attempting to add dependency,\n" ++
              "  Could not find package " ++ show pName  ++ " in known packages"
@@ -375,6 +405,12 @@ instance Show ConstructPlanException where
       appendLines = foldr (\pName-> (++) ("\n" ++ show pName)) ""
       indent = dropWhileEnd isSpace . unlines . fmap (\line -> "  " ++ line) . lines
       doubleIndent = indent . indent
+      appendFlags flags =
+          if Map.null flags
+              then ""
+              else " with flags:\n" ++
+                  (doubleIndent . intercalate "\n" . map showFlag . Map.toList) flags
+      showFlag (name, bool) = show name ++ ": " ++ show bool
       appendDeps = foldr (\dep-> (++) ("\n" ++ showDep dep)) ""
       showDep (name, (range, mlatestApplicable, badDep)) = concat
         [ show name
@@ -387,7 +423,7 @@ instance Show ConstructPlanException where
                     Nothing -> ""
                     Just la -> " (latest applicable is " ++ versionString la ++ ")"
            in case badDep of
-                NotInBuildPlan -> "not present in build plan" ++ latestApplicableStr
+                NotInBuildPlan -> "stack configuration has no specified version" ++ latestApplicableStr
                 Couldn'tResolveItsDependencies -> "couldn't resolve its dependencies"
                 DependencyMismatch version ->
                     case mlatestApplicable of
@@ -410,118 +446,6 @@ instance Show ConstructPlanException where
 
 ----------------------------------------------
 
--- | Which subset of packages to build
-data BuildSubset
-    = BSAll
-    | BSOnlySnapshot
-    -- ^ Only install packages in the snapshot database, skipping
-    -- packages intended for the local database.
-    | BSOnlyDependencies
-    deriving (Show, Eq)
-
--- | Configuration for building.
-data BuildOpts =
-  BuildOpts {boptsTargets :: ![Text]
-            ,boptsLibProfile :: !Bool
-            ,boptsExeProfile :: !Bool
-            ,boptsHaddock :: !Bool
-            -- ^ Build haddocks?
-            ,boptsHaddockDeps :: !(Maybe Bool)
-            -- ^ Build haddocks for dependencies?
-            ,boptsDryrun :: !Bool
-            ,boptsGhcOptions :: ![Text]
-            ,boptsFlags :: !(Map (Maybe PackageName) (Map FlagName Bool))
-            ,boptsInstallExes :: !Bool
-            -- ^ Install executables to user path after building?
-            ,boptsPreFetch :: !Bool
-            -- ^ Fetch all packages immediately
-            ,boptsBuildSubset :: !BuildSubset
-            ,boptsFileWatch :: !FileWatchOpts
-            -- ^ Watch files for changes and automatically rebuild
-            ,boptsKeepGoing :: !(Maybe Bool)
-            -- ^ Keep building/running after failure
-            ,boptsForceDirty :: !Bool
-            -- ^ Force treating all local packages as having dirty files
-
-            ,boptsTests :: !Bool
-            -- ^ Turn on tests for local targets
-            ,boptsTestOpts :: !TestOpts
-            -- ^ Additional test arguments
-
-            ,boptsBenchmarks :: !Bool
-            -- ^ Turn on benchmarks for local targets
-            ,boptsBenchmarkOpts :: !BenchmarkOpts
-            -- ^ Additional test arguments
-            ,boptsExec :: ![(String, [String])]
-            -- ^ Commands (with arguments) to run after a successful build
-            ,boptsOnlyConfigure :: !Bool
-            -- ^ Only perform the configure step when building
-            ,boptsReconfigure :: !Bool
-            -- ^ Perform the configure step even if already configured
-            ,boptsCabalVerbose :: !Bool
-            -- ^ Ask Cabal to be verbose in its builds
-            }
-  deriving (Show)
-
-defaultBuildOpts :: BuildOpts
-defaultBuildOpts = BuildOpts
-    { boptsTargets = []
-    , boptsLibProfile = False
-    , boptsExeProfile = False
-    , boptsHaddock = False
-    , boptsHaddockDeps = Nothing
-    , boptsDryrun = False
-    , boptsGhcOptions = []
-    , boptsFlags = Map.empty
-    , boptsInstallExes = False
-    , boptsPreFetch = False
-    , boptsBuildSubset = BSAll
-    , boptsFileWatch = NoFileWatch
-    , boptsKeepGoing = Nothing
-    , boptsForceDirty = False
-    , boptsTests = False
-    , boptsTestOpts = defaultTestOpts
-    , boptsBenchmarks = False
-    , boptsBenchmarkOpts = defaultBenchmarkOpts
-    , boptsExec = []
-    , boptsOnlyConfigure = False
-    , boptsReconfigure = False
-    , boptsCabalVerbose = False
-    }
-
--- | Options for the 'FinalAction' 'DoTests'
-data TestOpts =
-  TestOpts {toRerunTests :: !Bool -- ^ Whether successful tests will be run gain
-           ,toAdditionalArgs :: ![String] -- ^ Arguments passed to the test program
-           ,toCoverage :: !Bool -- ^ Generate a code coverage report
-           ,toDisableRun :: !Bool -- ^ Disable running of tests
-           } deriving (Eq,Show)
-
-defaultTestOpts :: TestOpts
-defaultTestOpts = TestOpts
-    { toRerunTests = True
-    , toAdditionalArgs = []
-    , toCoverage = False
-    , toDisableRun = False
-    }
-
--- | Options for the 'FinalAction' 'DoBenchmarks'
-data BenchmarkOpts =
-  BenchmarkOpts {beoAdditionalArgs :: !(Maybe String) -- ^ Arguments passed to the benchmark program
-                ,beoDisableRun :: !Bool -- ^ Disable running of benchmarks
-                } deriving (Eq,Show)
-
-defaultBenchmarkOpts :: BenchmarkOpts
-defaultBenchmarkOpts = BenchmarkOpts
-    { beoAdditionalArgs = Nothing
-    , beoDisableRun = False
-    }
-
-data FileWatchOpts
-  = NoFileWatch
-  | FileWatch
-  | FileWatchPoll
-  deriving (Show,Eq)
 
 -- | Package dependency oracle.
 newtype PkgDepsOracle =
@@ -596,14 +520,14 @@ instance Show TaskConfigOpts where
 -- | The type of a task, either building local code or something from the
 -- package index (upstream)
 data TaskType = TTLocal LocalPackage
-              | TTUpstream Package InstallLocation
+              | TTUpstream Package InstallLocation (Maybe GitSHA1)
     deriving Show
 
 taskLocation :: Task -> InstallLocation
 taskLocation task =
     case taskType task of
         TTLocal _ -> Local
-        TTUpstream _ loc -> loc
+        TTUpstream _ loc _ -> loc
 
 -- | A complete plan of what needs to be built and how to do it
 data Plan = Plan
@@ -624,23 +548,47 @@ data BaseConfigOpts = BaseConfigOpts
     , bcoSnapInstallRoot :: !(Path Abs Dir)
     , bcoLocalInstallRoot :: !(Path Abs Dir)
     , bcoBuildOpts :: !BuildOpts
+    , bcoBuildOptsCLI :: !BuildOptsCLI
     , bcoExtraDBs :: ![(Path Abs Dir)]
     }
+    deriving Show
 
 -- | Render a @BaseConfigOpts@ to an actual list of options
 configureOpts :: EnvConfig
               -> BaseConfigOpts
               -> Map PackageIdentifier GhcPkgId -- ^ dependencies
-              -> Bool -- ^ wanted?
               -> Bool -- ^ local non-extra-dep?
               -> InstallLocation
               -> Package
               -> ConfigureOpts
-configureOpts econfig bco deps wanted isLocal loc package = ConfigureOpts
+configureOpts econfig bco deps isLocal loc package = ConfigureOpts
     { coDirs = configureOptsDirs bco loc package
-    , coNoDirs = configureOptsNoDir econfig bco deps wanted isLocal package
+    , coNoDirs = configureOptsNoDir econfig bco deps isLocal package
     }
 
+-- options set by stack
+isStackOpt :: Text -> Bool
+isStackOpt t = any (`T.isPrefixOf` t)
+    [ "--dependency="
+    , "--constraint="
+    , "--package-db="
+    , "--libdir="
+    , "--bindir="
+    , "--datadir="
+    , "--libexecdir="
+    , "--sysconfdir"
+    , "--docdir="
+    , "--htmldir="
+    , "--haddockdir="
+    , "--enable-tests"
+    , "--enable-benchmarks"
+    , "--enable-library-profiling"
+    , "--enable-executable-profiling"
+    , "--exact-configuration"
+    ] || elem t
+    [ "--user"
+    ]
+
 configureOptsDirs :: BaseConfigOpts
                   -> InstallLocation
                   -> Package
@@ -677,38 +625,48 @@ configureOptsDirs bco loc package = concat
 configureOptsNoDir :: EnvConfig
                    -> BaseConfigOpts
                    -> Map PackageIdentifier GhcPkgId -- ^ dependencies
-                   -> Bool -- ^ wanted?
                    -> Bool -- ^ is this a local, non-extra-dep?
                    -> Package
                    -> [String]
-configureOptsNoDir econfig bco deps wanted isLocal package = concat
+configureOptsNoDir econfig bco deps isLocal package = concat
     [ depOptions
     , ["--enable-library-profiling" | boptsLibProfile bopts || boptsExeProfile bopts]
-    , ["--enable-executable-profiling" | boptsExeProfile bopts]
+    , ["--enable-executable-profiling" | boptsExeProfile bopts && isLocal]
+    , ["--enable-split-objs" | boptsSplitObjs bopts]
     , map (\(name,enabled) ->
                        "-f" <>
                        (if enabled
                            then ""
                            else "-") <>
                        flagNameString name)
-                    (Map.toList (packageFlags package))
-    , concatMap (\x -> ["--ghc-options", T.unpack x]) allGhcOptions
+                    (Map.toList flags)
+    , concatMap (\x -> ["--ghc-options", T.unpack x]) (packageGhcOptions package)
     , map (("--extra-include-dirs=" ++) . T.unpack) (Set.toList (configExtraIncludeDirs config))
     , map (("--extra-lib-dirs=" ++) . T.unpack) (Set.toList (configExtraLibDirs config))
     , if whichCompiler (envConfigCompilerVersion econfig) == Ghcjs
         then ["--ghcjs"]
         else []
+    , if useExactConf then ["--exact-configuration"] else []
     ]
   where
     config = getConfig econfig
     bopts = bcoBuildOpts bco
 
+    -- TODO: instead always enable this when the cabal version is new
+    -- enough. That way we'll detect bugs with --exact-configuration
+    -- earlier. Cabal also might do less work then.
+    useExactConf = configAllowNewer config
+
+    newerCabal = envConfigCabalVersion econfig >= $(mkVersion "1.22")
+
+    -- Unioning atop defaults is needed so that all flags are specified
+    -- with --exact-configuration.
+    flags | useExactConf = packageFlags package `Map.union` packageDefaultFlags package
+          | otherwise = packageFlags package
+
     depOptions = map (uncurry toDepOption) $ Map.toList deps
       where
-        toDepOption =
-            if envConfigCabalVersion econfig >= $(mkVersion "1.22")
-                then toDepOption1_22
-                else toDepOption1_18
+        toDepOption = if newerCabal then toDepOption1_22 else toDepOption1_18
 
     toDepOption1_22 ident gid = concat
         [ "--dependency="
@@ -726,21 +684,6 @@ configureOptsNoDir econfig bco deps wanted isLocal package = concat
       where
         PackageIdentifier name version = ident
 
-    allGhcOptions = concat
-        [ Map.findWithDefault [] Nothing (configGhcOptions config)
-        , Map.findWithDefault [] (Just $ packageName package) (configGhcOptions config)
-        , concat [["-fhpc"] | isLocal && toCoverage (boptsTestOpts bopts)]
-        , if includeExtraOptions
-            then boptsGhcOptions bopts
-            else []
-        ]
-
-    includeExtraOptions =
-        case configApplyGhcOptions config of
-            AGOTargets -> wanted
-            AGOLocals -> isLocal
-            AGOEverything -> True
-
 -- | Get set of wanted package names from locals.
 wantedLocalPackages :: [LocalPackage] -> Set PackageName
 wantedLocalPackages = Set.fromList . map (packageName . lpPackage) . filter lpWanted
