@@ -15,13 +15,18 @@ module Stack.Types.BuildPlan
     , Maintainer (..)
     , ExeName (..)
     , SimpleDesc (..)
+    , Snapshots (..)
     , DepInfo (..)
     , Component (..)
     , SnapName (..)
     , MiniBuildPlan (..)
     , MiniPackageInfo (..)
+    , CabalFileInfo (..)
+    , GitSHA1 (..)
     , renderSnapName
     , parseSnapName
+    , SnapshotHash (..)
+    , trimmedSnapshotHash
     ) where
 
 import           Control.Applicative
@@ -32,8 +37,12 @@ import           Data.Aeson                      (FromJSON (..), ToJSON (..),
                                                   object, withObject, withText,
                                                   (.!=), (.:), (.:?), (.=))
 import           Data.Binary.VersionTagged
+import           Data.ByteString                 (ByteString)
+import qualified Data.ByteString                 as BS
 import           Data.Hashable                   (Hashable)
 import qualified Data.HashMap.Strict             as HashMap
+import           Data.IntMap                     (IntMap)
+import qualified Data.IntMap                     as IntMap
 import           Data.Map                        (Map)
 import qualified Data.Map                        as Map
 import           Data.Maybe                      (fromMaybe)
@@ -98,6 +107,7 @@ instance FromJSON BuildPlan where
 
 data PackagePlan = PackagePlan
     { ppVersion     :: Version
+    , ppCabalFileInfo :: Maybe CabalFileInfo
     , ppGithubPings :: Set Text
     , ppUsers       :: Set PackageName
     , ppConstraints :: PackageConstraints
@@ -107,6 +117,7 @@ data PackagePlan = PackagePlan
 
 instance ToJSON PackagePlan where
     toJSON PackagePlan {..} = object
+        $ maybe id (\cfi -> (("cabal-file-info" .= cfi):)) ppCabalFileInfo $
         [ "version"      .= ppVersion
         , "github-pings" .= ppGithubPings
         , "users"        .= ppUsers
@@ -116,12 +127,32 @@ instance ToJSON PackagePlan where
 instance FromJSON PackagePlan where
     parseJSON = withObject "PackageBuild" $ \o -> do
         ppVersion <- o .: "version"
+        ppCabalFileInfo <- o .:? "cabal-file-info"
         ppGithubPings <- o .:? "github-pings" .!= mempty
         ppUsers <- o .:? "users" .!= mempty
         ppConstraints <- o .: "constraints"
         ppDesc <- o .: "description"
         return PackagePlan {..}
 
+-- | Information on the contents of a cabal file
+data CabalFileInfo = CabalFileInfo
+    { cfiSize :: !Int
+    -- ^ File size in bytes
+    , cfiHashes :: !(Map.Map Text Text)
+    -- ^ Various hashes of the file contents
+    }
+    deriving (Show, Eq, Generic)
+instance ToJSON CabalFileInfo where
+    toJSON CabalFileInfo {..} = object
+        [ "size" .= cfiSize
+        , "hashes" .= cfiHashes
+        ]
+instance FromJSON CabalFileInfo where
+    parseJSON = withObject "CabalFileInfo" $ \o -> do
+        cfiSize <- o .: "size"
+        cfiHashes <- o .: "hashes"
+        return CabalFileInfo {..}
+
 display :: DT.Text a => a -> Text
 display = fromString . DT.display
 
@@ -359,6 +390,34 @@ parseSnapName t0 =
         t1 <- T.stripPrefix "nightly-" t0
         Nightly <$> readMay (T.unpack t1)
 
+-- | Most recent Nightly and newest LTS version per major release.
+data Snapshots = Snapshots
+    { snapshotsNightly :: !Day
+    , snapshotsLts     :: !(IntMap Int)
+    }
+    deriving Show
+instance FromJSON Snapshots where
+    parseJSON = withObject "Snapshots" $ \o -> Snapshots
+        <$> (o .: "nightly" >>= parseNightly)
+        <*> (fmap IntMap.unions
+                $ mapM (parseLTS . snd)
+                $ filter (isLTS . fst)
+                $ HashMap.toList o)
+      where
+        parseNightly t =
+            case parseSnapName t of
+                Left e -> fail $ show e
+                Right (LTS _ _) -> fail "Unexpected LTS value"
+                Right (Nightly d) -> return d
+
+        isLTS = ("lts-" `T.isPrefixOf`)
+
+        parseLTS = withText "LTS" $ \t ->
+            case parseSnapName t of
+                Left e -> fail $ show e
+                Right (LTS x y) -> return $ IntMap.singleton x y
+                Right (Nightly _) -> fail "Unexpected nightly value"
+
 instance ToJSON a => ToJSON (Map ExeName a) where
   toJSON = toJSON . Map.mapKeysWith const unExeName
 instance FromJSON a => FromJSON (Map ExeName a) where
@@ -379,6 +438,7 @@ instance HasSemanticVersion MiniBuildPlan
 data MiniPackageInfo = MiniPackageInfo
     { mpiVersion :: !Version
     , mpiFlags :: !(Map FlagName Bool)
+    , mpiGhcOptions :: ![Text]
     , mpiPackageDeps :: !(Set PackageName)
     , mpiToolDeps :: !(Set Text)
     -- ^ Due to ambiguity in Cabal, it is unclear whether this refers to the
@@ -389,8 +449,21 @@ data MiniPackageInfo = MiniPackageInfo
     -- ^ Executables provided by this package
     , mpiHasLibrary :: !Bool
     -- ^ Is there a library present?
+    , mpiGitSHA1 :: !(Maybe GitSHA1)
+    -- ^ An optional SHA1 representation in hex format of the blob containing
+    -- the cabal file contents. Useful for grabbing the correct cabal file
+    -- revision directly from a Git repo
     }
     deriving (Generic, Show, Eq)
 instance Binary MiniPackageInfo
 instance HasStructuralInfo MiniPackageInfo
 instance NFData MiniPackageInfo
+
+newtype GitSHA1 = GitSHA1 ByteString
+    deriving (Generic, Show, Eq, NFData, HasStructuralInfo, Binary)
+
+newtype SnapshotHash = SnapshotHash { unShapshotHash :: ByteString }
+    deriving (Generic, Show, Eq)
+
+trimmedSnapshotHash :: SnapshotHash -> ByteString
+trimmedSnapshotHash = BS.take 12 . unShapshotHash
