@@ -13,27 +13,26 @@ module Stack.Types.GhcPkgId
 import           Control.Applicative
 import           Control.Monad.Catch
 import           Data.Aeson.Extended
-import           Data.Attoparsec.ByteString.Char8 as A8
+import           Data.Attoparsec.Text
 import           Data.Binary (getWord8, putWord8)
 import           Data.Binary.VersionTagged
-import           Data.ByteString.Char8 (ByteString)
-import qualified Data.ByteString.Char8 as S8
 import           Data.Data
 import           Data.Hashable
-import           Data.Text.Encoding (encodeUtf8)
+import           Data.Text (Text)
+import qualified Data.Text as T
 import           GHC.Generics
 import           Prelude -- Fix AMP warning
 
 -- | A parse fail.
 data GhcPkgIdParseFail
-  = GhcPkgIdParseFail ByteString
+  = GhcPkgIdParseFail Text
   deriving Typeable
 instance Show GhcPkgIdParseFail where
     show (GhcPkgIdParseFail bs) = "Invalid package ID: " ++ show bs
 instance Exception GhcPkgIdParseFail
 
 -- | A ghc-pkg package identifier.
-newtype GhcPkgId = GhcPkgId ByteString
+newtype GhcPkgId = GhcPkgId Text
   deriving (Eq,Ord,Data,Typeable,Generic)
 
 instance Hashable GhcPkgId
@@ -59,7 +58,7 @@ instance Show GhcPkgId where
 
 instance FromJSON GhcPkgId where
   parseJSON = withText "GhcPkgId" $ \t ->
-    case parseGhcPkgId $ encodeUtf8 t of
+    case parseGhcPkgId t of
       Left e -> fail $ show (e, t)
       Right x -> return x
 
@@ -67,8 +66,8 @@ instance ToJSON GhcPkgId where
   toJSON g =
     toJSON (ghcPkgIdString g)
 
--- | Convenient way to parse a package name from a bytestring.
-parseGhcPkgId :: MonadThrow m => ByteString -> m GhcPkgId
+-- | Convenient way to parse a package name from a 'Text'.
+parseGhcPkgId :: MonadThrow m => Text -> m GhcPkgId
 parseGhcPkgId x = go x
   where go =
           either (const (throwM (GhcPkgIdParseFail x))) return .
@@ -77,16 +76,8 @@ parseGhcPkgId x = go x
 -- | A parser for a package-version-hash pair.
 ghcPkgIdParser :: Parser GhcPkgId
 ghcPkgIdParser =
-    fmap GhcPkgId (A8.takeWhile isValid)
-  where
-    isValid c =
-        ('A' <= c && c <= 'Z') ||
-        ('a' <= c && c <= 'z') ||
-        ('0' <= c && c <= '9') ||
-        c == '.' ||
-        c == '-' ||
-        c == '_'
+    GhcPkgId . T.pack <$> many1 (choice [digit, letter, satisfy (`elem` "_.-")])
 
 -- | Get a string representation of GHC package id.
 ghcPkgIdString :: GhcPkgId -> String
-ghcPkgIdString (GhcPkgId x) = S8.unpack x
+ghcPkgIdString (GhcPkgId x) = T.unpack x
