@@ -21,15 +21,12 @@ import           Control.DeepSeq
 import           Control.Exception (Exception)
 import           Control.Monad.Catch (MonadThrow, throwM)
 import           Data.Aeson.Extended
-import           Data.Attoparsec.ByteString.Char8
+import           Data.Attoparsec.Text
 import           Data.Binary.VersionTagged (Binary, HasStructuralInfo)
-import           Data.ByteString (ByteString)
-import qualified Data.ByteString.Char8 as S8
 import           Data.Data
 import           Data.Hashable
 import           Data.Text (Text)
 import qualified Data.Text as T
-import           Data.Text.Encoding (encodeUtf8)
 import           GHC.Generics
 import           Prelude hiding (FilePath)
 import           Stack.Types.PackageName
@@ -37,7 +34,7 @@ import           Stack.Types.Version
 
 -- | A parse fail.
 data PackageIdentifierParseFail
-  = PackageIdentifierParseFail ByteString
+  = PackageIdentifierParseFail Text
   deriving (Typeable)
 instance Show PackageIdentifierParseFail where
     show (PackageIdentifierParseFail bs) = "Invalid package identifier: " ++ show bs
@@ -66,7 +63,7 @@ instance ToJSON PackageIdentifier where
   toJSON = toJSON . packageIdentifierString
 instance FromJSON PackageIdentifier where
   parseJSON = withText "PackageIdentifier" $ \t ->
-    case parsePackageIdentifier $ encodeUtf8 t of
+    case parsePackageIdentifier t of
       Left e -> fail $ show (e, t)
       Right x -> return x
 
@@ -82,21 +79,21 @@ fromTuple (n,v) = PackageIdentifier n v
 packageIdentifierParser :: Parser PackageIdentifier
 packageIdentifierParser =
   do name <- packageNameParser
-     char8 '-'
+     char '-'
      version <- versionParser
      return (PackageIdentifier name version)
 
--- | Convenient way to parse a package identifier from a bytestring.
-parsePackageIdentifier :: MonadThrow m => ByteString -> m PackageIdentifier
+-- | Convenient way to parse a package identifier from a 'Text'.
+parsePackageIdentifier :: MonadThrow m => Text -> m PackageIdentifier
 parsePackageIdentifier x = go x
   where go =
           either (const (throwM (PackageIdentifierParseFail x))) return .
           parseOnly (packageIdentifierParser <* endOfInput)
 
--- | Migration function.
+-- | Convenience function for parsing from a 'String'.
 parsePackageIdentifierFromString :: MonadThrow m => String -> m PackageIdentifier
 parsePackageIdentifierFromString =
-  parsePackageIdentifier . S8.pack
+  parsePackageIdentifier . T.pack
 
 -- | Get a string representation of the package identifier; name-ver.
 packageIdentifierString :: PackageIdentifier -> String
