@@ -8,21 +8,19 @@ import           Control.Monad               (when)
 import           Control.Monad.Catch
 import           Control.Monad.IO.Class
 import           Control.Monad.Logger
-import           Control.Monad.Reader        (MonadReader, asks)
+import           Control.Monad.Reader        (MonadReader)
 import           Control.Monad.Trans.Control
 import           Data.Foldable               (forM_)
 import qualified Data.Map                    as Map
 import           Data.Maybe                  (isNothing)
-import           Data.Monoid                 ((<>))
-import qualified Data.Monoid
-import qualified Data.Set                    as Set
+import           Data.Monoid.Extra
 import qualified Data.Text as T
+import           Lens.Micro                  (set)
 import           Network.HTTP.Client.Conduit (HasHttpManager)
 import           Path
 import           Path.IO
 import qualified Paths_stack as Paths
 import           Stack.Build
-import           Stack.Types.Build
 import           Stack.Config
 import           Stack.Fetch
 import           Stack.PackageIndex
@@ -34,12 +32,13 @@ import           System.Process              (readProcess)
 import           System.Process.Run
 
 upgrade :: (MonadIO m, MonadMask m, MonadReader env m, HasConfig env, HasHttpManager env, MonadLogger m, HasTerminal env, HasReExec env, HasLogLevel env, MonadBaseControl IO m)
-        => Maybe String -- ^ git repository to use
+        => ConfigMonoid
+        -> Maybe String -- ^ git repository to use
         -> Maybe AbstractResolver
         -> Maybe String -- ^ git hash at time of building, if known
         -> m ()
-upgrade gitRepo mresolver builtHash =
-  withCanonicalizedSystemTempDirectory "stack-upgrade" $ \tmp -> do
+upgrade gConfigMonoid gitRepo mresolver builtHash =
+  withSystemTempDir "stack-upgrade" $ \tmp -> do
     menv <- getMinimalEnvOverride
     mdir <- case gitRepo of
       Just repo -> do
@@ -56,12 +55,16 @@ upgrade gitRepo mresolver builtHash =
                 return Nothing
             else do
                 $logInfo "Cloning stack"
-                let args = [ "clone", repo , "stack", "--depth", "1"]
+                -- NOTE: "--recursive" was added after v1.0.0 (and before the
+                -- next release).  This means that we can't use submodules in
+                -- the stack repo until we're comfortable with "stack upgrade
+                -- --git" not working for earlier versions.
+                let args = [ "clone", repo , "stack", "--depth", "1", "--recursive"]
                 runCmd (Cmd (Just tmp) "git" menv args) Nothing
                 return $ Just $ tmp </> $(mkRelDir "stack")
       Nothing -> do
         updateAllIndices menv
-        caches <- getPackageCaches menv
+        caches <- getPackageCaches
         let latest = Map.fromListWith max
                    $ map toTuple
                    $ Map.keys
@@ -79,26 +82,23 @@ upgrade gitRepo mresolver builtHash =
                 return Nothing
             Just version -> do
                 let ident = PackageIdentifier $(mkPackageName "stack") version
-                paths <- unpackPackageIdents menv tmp Nothing $ Set.singleton ident
+                paths <- unpackPackageIdents menv tmp Nothing
+                    -- accept latest cabal revision by not supplying a Git SHA
+                    $ Map.singleton ident Nothing
                 case Map.lookup ident paths of
                     Nothing -> error "Stack.Upgrade.upgrade: invariant violated, unpacked directory not found"
                     Just path -> return $ Just path
 
-    config <- asks getConfig
     forM_ mdir $ \dir -> do
-        bconfig <- runInnerStackLoggingT $ do
-            lc <- loadConfig
-                (configConfigMonoid config <> Data.Monoid.mempty
-                    { configMonoidInstallGHC = Just True
-                    })
-                (Just $ dir </> $(mkRelFile "stack.yaml"))
-                mresolver
-            lcLoadBuildConfig lc Nothing
+        lc <- loadConfig
+            gConfigMonoid
+            mresolver
+            (Just $ dir </> $(mkRelFile "stack.yaml"))
+        bconfig <- lcLoadBuildConfig lc Nothing
         envConfig1 <- runInnerStackT bconfig $ setupEnv $ Just $
             "Try rerunning with --install-ghc to install the correct GHC into " <>
-            T.pack (toFilePath (configLocalPrograms config))
-        runInnerStackT envConfig1 $
-            build (const $ return ()) Nothing defaultBuildOpts
-                { boptsTargets = ["stack"]
-                , boptsInstallExes = True
+            T.pack (toFilePath (configLocalPrograms (getConfig bconfig)))
+        runInnerStackT (set (envConfigBuildOpts.buildOptsInstallExes) True envConfig1) $
+            build (const $ return ()) Nothing defaultBuildOptsCLI
+                { boptsCLITargets = ["stack"]
                 }
