@@ -83,10 +83,12 @@ spec = beforeAll setup $ afterAll teardown $ do
             mkMPI deps = MiniPackageInfo
                 { mpiVersion = version
                 , mpiFlags = Map.empty
+                , mpiGhcOptions = []
                 , mpiPackageDeps = Set.fromList $ map pn $ words deps
                 , mpiToolDeps = Set.empty
                 , mpiExes = Set.empty
                 , mpiHasLibrary = True
+                , mpiGitSHA1 = Nothing
                 }
             go x y = (pn x, mkMPI y)
             resourcet = go "resourcet" ""
