@@ -27,6 +27,32 @@ sampleConfig =
   "resolver: lts-2.10\n" ++
   "packages: ['.']\n"
 
+buildOptsConfig :: String
+buildOptsConfig =
+  "resolver: lts-2.10\n" ++
+  "packages: ['.']\n" ++
+  "build:\n" ++
+  "  library-profiling: true\n" ++
+  "  executable-profiling: true\n" ++
+  "  haddock: true\n" ++
+  "  haddock-deps: true\n" ++
+  "  copy-bins: true\n" ++
+  "  prefetch: true\n" ++
+  "  force-dirty: true\n" ++
+  "  keep-going: true\n" ++
+  "  test: true\n" ++
+  "  test-arguments:\n" ++
+  "    rerun-tests: true\n" ++
+  "    additional-args: ['-fprof']\n" ++
+  "    coverage: true\n" ++
+  "    no-run-tests: true\n" ++
+  "  bench: true\n" ++
+  "  benchmark-opts:\n" ++
+  "    benchmark-arguments: -O2\n" ++
+  "    no-run-benchmarks: true\n" ++
+  "  reconfigure: true\n" ++
+  "  cabal-verbose: true\n"
+
 stackDotYaml :: Path Rel File
 stackDotYaml = $(mkRelFile "stack.yaml")
 
@@ -43,6 +69,9 @@ setup = do
 teardown :: T -> IO ()
 teardown _ = return ()
 
+noException :: Selector SomeException
+noException = const False
+
 spec :: Spec
 spec = beforeAll setup $ afterAll teardown $ do
   let logLevel = LevelDebug
@@ -60,7 +89,6 @@ spec = beforeAll setup $ afterAll teardown $ do
         let resetVar = setEnv name originalValue
         bracket_ setVar resetVar action
 
-
   describe "loadConfig" $ do
     let loadConfig' m = runStackLoggingT m logLevel False False (loadConfig mempty Nothing Nothing)
     let loadBuildConfigRest m = runStackLoggingT m logLevel False False
@@ -74,6 +102,28 @@ spec = beforeAll setup $ afterAll teardown $ do
       -- TODO(danburton): more specific test for exception
       loadConfig' manager `shouldThrow` anyException
 
+    it "parses build config options" $ \T{..} -> inTempDir $ do
+      writeFile (toFilePath stackDotYaml) buildOptsConfig
+      BuildOpts{..} <- configBuild . lcConfig <$> loadConfig' manager
+      boptsLibProfile `shouldBe` True
+      boptsExeProfile `shouldBe` True
+      boptsHaddock `shouldBe` True
+      boptsHaddockDeps `shouldBe` (Just True)
+      boptsInstallExes `shouldBe` True
+      boptsPreFetch `shouldBe` True
+      boptsKeepGoing `shouldBe` (Just True)
+      boptsForceDirty `shouldBe` True
+      boptsTests `shouldBe` True
+      boptsTestOpts `shouldBe` (TestOpts {toRerunTests = True
+                                         ,toAdditionalArgs = ["-fprof"]
+                                         ,toCoverage = True
+                                         ,toDisableRun = True})
+      boptsBenchmarks `shouldBe` True
+      boptsBenchmarkOpts `shouldBe` (BenchmarkOpts {beoAdditionalArgs = Just "-O2"
+                                                   ,beoDisableRun = True})
+      boptsReconfigure `shouldBe` True
+      boptsCabalVerbose `shouldBe` True
+
     it "finds the config file in a parent directory" $ \T{..} -> inTempDir $ do
       writeFile (toFilePath stackDotYaml) sampleConfig
       parentDir <- getCurrentDirectory >>= parseAbsDir
@@ -86,7 +136,7 @@ spec = beforeAll setup $ afterAll teardown $ do
       bcRoot bc `shouldBe` parentDir
 
     it "respects the STACK_YAML env variable" $ \T{..} -> inTempDir $ do
-      withCanonicalizedSystemTempDirectory "config-is-here" $ \dir -> do
+      withSystemTempDir "config-is-here" $ \dir -> do
         let stackYamlFp = toFilePath (dir </> stackDotYaml)
         writeFile stackYamlFp sampleConfig
         withEnvVar "STACK_YAML" stackYamlFp $ do
