@@ -71,7 +71,8 @@ test :: FilePath -- ^ runghc
      -> String
      -> Spec
 test runghc env' currDir origStackRoot newHome name = it name $ withDir $ \dir -> do
-    removeDirectoryRecursive newHome
+    newHomeExists <- doesDirectoryExist newHome
+    when newHomeExists (removeDirectoryRecursive newHome)
     copyTree toCopyRoot origStackRoot (newHome </> takeFileName origStackRoot)
     let testDir = currDir </> "tests" </> name
         mainFile = testDir </> "Main.hs"
